/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.postit;

import com.tivoli.twg.libs.*;

/**
 * Command class for creating a reminder note association with a specific
 * set of mos that can be displayed in this task bean's association view.
 */
public class BCCreatePostItNoteCmd
       extends Command
{
  /**
   * Constructor for create post it note command
   * @param note_text - the text of the note association with the oids
   * @param oids - the list of one or more object ids to associate the note with
   */
  public BCCreatePostItNoteCmd( String note_text, LongValueSet oids )
  {
    super( BCPostItConstants.CREATE_POST_IT_NOTE_CMD );
    SetDestinationAddress( BCPostItConstants.POST_IT_SERVER );

    int   iBBLength = 0;
    IntelByteBuffer iBB = null;

    /* Parm 0 = active console id and list of object ids                */
    // Talley length of buffer.
    iBBLength = IntelByteBuffer.GetCompUnicodeLength(note_text) +
                IntelByteBuffer.GetLongValueSetLength(oids);

    // Allocate the Intel byte buffer.
    iBB = new IntelByteBuffer( iBBLength );

    // Write parameters to buffer and attach to command.
    iBB.WriteCompUnicode(note_text);
    iBB.WriteLongValueSet(oids);
    AddInputParm( iBB.GetBuffer() );
  }

  /**
   * Get ID assigned to the created post it note object
   * @return object ID, or 0 if failed
   */
  public long getPostItNoteID()
  {
    if ( OutputParmLength(0) >= IntelByteBuffer.SIZEOF_LONG64 )
      return IntelByteBuffer.ReadLong64( OutputParm(0), 0 );
    else
      return 0;
  }
}

