/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.postit;

import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;
import com.tivoli.twg.libs.IntelByteBuffer;

/**
 * Command class for retrieving the text for a specific reminder note for
 * any managed object displayed in this task bean's association view.
 */
public class BCGetPostItNoteTextCmd extends Command
{
    /**
     * Constructor for the retrieving the text for a post it note command.
     *
     * @param noteid the unique id of the note to get the text for
     */
    public BCGetPostItNoteTextCmd( long coid, CommandCompleteListener ccl )
    {
        super( BCPostItConstants.GET_POST_IT_NOTE_TEXT_CMD, ccl );
        SetDestinationAddress( BCPostItConstants.POST_IT_SERVER );
        SetTimeOut( 30000 );

        int   iBBLength = 0;
        IntelByteBuffer iBB = null;

        // Parm 0 = note id
        // Talley length of buffer.
        iBBLength = IntelByteBuffer.SIZEOF_LONG64;

        // Allocate the Intel byte buffer.
        iBB = new IntelByteBuffer( iBBLength );

        // Write parameter to buffer and attach to command.
        iBB.WriteLong64( coid );
        AddInputParm( iBB.GetBuffer() );
    }

    /**
     * Get the post it note text from the command.
     *
     * @return text for the note id specified.
     */
    public String getPostItNoteText()
    {
        if ( NumOutputParms() == 1 )
            return IntelByteBuffer.ReadCompUnicode( OutputParm(0), 0 );
        else
            return null;
    }
}

