/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.associations;

import com.tivoli.twg.engine.*;
import java.util.Vector;

/**
 * <P>SQL database table for association information
 *
 * <p>Primary key - MANAGED_OBJ_ID
 *
 *
 * <p>
 * <table width=450 border=5 cols=6>
 * <!- Table Name>
 * <tr><td colspan=6 align=center><strong>BOBCO_ASSOCIATIONS</strong></td>
 * <!- Table description >
 * <tr><td><strong>Column Name</strong></td>
 * <td><strong>Data Type</strong></td>
 * <td><strong>Description</strong></td>
 * <!- Table rows >
 * <tr><td>MANAGED_OBJ_ID</td>         <td>INT</td>       <td>Managed object id</td>
 * </table>
 */

public class BobCoAssociationsTable extends TWGDatabaseTable
{
   /* SQL Table Name */
   public static final String TABLE_NAME = "BOBCO_ASSOCIATIONS";

   /* Table Columns */
   public static final String MANAGED_OBJ_ID          = "MANAGED_OBJ_ID";
   
   /* table version */
   public static final String TABLE_VERSION                 = "1.0";


   public BobCoAssociationsTable()
   {
      super();
   }
   public void create() throws TWGDbException
   {

      //- IT Director Database name for the table
      String db = TWGDatabase.TWG_DB_NAME;
      String tableName = TABLE_NAME;
      String tableTokenName = TABLE_NAME;


      //----------------------------------------------------------------------
      //-Create the columns
      //----------------------------------------------------------------------
      TWGDbCharColumn charColumn;
      TWGDbIntegerColumn intColumn;
      // Object to hold the collection of columns
      TWGDbColumnData column_data = new TWGDbColumnData();

      // define the managed object id column specifying real and token column names
      intColumn = new TWGDbIntegerColumn(MANAGED_OBJ_ID,MANAGED_OBJ_ID);

      // managed object id column should not be a selectable column when creating
      // dynamic groups (filters)
      intColumn.setFilterColumn(false);

      // add the column to the column data
      column_data.addColumn(intColumn);

      TWGDbTableParms parms = new TWGDbTableParms();

      //---------------------------------------------------------------------
      // NLS bundle for the table
      //---------------------------------------------------------------------
      String tableBundle = "com.BobCo.associations.BobCoTableResources";


      //---------------------------------------------------------------------
      // Negotiate the SQL database name
      //---------------------------------------------------------------------
      int i = 1;
      String saveTableName      = tableName;
      String saveTableTokenName = tableTokenName;

      boolean tableNegotiationComplete = false;

      while (tableNegotiationComplete == false)
      {
         try
         {
            this.TWGInitTable(db, tableTokenName, tableName, column_data, parms,
                              tableBundle);

            tableNegotiationComplete = true;
         }
         catch (TWGDbRealTableNameNotUniqueException e)
         {
            tableName = saveTableName + String.valueOf(i);
            tableTokenName = saveTableTokenName + String.valueOf(i);
            i++;
         }
         catch (TWGDbTokenTableNameNotUniqueException e)
         {
            tableName = saveTableName + String.valueOf(i);
            tableTokenName = saveTableTokenName + String.valueOf(i);
            i++;
         }
      }
   }

   /*
   * Get the version number for the table
   *
   * @return the table version
   *
   */
   public String getTableVersion()
   {
      return TABLE_VERSION;
   }
}
