package com.BobCo.FileExplorerJVM;
import  com.tivoli.twg.engine.TWGTaskConstants;
import  com.tivoli.twg.engine.RemoteTaskActivationListener;
import  com.tivoli.twg.engine.TWGTaskCreationException;
import  com.tivoli.twg.engine.TWGRemoteTaskManager;
import  com.tivoli.twg.engine.TWGRemoteTaskActivation;
import  com.tivoli.twg.engine.TWGRemoteTaskClients;
import  com.tivoli.twg.engine.TWGRemoteManagedObject;
import  com.tivoli.twg.engine.RemoteMOLifeCycleListener;
import  com.tivoli.twg.engine.RemoteMOStateChangeListener;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.Command;
import  com.tivoli.twg.libs.CommandCompleteListener;
import  com.tivoli.twg.libs.CommandReceivedListener;
import  com.tivoli.twg.libs.IntelByteBuffer;
import  com.tivoli.twg.libs.LongKeyTable;
import  com.tivoli.twg.libs.LongValueSet;
import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeLocalImplFactory;
import  com.tivoli.twg.log.TWGRas;
import  java.io.IOException;
import  java.util.Enumeration;

/**
 * This class is the main class for the BobCo file explorer task server.  It
 * gets launched by the IT Director server as appropriate and receives task
 * activations via TWGRemoteTaskManager.
 */
public class BobCoFileExplorerJVMServer
       implements CommandReceivedListener,
                  RemoteTaskActivationListener,
                  BobCoFileExplorerJVMConstants {

   private static boolean keepGoing = true;
   private static ServiceNode sn;

   private BobCoFileExplorerJVMServer() {}

   /**
    * Command received callback.  This listener method is called when a
    * Command is received matching one of the command codes registered for
    * the given listener
    * @param sn - ServiceNode instance which received Command
    * @param cmd - Instance of Command received
    * @return true if ServiceNode should stay active, false if it should
    * terminate
    */
   public boolean CommandReceived(ServiceNode sn, Command cmd) {
      System.out.println("BobCoFileExplorerJVMServer.CommandReceived()"
                        +" to log\\com.BobCo.FileExplorerJVM.BobCoFileExplorerJVMServer.output\n"
                        +" cmd="+ cmd);

      long actID = IntelByteBuffer.ReadLong64(cmd.InputParm(0), 0);
      BobCoJVMTaskActivation act = BobCoJVMTaskActivation.findActivation(actID);

      if (act != null) {
         switch ((int)cmd.CommandCode()) {
            case consoleRequestLocalFixedDrives:
               new GetClientLocalFixedDrivesCommand(act, cmd);
               break;

            case consoleRequestDirectoryContents:
               new GetClientDirectoryContentsCommand(act, cmd);
               break;

            default:
               System.out.println("Unexpected command code received:  "+cmd.CommandCode());
               cmd.SetReturnCode(RETURN_UNRECOGNIZED_COMMAND_CODE);
               break;
         }
      } else {
         cmd.SetReturnCode(RETURN_UNRECOGNIZED_ACT_ID);
      }

      return true;
   }

   /**
    * This method is called when a subtask activation has occurred.
    * @param act the TWGRemoteTaskActivation object containing the task activation
    *            information.
    */
   public void taskActivated(TWGRemoteTaskActivation act) {
      System.out.println("BobCoFileExplorerJVMServer.taskActivated()"
                        +" to log\\com.BobCo.FileExplorerJVM.BobCoFileExplorerJVMServer.output");

      String subtaskID = act.getSubtaskID();
      if (subtaskID.equals("ExploreFilesJVM")) {
         System.out.println("BobCoFileExplorerJVMServer.taskActivated() ExploreFilesJVM activation received:");
         TWGRemoteTaskClients clients = act.getClients();
         if ((clients == null) ||
             (clients.size() != 1)) {
            System.out.println("Error: activation must be targeted for exactly one client");
            act.deactivate(TWGTaskConstants.ACT_STATUS_COMPLETE_WITH_ERRORS);
         } else {
            BobCoJVMTaskActivation myAct = new BobCoJVMTaskActivation(act);
         }
      } else {
         System.out.println("Unrecognized subtask ID.");
         act.deactivate(TWGTaskConstants.ACT_STATUS_COMPLETE_WITH_ERRORS);
      }
   }

   /**
    * This method is called when a subtask deactivation has occurred.
    * @param act the TWGRemoteTaskActivation object containing the task activation
    *            information.  This is the same object that was passed to the
    *            taskActivated() method.
    */
   public void taskDeactivated(TWGRemoteTaskActivation act) {
      long actID = act.getEngActID();
      System.out.println("BobCoFileExplorerJVMServer.taskDeactivated() Subtask deactivation received:");
      System.out.println("toid = "+act.getToid()+", engActID = "+act.getEngActID()+", rmtActID = "+act.getRmtActID());
      System.out.println("taskID = "+act.getTaskID()+", subtaskID = "+act.getSubtaskID());
      System.out.println("Activation count:  total = "+TWGRemoteTaskActivation.getTotal()+", current = "+TWGRemoteTaskManager.getCurrentActivationCount());
      BobCoJVMTaskActivation.deleteActivation(actID);
   }

   /**
    * The task server has been deactivated and any clean up required should be
    * done during this method.  After this method is called, the task server's
    * JVM will terminate.
    */
   public void serverDeactivated() {
      System.out.println("Server deactivation received:");
      keepGoing = false;
      System.out.println("   keepGoing = "+keepGoing);
      synchronized (this) {
         notifyAll();
      }
      System.out.println("   object notified, main loop should exit now");
   }

   /**
    * Since this task server gets launched in its own JVM by IT Director's
    * server, it must have a main() method.
    */
   public static void main(String argv[]) {
      System.out.println("BobCoFileExplorerJVMServer.main()"
                        +" to log\\com.BobCo.FileExplorerJVM.BobCoFileExplorerJVMServer.output\n"
                        +" argv.length="+ argv.length);
	  for (int i=0; i<argv.length; i++) {
	    System.out.println("  argv["+ i +"]="+ argv[i] +"/n");
	  }

      TWGRas.initialize();
      Exception except = null;
      BobCoFileExplorerJVMServer me = new BobCoFileExplorerJVMServer();

      // Bring up the service node
      // Set ServiceNode support to use local IPC implementation
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);
      try {
          sn = new ServiceNode(serviceNodeName);
          TWGRemoteTaskManager.setServiceNode(sn, RTMcommandCode, me);
          BobCoJVMTaskActivation.setServiceNode(sn);
          sn.RegisterDefaultCommandListener(me); // Catch any commands not handled
                                                 // by TWGRemoteTaskManager
          keepGoing = sn.Create();
          if (keepGoing) {
             System.out.println("BobCoFileExplorerJVMServer.main() Local service node created, name = "+sn.Name());
          } else {
             System.out.println("BobCoFileExplorerJVMServer.main() Local service node create failed.");
          }
      } catch(BadServiceNodeImplException snix) {
         System.out.println("BobCoFileExplorerJVMServer.main() caught BadServiceNodeImplException "+ snix);
         except = snix;
      } catch(ServiceNodeClosedException sncx) {
         System.out.println("BobCoFileExplorerJVMServer.main() caught ServiceNodeClosedException "+ sncx);
         except = sncx;
      } catch(ServiceNodeException snx) {
         System.out.println("BobCoFileExplorerJVMServer.main() caught ServiceNodeException "+ snx);
         except = snx;
      }
      if (except != null) {
         System.out.println("BobCoFileExplorerJVMServer.main() Exception encountered while creating service node");
         except.printStackTrace();
      }

      System.out.println("BobCoFileExplorerJVMServer.main() ready for subcommands...");
      TWGRemoteTaskManager.taskServerReady("BobCo|FileExplorerJVM");

      System.out.println("BobCoFileExplorerJVMServer.main() thread waiting...");
      while (keepGoing) {
         synchronized (me) {
            try {
               // This is where the task's main thread would go to work
               me.wait();
            } catch (InterruptedException ix) {
               System.out.println("BobCoFileExplorerJVMServer.main() InterruptedException encountered during wait.");
            }
         }
      }
      System.out.println("BobCoFileExplorerJVMServer.main() thread wait finished");

      sn.DeregisterDefaultCommandListener();
      TWGRemoteTaskManager.unsetServiceNode();
      try{
         sn.Close();
      }
      catch(ServiceNodeException err){
         System.out.println("BobCoFileExplorerJVMServer.main() Exception encountered while closing service node");
         err.printStackTrace();
      }
      sn = null;
      System.out.println("BobCoFileExplorerJVMServer.main() exiting.");
   }
}


