package com.BobCo.FileExplorer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import com.tivoli.twg.engine.TWGConManagedObject;
import com.tivoli.uif.core.UFImageLoader;
import java.awt.Component;

/**
 * This class is a simple extension of JTree that sets up the basic
 * display properties of the file explorer tree.  It also contains
 * an inner class that provides the tree cell renderer.
 */
public class BobCoFileExplorerTree extends JTree {
   // images for tree
   private static ImageIcon driveImage      = new ImageIcon(UFImageLoader.getImage("/com/BobCo/FileExplorer/images/drive.gif"));
   private static ImageIcon folderImage     = new ImageIcon(UFImageLoader.getImage("/com/BobCo/FileExplorer/images/folder.gif"));
   private static ImageIcon openFolderImage = new ImageIcon(UFImageLoader.getImage("/com/BobCo/FileExplorer/images/openfolder.gif"));
   private static ImageIcon fileImage       = new ImageIcon(UFImageLoader.getImage("/com/BobCo/FileExplorer/images/document.gif"));

   /**
    * Constructor
    * @param gui   the GUI creating this tree
    * @param moid  the ID of the managed object to explore
    */
   BobCoFileExplorerTree(BobCoFileExplorerGUI gui, long moid) {
      super(new BobCoFileExplorerTreeNode(gui, moid), true);
      setShowsRootHandles(true);
      setLargeModel(true);

      setRowHeight(18);

      // Set up the tree variables
      ((DefaultTreeModel)getModel()).setAsksAllowsChildren(true);
      setCellRenderer(new BobCoTreeCellRenderer ());
   }

   /**
    * Inner class which is the tree cell renderer for the
    * BobCoFileExplorerTreeNode objects.
    */
   class BobCoTreeCellRenderer extends DefaultTreeCellRenderer {

      /**
       * The constructor.  Sets up some of the look and feel.
       */
      public BobCoTreeCellRenderer () {
         super();
         setHorizontalTextPosition(SwingUtilities.RIGHT);
      }

      /**
       * Returns the appropriate JLabel for a node in this tree.
       */
      public Component getTreeCellRendererComponent(JTree theTree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
         super.getTreeCellRendererComponent(theTree,value,selected, expanded, leaf, row, hasFocus);

         switch (((BobCoFileExplorerTreeNode)value).getType()) {
            case BobCoFileExplorerTreeNode.ROOT_MO:
               setIcon(((BobCoFileExplorerTreeNode)value).getConManagedObject().getCurrentImage(TWGConManagedObject.MO_IMAGE_DETAILVIEW));
               break;
            case BobCoFileExplorerTreeNode.DRIVE:
               setIcon(driveImage);
               break;
            case BobCoFileExplorerTreeNode.DIRECTORY:
               if (expanded) {
                  setIcon(openFolderImage);
               } else {
                  setIcon(folderImage);
               }
               break;
            case BobCoFileExplorerTreeNode.FILE:
               setIcon(fileImage);
               break;
         }

         if (((BobCoFileExplorerTreeNode)value).getType()==BobCoFileExplorerTreeNode.FILE) {
            setIcon(fileImage);
         } else if (((BobCoFileExplorerTreeNode)value).getType()==BobCoFileExplorerTreeNode.DIRECTORY) {
            if (expanded) {
               setIcon(openFolderImage);
            } else {
               setIcon(folderImage);
            }
         } else if (((BobCoFileExplorerTreeNode)value).getType()==BobCoFileExplorerTreeNode.DRIVE) {
            setIcon(driveImage);
         }

         setText(((BobCoFileExplorerTreeNode)value).getName());

         setSize(getSize().width,18);

         return (this);
      }
   }
}
