package com.BobCo.FileExplorer;
import  com.tivoli.twg.engine.AddressEntryFormatException;
import  com.tivoli.twg.engine.TWGNativeAddressEntryShadow;
import  com.tivoli.twg.engine.TWGTaskActivation;
import  com.tivoli.twg.engine.TWGRemoteTaskClients;
import  com.tivoli.twg.engine.TWGLocalManagedObject;
import  com.tivoli.twg.libs.Command;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.CommandReceivedListener;
import  com.tivoli.twg.libs.IntelByteBuffer;
import  com.tivoli.twg.libs.LongKeyTable;
import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeRemoteImplFactory;
import  java.util.Enumeration;

/**
 * This utility class is used to maintain the additional information
 * needed for task activation processing beyond what is provided in
 * TWGTaskActivation.
 */
class BobCoTaskActivation implements BobCoFileExplorerConstants {
   static ServiceNode sn = null;
   private static LongKeyTable activations = new LongKeyTable();
   private static TWGNativeAddressEntryShadow nativeAddrShadow = new TWGNativeAddressEntryShadow();
   TWGTaskActivation act;
   TWGLocalManagedObject lmo;
   String consoleAddress = null;

   /**
    * Constructor
    * @param act  the TWGTaskActivation object for this activation.
    */
   BobCoTaskActivation(TWGTaskActivation act) {
      System.out.println("BobCoTaskActivation() contructor"
                        +" to log\\com.tivoli.twg.engine.TWGServerMain.output");
      this.act = act;
      // Even though the task is set up to be drag-drop and only accepting
      // a single target, we must use the array returned by getClients()
      // to extract the TWGLocalManagedObject that is the target.
      Object[] lmos = act.getClients().getAllClients();
	  lmo = (TWGLocalManagedObject)lmos[0];
      System.out.println("BobCoTaskActivation() contructor\n act.getConsoleActID()="+ act.getConsoleActID()
	                                                    +" act.getServerActID()="+ act.getServerActID()
	                                                    +" act.getMoid().GetValue(0)="+ act.getMoid().GetValue(0));
      activations.InsertObject(act.getMoid().GetValue(0), this);
   }

   /**
    * Find a BobCoTaskActivation based on the activation ID
    * @param actID   the activation ID to find
    */
   public static final BobCoTaskActivation findActivation(long actID) {
      System.out.println("BobCoTaskActivation.findActivation() actID="+ actID);
      return (BobCoTaskActivation)activations.FindObject(actID);
   }

   /**
    * Delete a BobCoTaskActivation from the collection of active activations.
    * @param actID   the activation ID to delete
    */
   public static final void deleteActivation(long actID) {
      System.out.println("BobCoTaskActivation.deleteActivation()");
      BobCoTaskActivation myAct = (BobCoTaskActivation)activations.RemoveObject(actID);
      if (myAct != null) {
         myAct.act = null;
      }
   }

   /**
    * Set the service node to use to communicate with the client.
    * @param SN a ServiceNode
    */
   public static void setServiceNode(ServiceNode SN) {
      System.out.println("BobCoTaskActivation.setServiceNode()"
                        +" to log\\com.tivoli.twg.engine.TWGServerMain.output");
      sn = SN;
   }

   /**
    * Get the activation ID of this object.
    * @return long the activation ID
    */
   public final long getActID() {
      System.out.println("BobCoTaskActivation.getActID()");
      return act.getServerActID();
   }

   /**
    * A utility method to do a SendAsynch to the client in this activation.
    * @param  cmd     the Command to send to the client
    * @return boolean true if no errors were encountered during the SendAsynch
    */
   public final boolean sendAsynchToClient(Command cmd) {
      System.out.println("BobCoTaskActivation.sendAsynchToClient()");
      boolean rc = true;
      if (sn != null) {
         byte[] addrEntry = lmo.getAddressEntryRecord();
         if (addrEntry != null) {
            try {
               nativeAddrShadow.initAddressEntry(addrEntry, 0, addrEntry.length);
               String ipcPath = nativeAddrShadow.getIPCPath();
               System.out.println("Sending command, ipcPath="+ipcPath);
               cmd.SetDestinationAddress(ipcPath+clientAddressSuffix);
               try {
                  sn.SendAsynchCommand(cmd);
               } catch (ServiceNodeClosedException sncx) {
                  rc = false;
                  System.out.println("ServiceNodeClosedException while sending command.");
               } catch (ServiceNodeException snx) {
                  rc = false;
                  System.out.println("ServiceNodeException while sending command.");
               } catch (InterruptedException ix) {
                  rc = false;
                  System.out.println("InterruptedException while sending command.");
               }
            } catch (AddressEntryFormatException aefx) {
               System.out.println("Caught AddressEntryFormatException");
            }
         } else {
            System.out.println("Client's address entry is null.");
            rc = false;
         }
      } else {
         System.out.println("Service node has not been initialized.");
         rc = false;
      }
      return rc;
   }

   /**
    * A utility function to send a postponed reply back to the console.
    * @param  cmd     the Command to send the postponed reply for
    * @return boolean true if no errors were encountered during
    *                 SendPostponedReply()
    */
   public final boolean sendPostponedReply(Command cmd) {
      System.out.println("BobCoTaskActivation.sendPostponedReply()");
      boolean rc = true;
      try {
         rc = sn.SendPostponedReply(cmd);
      } catch (ServiceNodeClosedException sncx) {
         rc = false;
         System.out.println("ServiceNodeClosedException while sending postponed reply to console.");
      } catch (ServiceNodeException snx) {
         rc = false;
         System.out.println("ServiceNodeException while sending postponed reply to console.");
      }
      return rc;
   }
}
