package com.ibm.sysmgt.sdk.CmdTask;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.applet.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import com.tivoli.uif.core.*;
import com.tivoli.uif.controls.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.console.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.procman.*;
import com.tivoli.twg.netipc.*;
import com.tivoli.twg.log.TWGRas;


/**
 * This class is extends TWGTaskFrame to provide the GUI for the CmdTask.
 * It gets instantiated and launched as appropriate by Director's
 * console.
 */
public class CmdTaskGUI
       extends TWGTaskFrame
       implements ActionListener {

     
  private static String agentCommandOutput = null;
  
  private static boolean active_run = false;

  // service node for Agent Commands
  private static ServiceNode sn;
  
  // service node for Console/URL Commands
  // added for compatibility with Director 3.1
  private ServiceNode snSync;

  public  static final String SERVICE_NODE_NAME = "CmdTaskAgent";
  private static final String execDestination = "ProcMgr";
  
  private static String cmdTaskName = null;
  private String msg = null;

   /**
    * TWGTaskFrame - This is the constructor for the tasks frame.
    */
   public CmdTaskGUI() {
      super( true );
      
      TWGRas.debug(TWGRas.EX0,"CmdTaskGUI constructor");
      
      // create a Service Node to handle Console/URL Commands
      // added for compatibility with Director 3.1
     if (snSync == null) try {
	 snSync = new ServiceNode();
     } catch (BadServiceNodeImplException bsnie) {
	 msg += "CmdTask.CmdTaskGUI.pInit BadServiceNodeImplException :" + bsnie.getMessage() + "\n";
	 TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI.pInit BadServiceNodeImplException :" + bsnie.getMessage());
     } catch (ServiceNodeClosedException snce) {
	msg += "Service Node: " + SERVICE_NODE_NAME + "  closed :" + snce.getMessage() + "\n";
	TWGRas.debug(TWGRas.EX0,"Service Node: " + SERVICE_NODE_NAME + "  closed");
	TWGRas.debug(TWGRas.EX0,snce.getMessage());
     } catch (ServiceNodeException sne) {
	msg += "Service Node: " + SERVICE_NODE_NAME + " exception :" + sne.getMessage() + "\n";
	TWGRas.debug(TWGRas.EX0,"Service Node: " + SERVICE_NODE_NAME + " exception :");
	TWGRas.debug(TWGRas.EX0,sne.getMessage());
     }
   }

   /**
    * Initialization routine for setting up non-gui items.
    */
   public boolean pInit( ) {
      TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.pInit()");
      boolean rc = false;
      msg = new String("");
      String cmdLine = null;
      String agentCmd = null;
      String agentCwd = null;
      String urlType = null;
      String urlPort = null;
      String urlFile = null;

      String taskID = this.getTaskActivator().getTask().getID();
      cmdTaskName = this.getTaskActivator().getTask().getName();
      
      GetCmdStringCommand gcmd;
      int platform;
      if (TWGEnvironment.isWindows()) {
          platform = GetCmdStringCommand.Windows;

      } else {
          platform = GetCmdStringCommand.Unix;
      }

      gcmd = new GetCmdStringCommand(  taskID, platform );

      // &&& send a command to get the command line from the server
      try {
	  TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI sending GetCmdStringCommand for taskID " + taskID);
	  //added for compatibility with Director 3.1
	  snSync.SendCommand (gcmd);
          //getTaskFrameServiceNode().SendCommand (gcmd);
      }
      catch (ServiceNodeClosedException e) {
	  msg += "ServiceNodeClosedException " + e + "\n";
          TWGRas.debug(TWGRas.EX0,"ServiceNodeClosedException " + e);
      }
      catch (ServiceNodeException e) {
	  msg += "ServiceNodeException " + e + "\n"; 
          TWGRas.debug(TWGRas.EX0,"ServiceNodeException " + e);
      }
      catch (InterruptedException e) {
	  msg += "InterruptedException " + e + "\n";
          TWGRas.debug(TWGRas.EX0,"InterruptedException " + e);
      }

      CmdLauncher cl = new CmdLauncher();
      
      if (TWGEnvironment.isWindows ()) {
	  cmdLine = gcmd.getCommandString();
	  if(cmdLine != null)
	      cl.setWindowsCommand( cmdLine );
      } else {
	  cmdLine = gcmd.getCommandString();
	  if(cmdLine != null)
	      cl.setUnixCommand( cmdLine );
      }

      cl.setShell ( gcmd.getShellRequired() );
      cl.setCwd ( gcmd.getCwd() );

      urlType = gcmd.getUrlType();
      urlPort = gcmd.getUrlPort();
      urlFile = gcmd.getUrlFile();

      cl.setUrl( urlType, urlPort, urlFile );

      TWGTaskActivator ta = getTaskActivator();
      LongValueSet moid = ta.getMoid();
      long [] oids = moid.GetValues();

      //TWGConObject tco = TWGConObject.FindObject(oids[i]);
      //tco.
      TWGGetAttributeCommand twggac = new TWGGetAttributeCommand(moid,
					  new String[]{TWGManagedObjectConstants.ATTRIB_IP_ADDRS,
						       TWGManagedObjectConstants.ATTRIB_MAC_ADDRESS,
						       TWGManagedObjectConstants.ATTRIB_COMPUTER_NAME,
						       TWGManagedObjectConstants.ATTRIB_OS_TYPE,
						       TWGManagedObjectConstants.ATTRIB_IS_ACCESS_DENIED,
	  					       TWGManagedObjectConstants.ATTRIB_NAME
					      });
      try {
	  // added for compatibility with Director 3.1
	  snSync.SendCommand (twggac);
	  //getTaskFrameServiceNode().SendCommand (twggac);
      }
      catch (ServiceNodeClosedException e) {
	  msg += "ServiceNodeClosedException " + e + "\n";
          TWGRas.debug(TWGRas.EX0,"ServiceNodeClosedException " + e);
      }
      catch (ServiceNodeException e) {
	  msg += "ServiceNodeException " + e + "\n"; 
          TWGRas.debug(TWGRas.EX0,"ServiceNodeException " + e);
      }
      catch (InterruptedException e) {
	  msg += "InterruptedException " + e + "\n";
          TWGRas.debug(TWGRas.EX0,"InterruptedException " + e);
      }


      // Loop though each selected Managed Object
      for(int i = 0; i < oids.length; i++){
	  DataValue dv;
    
	  // Get information for this i'th Managed Object ( oids[i] )
	  dv = twggac.getAttributeValue(oids[i],TWGManagedObjectConstants.ATTRIB_OS_TYPE);
	  String osType = (dv instanceof TWGString ? ((TWGString) dv).toString() : new String("Get OS type error"));

	  dv = twggac.getAttributeValue(oids[i],TWGManagedObjectConstants.ATTRIB_IP_ADDRS);
	  TWGStringArray ipAddresses = (dv instanceof TWGStringArray ? (TWGStringArray) dv : new TWGStringArray());

	  dv = twggac.getAttributeValue(oids[i],TWGManagedObjectConstants.ATTRIB_MAC_ADDRESS);
	  TWGByteArray macAddresses = (dv instanceof TWGByteArray ? (TWGByteArray) dv : new TWGByteArray());

	  dv = twggac.getAttributeValue(oids[i],TWGManagedObjectConstants.ATTRIB_COMPUTER_NAME);
	  String computerName = (dv instanceof TWGString ? ((TWGString) dv).toString() : new String(""));

	  if(computerName==null || computerName.equals("")){
	      dv = twggac.getAttributeValue(oids[i],TWGManagedObjectConstants.ATTRIB_NAME);
	      computerName = (dv instanceof TWGString ? ((TWGString) dv).toString() : new String(""));
	  }

	  dv = twggac.getAttributeValue(oids[i],TWGManagedObjectConstants.ATTRIB_IS_ACCESS_DENIED);
	  boolean accessible = !(dv instanceof TWGBoolean ? ((TWGBoolean) dv).getValue() : true);

	  if(accessible){
	      if (osType.startsWith("WINDOWS")) {
		  agentCmd = gcmd.getAgentCommandStringWindows();
		  agentCwd = gcmd.getAgentCwdWindows();
	      } else {
		  agentCmd = gcmd.getAgentCommandStringUnix();
		  agentCwd = gcmd.getAgentCwdUnix();
	      }
    
	      if (agentCmd != null) {
		 TWGRas.debug(TWGRas.EX0,"CmdTask Trying to execute \"" + agentCmd + "\" on AGENT " + agentCwd);
		  
    
		 // &&& send a command to execute agent command to ProcessManager subsystem
                 
		 boolean result = false;
                 
		 // create a Service Node to handle Agent Command
		 if (sn == null) try {
		     sn = new ServiceNode(SERVICE_NODE_NAME);
		 } catch (BadServiceNodeImplException bsnie) {
		     msg += "CmdTask.CmdTaskGUI.pInit BadServiceNodeImplException :" + bsnie.getMessage() + "\n";
		     TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI.pInit BadServiceNodeImplException :" + bsnie.getMessage());
		 } catch (ServiceNodeClosedException snce) {
		    msg += "Service Node: " + SERVICE_NODE_NAME + "  closed :" + snce.getMessage() + "\n";
		    TWGRas.debug(TWGRas.EX0,"Service Node: " + SERVICE_NODE_NAME + "  closed");
		    TWGRas.debug(TWGRas.EX0,snce.getMessage());
		 } catch (ServiceNodeException sne) {
		    msg += "Service Node: " + SERVICE_NODE_NAME + " exception :" + sne.getMessage() + "\n";
		    TWGRas.debug(TWGRas.EX0,"Service Node: " + SERVICE_NODE_NAME + " exception :");
		    TWGRas.debug(TWGRas.EX0,sne.getMessage());
		 }
                 
		 try {
		    
		    if (sn != null) {
			
			// Create a command to execute program
			SubExecuteCommand execCmd = new SubExecuteCommand();
			execCmd.setDestinationFromMoid(oids[i], sn);
			execCmd.setCommandLine( agentCmd );
			if(agentCwd != null) 
			    execCmd.setCWD( agentCwd );
			execCmd.setAsynch(false);
			execCmd.setCapture(true);
			execCmd.SetTimeOut(gcmd.getAgentTimeout());
			
			execCmd.SetDestinationAddress(execDestination);
			
			if (execCmd.buildCommandLine()) {

			  // Build command to request starting of process manager
			  StartProgramCommand startService = new StartProgramCommand(execCmd);
			  startService.SetDestinationAddress(TWGSvcMgrServiceNodeConstants.SERVICE_NODE);
			  IntelByteBuffer buf = new IntelByteBuffer(IntelByteBuffer.GetASCIIZLength(execDestination));
			  buf.WriteASCIIZ(execDestination);
			  startService.AddInputParm(buf.GetBuffer());
			
			  try {
			     if(active_run == false) {
				 active_run = true;
				 sn.SendAsynchCommand(startService);
			     } else {       /* Else, fake successful start                      */
				 startService.SetReturnCode(0);
				 startService.CommandComplete();
			     }
			     result = true;
			  }
			  catch (InterruptedException ie ) {
			     msg += "Connection failed while sending command to process manager : " + ie.getMessage() + "\n";
			     TWGRas.debug(TWGRas.EX0,"Connection failed while sending command to process manager");
			     TWGRas.debug(TWGRas.EX0,ie.getMessage());
			  }
			}
		       else {
			  msg += "Build command line failed; Execution terminated\n";
			  TWGRas.debug(TWGRas.EX0,"Build command line failed; Execution terminated");
		       }
		    } else {
			msg += "CmdTask AgentServiceNode is null!!!\n";
			TWGRas.debug(TWGRas.EX0,"CmdTask AgentServiceNode is null!!!");
		    }
		 } catch (ServiceNodeClosedException snce) {
		    msg += "Service Node: " + SERVICE_NODE_NAME + "  closed :" + snce.getMessage() + "\n";
		    TWGRas.debug(TWGRas.EX0,"Service Node: " + SERVICE_NODE_NAME + "  closed");
		    TWGRas.debug(TWGRas.EX0,snce.getMessage());
		 } catch (ServiceNodeException sne) {
		    msg += "Service Node: " + SERVICE_NODE_NAME + " exception :" + sne.getMessage() + "\n";
		    TWGRas.debug(TWGRas.EX0,"Service Node: " + SERVICE_NODE_NAME + " exception :");
		    TWGRas.debug(TWGRas.EX0,sne.getMessage());
		 }
    
		 //------------------------------------    
    
							 
	      } // of AgentCmd
    
	      
	      // Run the Console Command and URL BrowserLauncher Command
	      // Build Environment information for this i'th Managed Object ( oids[i] )
	      Hashtable env = new Hashtable();
	      env.put("CMDTASK_COMPUTERNAME", computerName);
	      for(int j = 0; j < ipAddresses.getLength(); j++){
		  env.put(new String("CMDTASK_IP_ADDRESS" + j) ,new String(ipAddresses.getValueAt(j)));
	      }
	      
	      String ma = new String();
	      int j = 0;
	      for(ma = ""; j < macAddresses.getLength();j++){
		  // add next byte on
		  ma += Byte.toString(macAddresses.getValueAt(j)) + " ";
		  // if this was the sixth byte, add the mac address and clear the old ma
		  if(j > 0 && (j+1)%6 == 0){
		      env.put(new String("CMDTASK_MAC_ADDRESS" + j/6),  ma);
		      ma = new String("");
		  }
	      }
    
	      TWGRas.debug(TWGRas.EX0,"CmdTask::CmdTaskGUI found    COMPUTERNAME  =  " + computerName);
	      TWGRas.debug(TWGRas.EX0,"CmdTask::CmdTaskGUI found       IPADDRESS  =  " + ipAddresses.toString());
	      TWGRas.debug(TWGRas.EX0,"CmdTask::CmdTaskGUI found      MACADDRESS  =  " + macAddresses.toString());
	      
	      cl.setEnvironment(env);
    
	      // &&& run the command
	      cl.launch ();
    
	      if(cmdLine==null && agentCmd==null && urlType==null && urlPort==null && urlFile==null){
		  msg += "No \"" + cmdTaskName + "\" CmdTask operations defined for " + computerName + ". The target's Operating System may not be supported by this CmdTask.";
	      }
	 } else {
             msg += computerName + " is not Accessible. It may be locked. Skipping CmdTask operations.";
	     TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI.pInit : " + msg);
	 }
      }
      //this.update();
      TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI.pInit returning:" + !(msg==null || msg.equals("")));
      return !(msg==null || msg.equals("")); // leave console up?

   } // end pInit

   /**
    * Routine to allow task frames to clean up.
    */
   public void pCleanUp( ) {
	  TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.pCleanUp()");
   }

   /**
    * Notification method to allow the task frame to know when the
    * server portion of a task has died so that it can take any
    * appropriate action.
    */
   public void serverTaskEnded() {
	  TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.serverTaskEnded()");
   }

   /**
    * Routine to allow task frames to prompt the user if they
    * should close or not.  Return true to close the frame, return
    * false to stop the frame from closing.
    */
   public boolean verifyOnClose( int cancelType ) {
      
      return true;
      
      /*
      TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.verifyOnClose()");
      int confirm = JOptionPane.showConfirmDialog( this,
                                                   "Do you really wish to quit?", "Verify on close",
                                                   JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE );

      
      if ( confirm == JOptionPane.YES_OPTION )
         return true;

      return false;
      */
   }


   /**
    * Create the client pane and return it.
    */
   public Container buildView( ) {
      TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.buildView()");
      // Set the frame's icon to the document
      setIconImage(UFImageLoader.getImage("/com/ibm/sysmgt/sdk/CmdTask/images/document.gif"));
      // Construct the top level client area pane that all panes are
      // placed into.  This code is provided as a temp for frames.
      JPanel returnPanel = new JPanel( );
      returnPanel.setBorder( new BevelBorder( BevelBorder.LOWERED ));
      returnPanel.setLayout(new BorderLayout());
      
      // Load and set the frame's title from the task resources.
      TWGTaskActivator ta = getTaskActivator();
      setTitle(ta.getTask().getName()+ta.getTargetTitle());
      LongValueSet moidset = ta.getMoid();
	  TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.buildView() moidset.Length()="+ moidset.Length());
	  for (int i=0; i<moidset.Length(); i++ ) {
	      TWGRas.debug(TWGRas.EX0," moidset.GetValue("+ i +")="+ moidset.GetValue(i));
      }
      
      TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI.buildView msg=" + msg);
      
      // Create a horizontal panel for the message (msg).
      JPanel msgPane = new JPanel();
      msgPane.setLayout(new BoxLayout(msgPane, BoxLayout.X_AXIS));
      
      // Create text area for msg and add to Panel.
      JTextArea msgTA = new JTextArea( msg );
      msgTA.setText( msg );
      msgTA.setLineWrap(true);
      msgTA.setWrapStyleWord(true);
      msgPane.add(new JScrollPane( (msgTA)));
      returnPanel.add( msgPane, BorderLayout.CENTER );
      
      // Create a horizontal panel for the buttons.
      JPanel buttonPane = new JPanel();
      buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));

      // Put glue in first so that all buttons are justified left.
      buttonPane.add(Box.createHorizontalGlue());

      // Create ok button and add to button panel.
      JButton okBtn = new JButton( "Ok" );
      buttonPane.add( okBtn );
      okBtn.addActionListener( this );

      returnPanel.add( buttonPane, BorderLayout.SOUTH );

      return( returnPanel );
   } // end buildView

   /**
    * Initial frame size to use the first time the frame is
    * created.  Subsequent invocations of the same console
    * will use the cached value (which may be different if
    * the user resized the window.
    */
   public Dimension getFrameSize() {
      return new Dimension( 300, 200 );
   }

   /**
    * Action handler for any menu or button actions executed.
    */
   public void actionPerformed( ActionEvent evt ) {
      // Request this window to shutdown for either ok or cancel.
      consoleCancel();

   } // end actionPerformed

   /**
    * A utility method to send a command to the IT Director server.  The
    * command should already be completely initialized.
    * @param  cmd     the Command to send to the server.
    * @return boolean true if no errors were encountered during the SendCommand()
    */
   public boolean sendToServer(Command cmd) {
	  TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.sendToServer()");
      boolean rc = true;
      // send the command
      try {
         sn.SendCommand(cmd);
      } catch (BadServiceNodeImplException snix) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  BadServiceNodeImplException while sending command to server");
         rc = false;
      } catch (ServiceNodeClosedException sncx) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  ServiceNodeClosedException while sending command to server");
         rc = false;
      } catch (ServiceNodeException snx) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  ServiceNodeException while sending command to server");
         rc = false;
      } catch (InterruptedException ix) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  InterruptedException while sending command to server");
         rc = false;
      }
      return rc;
   }

   /**
    * A utility method to asynchronously send a command to the IT Director
    * server.  The command should already be completely initialized.
    * @param  cmd     the Command to send to the server.
    * @return boolean true if no errors were encountered during the
    *                 SendAsynchCommand()
    */
   public boolean sendAsynchToServer(Command cmd) {
      TWGRas.debug(TWGRas.EX0,"CmdTaskGUI.sendAsynchToServer()");
      boolean rc = true;
      // send the command
      try {
         sn.SendAsynchCommand(cmd);
      } catch (BadServiceNodeImplException snix) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  BadServiceNodeImplException while sending command to server");
         rc = false;
      } catch (ServiceNodeClosedException sncx) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  ServiceNodeClosedException while sending command to server");
         rc = false;
      } catch (ServiceNodeException snx) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  ServiceNodeException while sending command to server");
         rc = false;
      } catch (InterruptedException ix) {
         TWGRas.debug(TWGRas.EX0,"BobCo GUI Error:  InterruptedException while sending command to server");
         rc = false;
      }
      return rc;
   }



private static class StartProgramCommand extends Command {
    // Starts Process Manager to handle Agent Commands
    // also takes agent exec_command and executes it via
    // the Process Manager.

      SubExecuteCommand exec_cmd;
      
      StartProgramCommand(SubExecuteCommand exec) {
          super(TWGSvcMgrServiceNodeConstants.SVCMGR_START_SVC);
          exec_cmd = exec;
      }
      public void CommandComplete() {
          String msg = new String("");
          if(ReturnCode() == 0) {
              try {
		  sn.SendAsynchCommand(exec_cmd);
              } catch (InterruptedException ie ) {
                  msg += "Interrupted Exeception : Connection failed while sending command to process manager X'" + Long.toHexString(ReturnCode()) + "'" ;
                  TWGRas.debug(TWGRas.EX0,msg);
                  TWGRas.debug(TWGRas.EX0,ie.getMessage());
              } catch (ServiceNodeException sne) {
		  msg += "Service Node Exception : Connection failed while sending command to process manager X'" + Long.toHexString(ReturnCode()) + "'" ;
		  TWGRas.debug(TWGRas.EX0,msg);
		  TWGRas.debug(TWGRas.EX0,sne.getMessage());

              }
          } else {
              msg += "Process manager could not be started X'" + Long.toHexString(ReturnCode()) + "'";
              TWGRas.debug(TWGRas.EX0,msg);

              active_run = false;
          }

	  // Popup message dialog if any problem occurs
	  if(msg != null && !msg.equals("")) {
	    final JDialog dialog = new JDialog();
	    dialog.getRootPane().setUI((new CmdTaskGUI()).getRootPane().getUI());

	    dialog.setTitle("CmdTask.StartProgramCommand error");
	    JTextArea jta = new JTextArea(msg);
	    
            dialog.getContentPane().add(jta);

	    dialog.setDefaultCloseOperation(
		JDialog.DISPOSE_ON_CLOSE);
	    dialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent we) {
		    //setLabel("CmdTask.StartProgramCommand error.");
		}
	    });
	    	    
	    dialog.pack();
	    
            dialog.setVisible(true);
	    dialog.show();

	  }
	  TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI StartProgramCommand.CommandComplete");
      }
  }

  private static class SubExecuteCommand extends ExecuteCommand
  {
     
     public SubExecuteCommand()
     {
         super();
     }

     public void CommandComplete()
     {
	 
	 long agentRC = ReturnCode();
	 TWGRas.debug(TWGRas.EX0,"Results cmd.rc:0x" + Long.toHexString(agentRC));
	 if(agentRC == 0) {
	    TWGRas.debug(TWGRas.EX0,"ExecRC:0x" + Long.toHexString(getExecRC()));
	    agentCommandOutput = new String(getOutput());
	    // format CRLF for Windows or UNIX 
	    // Windows : replace Carriage Returns ('\r') with ' ' so 
	    //   control characters don't appear in output
	    agentCommandOutput = TWGEnvironment.isWindows() ? agentCommandOutput.replace('\r',' ') : agentCommandOutput;
	    TWGRas.debug(TWGRas.EX0,"CmdTask agent output");
	    TWGRas.debug(TWGRas.EX0,agentCommandOutput);
	    TWGRas.debug(TWGRas.EX0,"CmdTask end agent output");

            // Popup dialog displaying Agent Command output
	    final JDialog dialog = new JDialog();
	    dialog.getRootPane().setUI((new CmdTaskGUI()).getRootPane().getUI());
	    dialog.setTitle(cmdTaskName);

	    JTextArea jta = new JTextArea(agentCommandOutput);
	    
            dialog.getContentPane().add(jta);

	    dialog.setDefaultCloseOperation(
		JDialog.DISPOSE_ON_CLOSE);
	    dialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent we) {
		    //setLabel("Thwarted user attempt to close window.");
		}
	    });
	    	    
	    dialog.pack();
	    
            dialog.setVisible(true);
	    dialog.show();
	    
	    //<Agent has now returned (if possibly an agent Daemon for following cmds) >
	    TWGRas.debug(TWGRas.EX0,"CmdTask.CmdTaskGUI SubExecuteCommand.CommandComplete");
	 }
     }
  }
} // end class TWGTaskFrame  
