/***************************************************************************
 * IBM Confidential                                    	                   *
 *                                                                         *
 * OCO Source Materials                                                    *
 *                                                                         *
 * program number (04L8267)                                                *
 *                                                                         *
 * (C) Copyright 1998 by IBM                                               *
 *                                                                         *
 * The source code for this program is not published or otherwise divested *
 * of its trade secrets, irrespective of what has been deposited with the  *
 * U.S. Copyright Office.                                                  *
 ***************************************************************************/

package com.BobCo.associations;

import com.tivoli.twg.engine.*;
import java.util.Properties;
import com.tivoli.twg.log.*;
import java.util.Vector;

/**
 * Extension registration class for <code>DomainWorkgroupAssociation</code>
 * sample class.  Extension class is registered with the
 * <code>DomainWorkgroupAssociation.TWGExt</code> file.  If needed, the
 * <code>DomainWorkgroupAssociation.TWGExt</code> can be modified to include
 * additional properties particular to the extension.  These properties
 * will be loaded with the extension, and are available using
 * the <code>TWGExtension.getPropertySet()</code> method.
 * 
 * @see com.tivoli.twg.engine.TWGExtension
 */
public class DomainWorkgroupAssociationExtension extends TWGExtension {

    private static String bundleClassName = "com.BobCo.associations.BobCoTableResources";
    /**
     * Default constructor : usually does nothing, but can access
     * properties loaded from our property file.
     */
    public DomainWorkgroupAssociationExtension() {
        super();
        /* Access out extension's properties (just for an example)        */
        Properties p = getPropertySet();
        p.list(System.out);   /* Dump properties                          */
    }
    /**
     * First initialization method, intended to allow extensions to register
     * classes.  No persistent object instances (subclasses of
     * <code>TWGPersistentObject</code>) should be create nor be expected
     * to exist during this phase of initialization.  All new subclasses
     * must be registered during this phase.
     *
     * @exception <code>TWGExtensionInitException</code> if a fatal error 
     * occurs during the initialization method
     */
    public void InitClassRegistration() throws
        TWGExtensionInitException {
        String c = "";
        try {
            c = "com.BobCo.associations.DomainWorkgroupAssociation";
            RegisterClass(c);
	    c = "com.BobCo.associations.BobCoAssociationsTable";
	    RegisterClass(c);
	    //---------------------------------------------------------------
            // Associate each new database table with the all managed object
            // types that could have data in that table
            //---------------------------------------------------------------
            TWGManagedObjectFactory.associateDatabaseTable("com.tivoli.twg.engine.TWGNativeManagedObject",
                                                           c);
        } catch (ClassNotFoundException cnfx) {
            throw new TWGExtensionInitException("Class '" + c + "' not found: " +
                cnfx);
        }
    }
    /**
     * Second initialization method, intended to allow extension to create
     * default instances of classes registered during 
     * <code>InitClassRegistration()</code>.  All <code>TWGPersistentObject</code>
     * instances which previously existed have been restored before this
     * phase was started.  To prevent problems with the order of initialization
     * of class instances created by different extensions during this phase,
     * implementations of this method should avoid attempting to access
     * objects which would be initialized by other extensions.  References
     * to objects created by other extensions should wait until
     * <code>InitCompletion()</code>.
     *
     * @exception <code>TWGExtensionInitException</code> if a fatal error 
     * occurs during the initialization method
     */
    public void InitClassInstances() throws TWGExtensionInitException {
	String  errmsg = "";
        /* Create default instance of our association, if needed		    */
        try {
            DomainWorkgroupAssociation.initializeDefault();
        } catch (TWGPersistentObjectSaveException posx) {
            throw new TWGExtensionInitException("error creating default DomainWorkgroupAssociation instance");
        }

	//---------------------------------------------------------------
        // Create default queries for the BobCo tables if they don't already exist
        //---------------------------------------------------------------
        String tableClass = null;
        try
        {

           // The query id string is used to identify and search for th query.  It must be
           // unique across all saved queries created by all IT Director extensions.
           String queryIdString = "com.BobCo.BobCoAssociationsTable";

           // see if the query already exists.  No need to create it again
           TWGDatabaseQuery query = TWGDatabaseQuery.findQueryByIdString(queryIdString);
           if ( query == null )            {

              // Find out what the token name is for our table
              String BobCoToken = TWGDatabaseTable.findTableByClassName("com.BobCo.associations.BobCoAssociationsTable").getTokenName();
              // Find out what the token name is for the Director Managed Object table (System table)
              String moidToken = TWGDatabaseTable.findTableByClassName("com.tivoli.twg.inventory.TWGManagedObjectTable").getTokenName();

              // vector to contain a query field object for each selected column in the query
              Vector selectFields = new Vector(1);

              // create a query parameters object using the vector of selected columns
              TWGDbQueryParms qParms = new TWGDbQueryParms( selectFields );

              // set the resource bundle name and the bundle key that will be used to look up
              // the translated query name in the bundle
              qParms.setResourceBundleName(bundleClassName);
              qParms.setResourceBundleKey("QueryName.BobCoAssociationsTable");

              // the id string identifies the query and can be used to search for a particular query
              qParms.setIdString(queryIdString);

              // keep all rows in query even if duplicate
              qParms.setRowDistinct(false);

              // this is a default query (not a user query)
              qParms.setDefault(true);

              // this query will be saved
              qParms.setSavedQuery(true);

              // create the query
              query = new TWGDatabaseQuery(qParms);
              query.save();
           }
        }
        catch (TWGDbException e)
        {
         errmsg = "BobCoDatabaseExtension.InitClassInstances: Error creating default query";
         TWGRas.error( TWGRas.EX1, errmsg, e);
        }
        catch (TWGPersistentObjectSaveException e)
        {
         errmsg = "BobCoDatabaseExtension.InitClassInstances: Error saving default query";
         TWGRas.error( TWGRas.EX1, errmsg, e);
        }

	//---------------------------------------------------------------
        // Create a filter factory that contains the tables we've defined.
        //---------------------------------------------------------------


        // The factory id string is used to identify and search for the factory.  It must be
        // unique across all filter factories created by all IT Director extensions.
        String factoryIdString = "com.BobCo.associations.factory1";

        // See if our filter factory already exists.  If not, create it.
        TWGFilterFactory factory = TWGFilterFactory.findByIdString( factoryIdString );

        if (factory == null)
        {
           try
           {
              TWGFilterFactoryParms     parms = new TWGFilterFactoryParms();

              // set the resource bundle name and the bundle key that will be used to look up
              // the translated factor name in the bundle.
              parms.setBundleName(bundleClassName);
              parms.setBundleKey("FactoryName.factory1");

              // add each table we created to the factory.
              parms.addTable("com.BobCo.associations.BobCoAssociationsTable");

              // set the factory id
              factory = new TWGFilterFactory( factoryIdString, parms);

           }
           catch (TWGDbException e)
           {
              errmsg = "DomainWorkgroupAssociationExtension: exception initializing BobCo filter factory"
                    + factoryIdString;
              TWGRas.error( TWGRas.EX1, errmsg , e);
           }
        } 
    }
    /**
     * Final initialization method, intended to allow an extension to do any
     * actions which may have required the other extensions to have completed
     * the previous passes of initialization.
     *
     * @exception <code>TWGExtensionInitException</code> if a fatal error occurs during
     * the initialization method
     */
    public void InitCompletion() throws TWGExtensionInitException {
        boolean err = false;

        /* No action required for this extension                                    */

        if(err) {
            throw new TWGExtensionInitException();
        }
    }
    /**
     * First termination method, intended to allow an extension to begin
     * the process of termination.  The only termination processing which
     * should occur here is that which needs to be done before other
     * extensions have executed TermCompletion() pass.
     *
     * @exception TWGExtensionTermException if an error occurs during the
     * termination method
     */
    public void TermBegin() throws TWGExtensionTermException {
        boolean err = false;
        if(err) {
            throw new TWGExtensionTermException();
        }
    }
    /**
     * Final termination method, intended to allow an extension to terminate
     * any processing which may still be active.
     *
     * @exception TWGExtensionTermException if an error occurs during the
     * termination method
     */
    public void TermComplete() throws TWGExtensionTermException {
        boolean err = false;
        if(err) {
            throw new TWGExtensionTermException();
        }
    }
}
