package com.tivoli.twg.mscs;

import com.tivoli.twg.libs.*;
import com.tivoli.twg.netipc.TWGSvcMgrServiceNodeConstants;

/**
 * Simple tool to dump full configuration and state of a cluster on
 * a MSCS agent bearing system.
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class DumpMSCSConfig {
    /** Service manager command to start a given service                    */
    //private static final int SVCMGR_START_SVC = 0x0101;
    /** Service manager command to stop a given service                     */
    //private static final int SVCMGR_STOP_SVC = 0x0102;
    /**
     * Main entry point
     * @param args - command line args : args[0] is IPC path to target (proto::addr)
     */
    public static void main(String[] args) {
        int rc = -1;
        /* Set ServiceNode support to use local IPC implementation          */
        ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
        ServiceNode.SetServiceNodeFactory(snlif);
        /* Create service-node                                              */
        try {
            ServiceNode sn = new ServiceNode();
            if(sn.Create() == false) {
                System.out.println("Create() failed");
                System.exit(1); // exit from stand alone utility
            }
            String svcmgr_ipc = TWGSvcMgrServiceNodeConstants.SERVICE_NODE;
            String agt_ipc = null;
            if(args.length > 0) {
                svcmgr_ipc = args[0] + "::" + TWGSvcMgrServiceNodeConstants.SERVICE_NODE;
                agt_ipc = args[0];
            }
            /* Build service start cmd                                      */
            Command startcmd = new Command(TWGSvcMgrServiceNodeConstants.SVCMGR_START_SVC);
            startcmd.SetDestinationAddress(svcmgr_ipc);
            byte[] p = new byte[IntelByteBuffer.GetASCIIZLength(MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME)];
            IntelByteBuffer.WriteASCIIZ(p, MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME, 0);
            startcmd.AttachInputParm(p);
            sn.SendCommand(startcmd);
            if(startcmd.ReturnCode() == 1) {
                System.out.println("Agent not available on target system");
                System.exit(2); // exit from stand alone utility
            }
            else if(startcmd.ReturnCode() != 0) {
                System.out.println("Error communicating with agent - rc=" +
                    Long.toHexString(startcmd.ReturnCode()));
                System.exit(2); // exit from stand alone utility
            }
            else {      /* Else, good start                                 */
                try {
                    /* Build request config command                         */
                    MSCSGetFullConfigCommand getcfgcmd = new
                        MSCSGetFullConfigCommand(agt_ipc);
                    sn.SendCommand(getcfgcmd);
                    if(getcfgcmd.ReturnCode() == 0) {   /* If successful    */
                        System.out.println("Cluster version=" +
                            getcfgcmd.getClusterMajorVersion() + "." +
                            getcfgcmd.getClusterMinorVersion() + "." +
                            getcfgcmd.getClusterBuildNumber());
                        System.out.println("Vendor ID=" + getcfgcmd.getClusterVendorID() +
                            ", CSD Version=" + getcfgcmd.getClusterCSDVersion());
                        int cnt = getcfgcmd.getNumberOfElements();
                        long state;
                        for(int i = 0; i < cnt; i++) {
                            long type = getcfgcmd.getElementType(i);
                            int j;
                            String[] slist;
                            String name = getcfgcmd.getElementName(i);
                            System.out.println("Element " + i + ": type=" +
                                Long.toHexString(type) + ", name=" + name);
                            if(type == getcfgcmd.CLUSTER_ENUM_NODE) {
                                state = getcfgcmd.getElementNodeState(i);
                                slist = getcfgcmd.getElementNodeNetworkInterfaceList(i);
                                System.out.print("  state=" + Long.toHexString(state) + ", netints={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                            }
                            else if(type == getcfgcmd.CLUSTER_ENUM_NETWORK) {
                                slist = getcfgcmd.getElementNetworkNetworkInterfaceList(i);
                                System.out.print("  netints={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                            }
                            else if(type == getcfgcmd.CLUSTER_ENUM_NETINTERFACE) {
                                state = getcfgcmd.getElementNetInterfaceState(i);
                                System.out.println("  state=" + Long.toHexString(state));
                            }
                            else if(type == getcfgcmd.CLUSTER_ENUM_GROUP) {
                                state = getcfgcmd.getElementGroupState(i);
                                System.out.println("  state=" + Long.toHexString(state) +
                                    ", node=" + getcfgcmd.getElementGroupNodeName(i));
                                slist = getcfgcmd.getElementGroupResourceList(i);
                                System.out.print("  resources={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                                slist = getcfgcmd.getElementGroupPreferredOwnerList(i);
                                System.out.print("  prefnodes={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                            }
                            else if(type == getcfgcmd.CLUSTER_ENUM_RESOURCE) {
                                state = getcfgcmd.getElementResourceState(i);
                                System.out.println("  state=" + Long.toHexString(state) +
                                    ", owner_node=" + getcfgcmd.getElementResourceOwnerNode(i) +
                                    ", owner_group=" + getcfgcmd.getElementResourceCurrentOwner(i) +
                                    ", dep_network=" + getcfgcmd.getElementResourceDependentNetwork(i));
                                slist = getcfgcmd.getElementResourceDependenciesList(i);
                                System.out.print("  dependencies={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                                slist = getcfgcmd.getElementResourceDependentsList(i);
                                System.out.print("  dependents={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                                slist = getcfgcmd.getElementResourceHostNodesList(i);
                                System.out.print("  host_nodes={");
                                for(j = 0; j < slist.length; j++) {
                                    if(j != 0)
                                        System.out.print(", " + slist[j]);
                                    else
                                        System.out.print(slist[j]);
                                }
                                System.out.println(" }");
                                System.out.println("  type=" + getcfgcmd.getElementResourceType(i));
                            }
                        }
                    } else {
                        System.out.println("Error requesting configuration, rc=" +
                            Long.toHexString(getcfgcmd.ReturnCode()));
                    }
                } finally {
                    /* Send service stop command                                             */
                    Command stopcmd = new Command(TWGSvcMgrServiceNodeConstants.SVCMGR_STOP_SVC);
                    stopcmd.SetDestinationAddress(svcmgr_ipc);
                    p = new byte[IntelByteBuffer.GetASCIIZLength(MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME)];
                    IntelByteBuffer.WriteASCIIZ(p, MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME, 0);
                    stopcmd.AttachInputParm(p);
                    sn.SendCommand(stopcmd);
                }
                rc = 0;
            }
            sn.Close();
        } catch(BadServiceNodeImplException snix) {
            System.out.println(snix);
            rc = 3;
        } catch(ServiceNodeClosedException sncx) {
            System.out.println(sncx);
            rc = 4;
        } catch(ServiceNodeException snx) {
            System.out.println(snx);
            rc = 5;
        } catch(InterruptedException ix) {
            System.out.println(ix);
            rc = 6;
        }
        System.exit(rc); // exit from stand alone utility
    }
}
