package com.tivoli.twg.mscs;

import com.tivoli.twg.engine.TWGServerCommand;
import com.tivoli.twg.engine.TWGServerServiceNodeConstants;
import com.tivoli.twg.libs.IntelByteBuffer;

/**
 * Command class for requesting adding new MSCS managed object
 *
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSAddObjectCmd extends TWGServerCommand {
    /** Command name for adding new object                                  */
    public static final String CMDNAME = "AddObject.MSCSMO.tivoli";
    /**
     * Construct command for adding a new object
     * @param nam - name of object to add
     * @param cnam - cluster name
     */
    public MSCSAddObjectCmd(String nam, String cnam) {
        super(CMDNAME);
        SetDestinationAddress(TWGServerServiceNodeConstants.TWGSERVERNAME);
        /* Add parameter for name string                                    */
        byte[] buf = new byte[IntelByteBuffer.GetCompUnicodeLength(nam)];
        IntelByteBuffer.WriteCompUnicode(buf, nam, 0);
        AttachInputParm(buf);
        /* Add parameter for cluster name string                            */
        cnam = cnam.toUpperCase();
        buf = new byte[IntelByteBuffer.GetCompUnicodeLength(cnam)];
        IntelByteBuffer.WriteCompUnicode(buf, cnam, 0);
        AttachInputParm(buf);
    }
    /**
     * Test to see if add command was successful
     * @return true if successful
     */
    public boolean wasSuccessful() { return (ReturnCode() == 0); }
    /**
     * Test to see if command failed with duplicate cluster name
     * @return true if failed due to duplicate cluster name
     */
    public boolean wasDuplicateClusterName() { return (ReturnCode() == 3); }

}
