/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.monitors;

import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;

import com.tivoli.twg.monitor.api.*;

import java.lang.NumberFormatException;
import java.util.Locale;
import java.util.Enumeration;
import java.lang.Long;


/*******************************************************************************
 *                                                                             *
 * This is the Java sample for the Director Monitor Server APIs.            *
 *                                                                             *
 * @author William Trautman                                                    *
 * @version July 1999                                                          *
 *                                                                             *
 *******************************************************************************/
public class MonAPISample2 {
   // ----------------
   // class variables
   // ----------------

   static TWGMonAPI   api = null;


   public static void main(String argv[])
   {
      Long        lVal = null;
      long        moid = -1;


      // print out heading and usage information
      if (argv.length == 1) {
            try {
               lVal = java.lang.Long.valueOf(argv[0]);
               moid = lVal.longValue();
               System.out.println("\nMonAPISample2 with moid="+argv[0]+"\n");
            } catch (NumberFormatException nfe) {
               System.out.println("\nUsage: MonAPISample2 <moid> \n");
               return;
            } // end try
      } else {
         System.out.println("\nUsage: MonAPISample2 <moid> \n");
         return;
      } // end if

      // Enable Component tracing.
      TWGRas.initialize();

      // required initialization of the ServiceNode class with a service node factory
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);

      try {
         //--------------------------------------------------------------------
         // create a Monitor API object to communicate with the monitor server.
         // indicate that responses are expected in english and that we would like
         // the object to check for data updates every 5 seconds.
         //--------------------------------------------------------------------
         api = new TWGMonAPI( Locale.ENGLISH, 5000);
      } catch (com.tivoli.twg.monitor.api.TWGMonException e) {
         ;
      } // end try

      //--------------------------------------------------------------------
      // get an attribute and monitor its data values for a bit
      //--------------------------------------------------------------------
      TWGMonPath path = new TWGMonPath();
      path.addLevel("Director Agent");
      path.addLevel("MONCPU");
      path.addLevel("2872344980");     // CPU Utilization

      System.out.println("\nabout to get a 'CPU Utilization' attribute for path="+path.toString());
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS, "MonAPISample2.main: about to start monitoring");
      TWGMonAttribute attribute = api.getAttribute( path, moid );

      /*- need to create object to be notified -*/
      UpdateObject listener = new UpdateObject();

      attribute.registerUpdateListener(listener);

      System.out.println("about to start checking every 15 secs for data");
      for (int index = 0; index < 6; index++) {
         try {
            Thread.sleep(15000);
         } catch (InterruptedException e) {
            ;
         } // end try
         System.out.println("check number "+(index+1)+" of 6, "+
                            "attribute data = "+ attribute.getDataAsString());
         if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
            TWGRas.debug(TWGRas.MONITORS,
                         "MonAPISample2.main: check number "+(index+1)+" of 6, "+
                         "attribute data = "+ attribute.getDataAsString());
      } // end for index < 12

      attribute.deregisterUpdateListener(listener);

      System.out.println("stopping monitoring");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample2.main: stopping monitoring");
      api.freeAttribute(attribute);

      System.out.println("nulling attribute");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample2.main: null attribute");
      attribute = null;

      // we have finished using the api object,  it is now time to tell it
      // we will not be using it anymore. this must be done before the
      // reference is released so that garbage collection will succeed in
      // reclaiming the object.
      System.out.println("shutdown API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample2.main: shutdown API");
      api.shutdown();

      // free up the reference to the object to allow garbage collection
      // to clean up the remains of the object.
      System.out.println("nulling API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample2.main: null API");
      api = null;

      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample2.main: exiting");
      System.out.println("exiting");
      return;
   } // end method main

   static public class UpdateObject implements TWGMonAttributeUpdateListener {

      public UpdateObject() {
      }

      /**
       * Attribute Updated callback.  This listener method is called when an attribute has
       * received updated information.
       * @param attribute the attribute that is reporting the update
       */
      public void attributeUpdated(TWGMonAttribute attribute) {
         // let folks know we got data
         System.out.println("listener got update - "+
                            "attribute data = "+ attribute.getDataAsString());
         if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
            TWGRas.debug(TWGRas.MONITORS,
                         "listener got update - "+
                         "attribute data = "+ attribute.getDataAsString());

      }
   }

} // end MonAPISample2 Class

