/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.monitors;

import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;

import com.tivoli.twg.monitor.api.*;

import java.lang.NumberFormatException;
import java.util.Locale;
import java.util.Enumeration;
import java.lang.Long;


/*******************************************************************************
 *                                                                             *
 * This is the Java sample for the Director Monitor Server APIs.            *
 *                                                                             *
 * @author William Trautman                                                    *
 * @version July 1999                                                          *
 *                                                                             *
 *******************************************************************************/
public class MonAPISample3 {
   // ----------------
   // class variables
   // ----------------

   static TWGMonAPI   api = null;


   public static void main(String argv[])
   {

      // print out heading and usage information
      System.out.println("\nMonAPISample3 - recursive listing of attributes on 3 Native MOIDs\n");

      // Enable Component tracing.
      TWGRas.initialize();

      // required initialization of the ServiceNode class with a service node factory
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);

      try {
         //--------------------------------------------------------------------
         // create a Monitor API object to communicate with the monitor server.
         // indicate that responses are expected in english and that we would like
         // the object to check for data updates every 5 seconds.
         //--------------------------------------------------------------------
         api = new TWGMonAPI( Locale.ENGLISH, 5000);
      } catch (com.tivoli.twg.monitor.api.TWGMonException e) {
         ;
      } // end try


      //--------------------------------------------------------------------
      // prepare to recursively traverse attribute tree and just print it out
      //--------------------------------------------------------------------
      System.out.println("doing tree traversal now");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "doing tree traversal now");

      //--------------------------------------------------------------------
      // get a list of all Native Managed Object ids that are currently available
      //--------------------------------------------------------------------
      long[] moidList= api.getAllNativeMOIDs();
      System.out.println("found " + moidList.length + "moids");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "found " + moidList.length + "moids");

      //--------------------------------------------------------------------
      // change format of list for call (keep list to three or less systems
      // for speed of example)
      //--------------------------------------------------------------------
      LongValueSet moidSet = new LongValueSet(moidList, 0, Math.min(moidList.length,3) );
      System.out.println("moids to be checked  = " + moidSet);
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS, "moids to be checked = " + moidSet);

      System.out.println("experiment one: monpath(director agent)");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "experiment one: monpath(director agent)");

      TWGMonPath path = new TWGMonPath();
      path.addLevel("Director Agent");  //start with NMOs for now...
      listAttrs(path, moidSet); // call recursive routine with root

      // now try this one
      System.out.println("experiment two: empty monpath ");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "experiment two: empty monpath");

      TWGMonPath path2 = new TWGMonPath();
      listAttrs(path2, moidSet); // call recursive routine with root

      // we have finished using the api object,  it is now time to tell it
      // we will not be using it anymore. this must be done before the
      // reference is released so that garbage collection will succeed in
      // reclaiming the object.
      System.out.println("shutdown API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample3.main: shutdown API");
      api.shutdown();

      // free up the reference to the object to allow garbage collection
      // to clean up the remains of the object.
      System.out.println("nulling API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample3.main: null API");
      api = null;

      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample3.main: exiting");
      System.out.println("exiting");
      return;
   } // end method main

   //--------------------------------------------------------------------
   // recusively traverse attributes tree and just print it out for now...
   //--------------------------------------------------------------------
   static public void listAttrs(TWGMonPath mPath, LongValueSet mSet) {

      // state what attributes we are now looking for
      System.out.println("calling findAttributes for path = " + mPath);
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "calling findAttributes for path = " + mPath); //has toString

      try {
         // ask the api for the list of attributes using the synchronous
         // call to get the data,  pass in the path,  the list of systems,
         // the fact that we want the response in english and that we only wish to
         // wait 20 seconds for a response
         TWGMonNodeList nodelist = api.findAttributes(mPath, mSet,Locale.ENGLISH, 20000);

         // print out the node list that was returned
         System.out.println(""+nodelist);
         if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
            TWGRas.debug(TWGRas.MONITORS,
                         "" + nodelist);

         // for each node in the list, check for child attribute nodes in the
         // attribute tree
         for (Enumeration e = nodelist.getNodes(); e.hasMoreElements();) {
            TWGMonNode node =(TWGMonNode)e.nextElement();
            // if node is not a leaf node, recursively call this routine to
            // traverse down the next level in the tree under the currently
            // selected node.
            if ( node.isPathNode() ) {
               TWGMonPath mp = (TWGMonPath)mPath.clone();
               mp.addLevel( node.getProgName() );
               listAttrs( mp, mSet); //here we go again
               mp = null;
            } // end if isPathNode
         } //for enum
      } catch (com.tivoli.twg.monitor.api.TWGMonException e) {
         TWGRas.error(TWGRas.MONITORS, "Error connecting to monitorAPI");
      } // end try/catrchg block

      return;
   } // end method listAttrs

} // end MonAPISample3 Class

