/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.browserlaunch;

import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGExtension;
import com.tivoli.twg.engine.TWGExtensionInitException;
import com.tivoli.twg.engine.TWGExtensionTermException;
import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskCreationException;
import com.tivoli.twg.engine.TWGDefaultTask;
import java.util.Properties;

/**
 * Browser Launcher extension class : defined by the
 * SampleBrowserLaunch.TWGExt property file.
 */
public class SampleBrowserLaunchExtension extends TWGExtension
{
  /**
   * Default constructor : usually does nothing, but can access
   * properties loaded from our property file.
   */
  public SampleBrowserLaunchExtension()
  {
    super();
    System.out.print("SampleBrowserLaunchExtension(): id=");
    try {
      System.out.println(ObjectID());
    } catch (InvalidObjectIDException ioidx) {
      System.out.println("<invalid>");
    }
    Properties p = getPropertySet();
    p.list(System.out);     /* Dump properties */
    System.out.println("SampleBrowserLaunchExtension() done");
  }

  /**
   * First initialization method, intended to allow extensions to
   * register classes.
   *
   * @exception TWGExtensionInitException if a fatal error occurs
   * during the initialization method
   */
  public void InitClassRegistration() throws TWGExtensionInitException
  {
    System.out.println("SampleBrowserLaunchExtension.InitClassRegistration()");
  }

  /**
   * Second initialization method, intended to allow extension to
   * create default instances of classes registered during
   * InitClassRegistration().
   *
   * @exception TWGExtensionInitException if a fatal error occurs
   * during the initialization method.
   */
  public void InitClassInstances() throws TWGExtensionInitException
  {
    System.out.println("SampleBrowserLaunchExtension.InitClassInstances()");
    boolean err = false;

    // Instantiate "SampleBrowserLaunch" task
    if (!TWGTask.isTaskID("BobCo|BrowserLaunchSample"))
    {
      try {
        TWGDefaultTask testTask = new TWGDefaultTask(
                  "com/BobCo/browserlaunch/SampleBrowserLaunch.properties" );
      } catch (TWGTaskCreationException e) {
        System.out.println("Task creation exception!!!");
        e.printStackTrace();
        err = true;
      }
    }

    if (err)
      throw new TWGExtensionInitException( "Error creating Sample browser launch default task");
  }

  /**
   * Final initialization method, intended to allow an extension to
   * do any actions which may have required the other extensions to
   * have completed the previous passes of initialization.
   *
   * @exception TWGExtensionInitException if a fatal error occurs
   * during the initialization method
   */
  public void InitCompletion() throws TWGExtensionInitException
  {
    System.out.println("SampleBrowserLaunchExtension.InitCompletion()");
    /*- Do task association stuff here */
    boolean err = false;
    if (err)
      throw new TWGExtensionInitException();
  }

  /**
   * First termination method, intended to allow an extension to
   * begin the process of termination.  The only termination
   * processing which should occur here is that which needs to be
   * done before other extensions have executed TermCompletion() pass.
   *
   * @exception TWGExtensionTermException if an error occurs during
   * the termination method.
   */
  public void TermBegin() throws TWGExtensionTermException
  {
    System.out.println("SampleBrowserLaunchExtension.TermBegin()");
    boolean err = false;
    if (err)
      throw new TWGExtensionTermException();
  }

  /**
   * Final termination method, intended to allow an extension to
   * terminate any processing which may still be active.
   *
   * @exception TWGExtensionTermException if an error occurs during
   * the termination method.
   */
  public void TermComplete() throws TWGExtensionTermException
  {
    System.out.println("SampleBrowserLaunchExtension.TermComplete()");
    boolean err = false;
    if (err)
      throw new TWGExtensionTermException();
  }
}
