/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved.                            *
 *****************************************************************************/	
package com.BobCo.inventory;

import  com.tivoli.twg.engine.TWGDatabaseConstants;
import  com.tivoli.twg.engine.TWGDatabaseQuery;
import  com.tivoli.twg.engine.TWGDbCharColumn;
import  com.tivoli.twg.engine.TWGDbCreateQueryCmd;
import  com.tivoli.twg.engine.TWGDbException;
import  com.tivoli.twg.engine.TWGDbFindTableByClassnameCmd;
import  com.tivoli.twg.engine.TWGDbFindTableReply;
import  com.tivoli.twg.engine.TWGDbIntegerColumn;
import  com.tivoli.twg.engine.TWGDbISOTimeParse;
import  com.tivoli.twg.engine.TWGDbResultSetMetaData;
import  com.tivoli.twg.engine.TWGDbQueryResults;
import  com.tivoli.twg.engine.TWGDbResultSet;
import  com.tivoli.twg.engine.TWGDbQueryField;
import  com.tivoli.twg.engine.TWGDbQueryParms;
import  com.tivoli.twg.engine.TWGDbRunQueryCmd;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.Command;
import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeImplFactory;
import  com.tivoli.twg.libs.ServiceNodeLocalImplFactory;
import  java.io.NotSerializableException;
import  java.io.ObjectStreamException;
import  java.util.Vector;
import  java.util.Locale;
import  java.util.TimeZone;
import  java.util.Date;
import  java.lang.reflect.*;




class SampleCreateRunQueryCmd extends Command
{
   public SampleCreateRunQueryCmd(int cc) {
      super(cc);
   }
   public void CommandComplete() {

   try
   {


      TWGDbResultSet dbrs = null;
      if (ReturnCode() == 0)
      {
          dbrs = (TWGDbResultSet)this.GetObjectFromOutputParm(0);
          if (dbrs == null)
          {
             System.out.println(" query result set is null!");
             return;
          }
      }
      else
      {
         System.out.println("Return code from Create Run Query was " + ReturnCode() + "\n" +
                            "Create Run Query Cmd failed!");
         return;
      }

      if (dbrs.getNumOfRows() == 0)
      {
          System.out.println( "No information in this query result set!");
          return;
      }

      // set pointer before the first row in the result set
      dbrs.setPosition(0);
      // get values for columns and display them.
      System.out.println(" Values:");

      //=======================

      /*// Abstract display routine (more general)
      // This display routine prints a tab delimited view of all
      // fields return by the query Command. This construction is
      // more abstract and does not rely as closely upon the type
      // of database.
      while(dbrs.next()){
	  Object o = null;
	  int i;
	  try{
	      for(i = 1, o = dbrs.getObject(i); ; i++,o = dbrs.getObject(i)){
		  System.out.print(o.toString() + "\t");
	      }
	  }catch(ArrayIndexOutOfBoundsException aioobe){
	      //do nothing, end of query selectfields
	  }
	  System.out.println();
	  //next Managed Object
      }*/

      //=======================

      // NON-abstract display code
      // This display routine works specifically on only
      // the com.BobCo.inventory.BobCoDeviceTable type.
      System.out.println("MOID\tIndex\tType\t\tChipset\t\tDate\t\tSize");
      while (dbrs.next())
      {
         // Print managed object id, specifying the column using its query position
         if (dbrs.getInt(1) == TWGDbIntegerColumn.NULL)
            System.out.println("No value for managed object id");
         else
            System.out.print(dbrs.getInt(1) + "\t");

         // Print device index, specifying the column using column token name
         if (dbrs.getInt(2) == TWGDbIntegerColumn.NULL)
            System.out.println("No value for device index");
         else
            System.out.print(dbrs.getInt(2) + "\t");

         // print device type
         if ((dbrs.getString(3)).equals(TWGDbCharColumn.NULL))
            System.out.println("No value for device type");
         else
            System.out.print(dbrs.getString(3).toString() + "\t");

	 // print device chipset
	 if ((dbrs.getString(4)).equals(TWGDbCharColumn.NULL))
            System.out.println("No value for device chipset");
         else
            System.out.print(dbrs.getString(4).toString() + "\t");

	 // print device date
         if ((dbrs.getObject(5)).equals(TWGDbCharColumn.NULL))
            System.out.println("No value for device date");
         else{
	    TWGDbISOTimeParse isoparse = new TWGDbISOTimeParse(TimeZone.getTimeZone("GMT"));
	    Date date = new Date();
	    date.setTime(dbrs.getLong(5));
            System.out.print(isoparse.formatUsingYYMMDD(date) + "\t");
	 }

	 // print device size
	 if ((dbrs.getObject(6)).equals(TWGDbCharColumn.NULL))
            System.out.println("No value for device size");
         else
            System.out.print(dbrs.getDouble(6) + "\t");

	 // print line before going on to next managed object info
	 System.out.println();
      }

      //=======================

   }
   catch (TWGDbException dbx)
   {
      System.out.println(dbx.getMessage());
   }
   catch (ObjectStreamException osx)
   {
      System.out.println(osx);
   }
   catch (ClassNotFoundException cnfx)
   {
      System.out.println(cnfx);
   }
  }
}

public class SampleCreateRunQuery
{

   /*
   * Create a query and run it.
   */

   public static void main(String argv[])
   {
      // Set ServiceNode support to use local IPC implementation
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);


      try
      {
         // Create service node
         String serviceNodeName = "SampleCreateRunQuery";
         ServiceNode sn = new ServiceNode(serviceNodeName, true);
         if (sn.Create() == false)
         {
            System.out.println("Create() failed");
            return;
         }

         // Find out what the token name is for our table.  The token name for a given
         // table needs to be retrieved only once, which can be done when an application
         // is started because the token name for a given table does not change.

         // create a database command object to hold the table class name
         String tableTokenName = null;
	 String tableName = null;

	 // a different table class may be queried as long as it has the appropriate fields
	 // required by the specified select fields. Using the Abstract methods listed below
	 // allows tables to be queried using unspecified select fields.
	 if(argv.length>0)
	     tableName = argv[0];
	 else
	    tableName = "com.BobCo.inventory.BobCoDeviceTable";
         TWGDbFindTableByClassnameCmd dbFindTableByClassnameCmd =
                        new TWGDbFindTableByClassnameCmd(tableName);

         // create the IPC command and add the database command as its input parm
         Command dbCmd = new Command(TWGDbFindTableByClassnameCmd.TWG_DB_FIND_TABLE_BY_CLASSNAME_CMD);
         dbCmd.SetDestinationAddress(TWGDatabaseConstants.UPDATE_SERVICE_NODE_NAME);

         // add the database command as the input parm.
         dbCmd.AddObjectAsInputParm(dbFindTableByClassnameCmd);
         // send the command
         sn.SendCommand(dbCmd);

         if (dbCmd.ReturnCode() == 0)
         {
            TWGDbFindTableReply dbReply = (TWGDbFindTableReply)dbCmd.GetObjectFromOutputParm(0);

            if (dbReply == null)
            {
               System.out.println("Find Table reply is null for table : " + tableName);
               return;
            }
            else
            {
               tableTokenName =  dbReply.getTableTokenName();
            }
         }
         else
         {
	
            System.out.println("Return code from Find Table was " + dbCmd.ReturnCode() + " for : " + tableName);
            return;
         }
         System.out.println("table token is " +  tableTokenName);
         /*
         Create a user query with JOIN format.
         First builds query parameters and then sends these in a create run query command to the DB.

         */
         Long queryId = null;
         // vector to contain a query field object for each selected column in the query
         Vector selectFields = new Vector(6);
         // add the managed object id column, device index column, and device type column, etc...
	
	 //=======================

	 /*// Abstract method (more general)
	 // This method of forming a query is more abstract and does not rely on naming specific
	 // fields in the table, tableName. Using java introspection/reflection, this method
	 // looks for Field strings that have been set to their own names, and adds these fields
	 // to the query vector. (ie MANAGED_OBJ_ID = "MANAGED_OBJ_ID" would be selected to be queried)
	 Class c = Class.forName(tableName);
	 Field[] fs = c.getDeclaredFields();
	 Field f;
	 Object o = null;
	 try{
	    // class instance from which to invoke static Field objects
	    o = c.newInstance();
	 } catch(InstantiationException ie){
	     System.out.println("Can't get class instance for " + tableName);
	     return;
	 } catch(IllegalAccessException iae){
	     System.out.println("Illegal Access to " + tableName + "::" + iae.getMessage());
	 }

	 int i;
	 for(i = 0; i<fs.length; i++){
	    f = fs[i];
	    try{
	        System.out.println(f.getName() + " : " + f.get(o));
		//get only the Fields who's variable name matches the variable data string
		if(f.getName().compareTo(f.get(o)) == 0)
		    selectFields.addElement(new TWGDbQueryField(tableTokenName, (String)f.get(o)));
	    } catch(IllegalAccessException iae){}
	 }*/

	 //=======================

	 // NON-abstract method
	 // This method specifically names the fields in the com.BobCo.inventory.BobCoDeviceTable .
	 // It therefore relies on these field names being present.
         selectFields.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.MANAGED_OBJ_ID));
         selectFields.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.DEVICE_INDEX));
         selectFields.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.DEVICE_TYPE));
         selectFields.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.DEVICE_CHIPSET));
         selectFields.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.DEVICE_DATE));
         selectFields.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.DEVICE_SIZE));

	 //=======================

         // Build a query parameter with the selected field
         TWGDbQueryParms qParms = new TWGDbQueryParms(selectFields);
         // The query will not be saved.
         qParms.setSavedQuery(false);
         // The query values will be translated.
         qParms.setTranslateValues(true);
         // Keep all rows in query even if duplicate
         qParms.setRowDistinct(false);
         // This is a user query)
         qParms.setDefault(false);
         // The query is JOIN format
         qParms.setFormat(TWGDatabaseQuery.JOIN_FORMAT);
         // set column fields by which to sort the result set
         Vector sortField = new Vector();
         // sort the result set by Managed Object id.
         sortField.addElement(new TWGDbQueryField(tableTokenName, BobCoDeviceTable.MANAGED_OBJ_ID));
         qParms.setSortField(sortField);
         // Build create query command
         TWGDbCreateQueryCmd cmdParm = new TWGDbCreateQueryCmd(qParms);
         // Sets the locale for the query result set.
         cmdParm.setLocale(Locale.getDefault());
         // Create a command to send create query request
         dbCmd = new SampleCreateRunQueryCmd(TWGDbCreateQueryCmd.TWG_DB_CREATE_RUN_QUERY_CMD);
         // set the destination to the database query service node
         dbCmd.SetDestinationAddress(TWGDatabaseConstants.QUERY_SERVICE_NODE_NAME);
         // add the create query database command object as an input parameter
         // to the IPC command
         dbCmd.AddObjectAsInputParm(cmdParm);
         // send the command and wait for returning.
         sn.SendAsynchCommand(dbCmd);
      }
      catch (TWGDbException dbx)
      {
         System.out.println(dbx.getMessage());
      }
      catch (NotSerializableException nsx)
      {
         System.out.println(nsx.getMessage());
      }
      catch (BadServiceNodeImplException snix)
      {
         System.out.println(snix);
      }
      catch (ServiceNodeClosedException sncx)
      {
         System.out.println(sncx);
      }
      catch (ServiceNodeException snx)
      {
         System.out.println(snx);
      }
      catch (InterruptedException ix)
      {
         System.out.println(ix);
      }
      catch (ClassNotFoundException cnfx)
      {
         System.out.println(cnfx);
      }
      catch (ObjectStreamException osx)
      {
         System.out.println(osx);
      }
   }
}

