/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2004 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
package com.BobCo.snapins;

import java.awt.*;
import java.beans.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.snmp.*;
import com.tivoli.twg.snmputils.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.guilibs.*;

/**
 * The SystemSnapIn class is a sample SNMP info browser.
 * It implements the TWGSNMPInfoBrowserInterface and is
 * registered to display instances SNMP system information.
 *
 * @version 1.1 - 6/8/05
 */
public class SystemSnapIn implements TWGSNMPInfoBrowserInterface2,
                                     SNMPRemoteResponse
{

   /* Private data */
   private ImagePanel              image_panel      = null;
   private JPanel                  panel            = null;
   private JLabel                  icon_panel;
   private int                     status           = TWGSNMPConstants.UNKNOWN;
   private String                  statusText       = null;
   private WindowAttributes        windowAttributes = null;
   private PropertyChangeSupport   prop             = null;
   private ListResourceBundle      bundle           = null;
   private TWGMIBItem2             mibitem          = null;
   private TWGRemoteMIB            remotemib        = null;
   private SNMPRemoteRequest       remoteRequest    = null;
   private Object                  mibValue         = null;
   private long                    moid;
   private int                     mibtype;
   private String                  oid;
   private String                  rawoid;
   private JLabel                  separator;
   private JLabel                  label_text;
   private JTextArea               label_value;
   private JLabel                  oid_text;
   private JTextArea               oid_value;
   private ImageIcon               tivoliIcon       = null;
   private ImageIcon               ibmIcon          = null;
   private boolean                 view_built       = false;
   private static final String     tivoliIconPath   = "com/BobCo/snapins/images/tivoli.gif";
   private static final String     ibmIconPath      = "com/BobCo/snapins/images/ibm.gif";

   /**
    * Constructor.  The snap-in interface requires a default
    * constructor that does not have any parameters.
    */
   public SystemSnapIn ()
   {
      /* Create a Java Bean property change support object */
      /* to manage the property change listeners           */
      prop = new PropertyChangeSupport( this );
   }

   /**
    * The getDisplayWindow method is used to return a JPanel
    * that will be displayed by the caller.  The snap-in is
    * responsible for the contents and appearance of the panel.
    */
   public JPanel getDisplayWindow ()
   {
      /* If we have already created a panel, just re-use it*/
      if (image_panel == null)
      {
         /* Load the NLS resource bundle for display strings */
         if (bundle == null)
         {
            bundle = (ListResourceBundle)ListResourceBundle.getBundle
                     ( "com.BobCo.snapins.BobCoSnapInsResources",
                     Locale.getDefault() );
         }

         /* Create the panel with a Border layout. (Note that  */
         /* we actual create an ImagePanel which is a subclass */
         /* of JPanel).                                        */
         image_panel = new ImagePanel();
         image_panel.setLayout(new BorderLayout());

         /* Apply any window attributes */
         if (windowAttributes != null)
         {
            JComponent panels[] = {image_panel};
            ImageLabel labels[] = {};
            JComponent others[] = {};
            windowAttributes.applyColors(panels, labels, others);
            image_panel.invalidate();
            image_panel.validate();
            image_panel.repaint();
         }
      }

      /* Return the panel */
      return image_panel;
   }

   /**
    * Retrieve the current status of the snap-in.  Status values
    * should be one of the constants defined in TWGSNMPConstants.
    */
   public int getStatus()
   {
      return status;
   }

   /**
    * Set the current status of the snap-in.  Status values
    * should be one of the constants defined in TWGSNMPConstants.
    * After changing the status, any property change listeners
    * are notified of the change.
    */
   public void setStatus( int newStatus )
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* Take actions based on the new status */
      switch ( newStatus )
      {
         /* If we are closing, then remove all references to    */
         /* our data objects so that they are garbage collected */
         /* by the Java run-time.                               */
         case TWGSNMPConstants.CLOSE:
//            if (response != null)
//            {
               /* If we are in the middle of an SNMP response, then */
               /* cancel the response processing.                 */
//               response.cancel();
//               response = null;
//            }
            panel      = null;
            bundle     = null;
            statusText = null;
            mibitem    = null;
            windowAttributes = null;
            break;

         /* If we are in-progress, then update the status text */
         case TWGSNMPConstants.IN_PROGRESS:
            setStatusText(bundle.getString("ReadingDisk"));
            break;

         /* Otherwise we must be ready, so update the status text */
         default:
            setStatusText(bundle.getString("Ready"));
            break;
      }

      /* Notify any property change listeners */
      int oldStatus = status;
      status = newStatus;
      if (newStatus != TWGSNMPConstants.CLOSE)
      {
         prop.firePropertyChange( "statusChange",
                                  new Integer(oldStatus),
                                  new Integer(newStatus));
      }
   }

   /**
    * Retrieve a text string that describes the current status
    * of the snap-in.  This string may be used by the parent of
    * the snap-in to update the status area at the bottom of
    * the window.
    */
   public String getStatusText()
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* If we have never set the status text, then assume */
      /* we are ready                                      */
      if ( statusText == null )
      {
         statusText = bundle.getString("Ready");
      }

      /* Return the latest status text string */
      return statusText;
   }

   /**
    * Set the current status text message for the snap-in.
    */
   public void setStatusText( String newStatusText )
   {
      statusText = newStatusText;
   }

   /**
    * Retrieve the SNMP MIB Data object that is being displayed
    * by the snap-in.
    */
   public TWGMIBItem2 getMIBItem()
   {
      return mibitem;
   }

   /**
    * Set the MIB info that the snap-in will display.
    */
   //public void setSNMPInfo( aSNMPClass newSNMPClass, aSNMPInstance newSNMPInstance )
   //public void setMIBItem( SNMPMIBNode newmibnode )
   public void setMIBItem( long moid, String rawoid, String OID )
   {
      /* Save the class and instance objects */
      remotemib  = new TWGRemoteMIB();
      mibitem = remotemib.OIDLookup( OID );

      try{
         remoteRequest = new SNMPRemoteRequest();
         long rc = remoteRequest.sendGetRequest(moid, rawoid, this);
      } catch(ServiceNodeException Err)
      {
      //   System.out.println("ib.setmibitem.snexception");
      }

      /* Set the status to in-progress */
      setStatus(TWGSNMPConstants.IN_PROGRESS);

      /* Be sure that we have a display window to update */
      getDisplayWindow();

      /* Build new sub-components of our display window */
      buildView();

      updateView();

      setStatus(TWGSNMPConstants.READY);
   }

   /**
    * Set the MIB info that the snap-in will display, no value.
    *  @param long  MOID - The Managed Object ID of the TWGSNMPDevice that the operation was targeted at.
    *  @param String OID - The SNMP OID that is to be used as a target for this operation.
    */
   public void setMIBItem( long moid, String OID )
   {
   }

   /**
    *  For use with an old server.  snap-ins are not allowed with old browser,
    *  but this has to be here anyway for use with the default browser
    */
   public void setMIBItem( long MOID, String RawOID, String OID, String Value, ServiceNode sn )
   {
   }
   /**
    *  For use with an old server.  snap-ins are not allowed with old browser,
    *  but this has to be here anyway for use with the default browser
    */
   public void setResp( int RC, String OID, Object Value)
   {
   }

   public void response(SNMPRemoteRequest caller, long moid, int requestType, long requestID, VariableBinding[] vbl, long errorIndex)
   {
      if (requestType == SNMPRemoteResponse.GET) {
         if (errorIndex >= 0) {
            if (panel == null) {
               //System.out.println("vPanel is null");
            } else
            {
               this.moid = moid;

               if (vbl[0] instanceof CounterVbl) {
                  rawoid = ((CounterVbl)vbl[0]).getOid();
                  mibValue  = ((CounterVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof Counter64Vbl) {
                  rawoid = ((Counter64Vbl)vbl[0]).getOid();
                  mibValue  = ((Counter64Vbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof GaugeVbl) {
                  rawoid = ((GaugeVbl)vbl[0]).getOid();
                  mibValue  = ((GaugeVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof IntegerVbl) {
                  rawoid = ((IntegerVbl)vbl[0]).getOid();
                  mibValue  = ((IntegerVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof IpAddressVbl) {
                  rawoid = ((IpAddressVbl)vbl[0]).getOid();
                  mibValue  = ((IpAddressVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof NsapVbl) {
                  rawoid = ((NsapVbl)vbl[0]).getOid();
                  mibValue  = ((NsapVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof OidVbl) {
                  rawoid = ((OidVbl)vbl[0]).getOid();
                  mibValue  = ((OidVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof OpaqueVbl) {
                  rawoid = ((OpaqueVbl)vbl[0]).getOid();
                  mibValue  = ((OpaqueVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof StringVbl) {
                  rawoid = ((StringVbl)vbl[0]).getOid();
                  mibValue  = ((StringVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof TimeticksVbl) {
                  rawoid = ((TimeticksVbl)vbl[0]).getOid();
                  mibValue  = ((TimeticksVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof UnsignedVbl) {
                  rawoid = ((UnsignedVbl)vbl[0]).getOid();
                  mibValue  = ((UnsignedVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof NoSuchObjectVbl) {
                  rawoid = ((NoSuchObjectVbl)vbl[0]).getOid();
                  mibValue  = ((NoSuchObjectVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof NoSuchInstanceVbl) {
                  rawoid = ((NoSuchInstanceVbl)vbl[0]).getOid();
                  mibValue  = ((NoSuchInstanceVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof EndOfMibViewVbl) {
                  rawoid = ((EndOfMibViewVbl)vbl[0]).getOid();
                  mibValue  = ((EndOfMibViewVbl)vbl[0]).getValueAsObject();
               } else if (vbl[0] instanceof EmptyVbl) {
                  rawoid = ((EmptyVbl)vbl[0]).getOid();
                  mibValue  = ((EmptyVbl)vbl[0]).getValueAsObject();
               }
               if (mibValue != null) {
                  label_value.setText(mibValue.toString());
               }
               oid_value.setText(rawoid);
               image_panel.invalidate();
               image_panel.validate();
               image_panel.repaint();

               //pdateMIBValue( mibitem, MOID, RawOID, Value ) ;
            }
         } else {
            JOptionPane.showMessageDialog(panel,
                           "Failed to get reponse",
                           "ERROR",
                           JOptionPane.INFORMATION_MESSAGE);
         }
      }
   }

   /**
    * This method is called by the parent of the snap-in to
    * update the window attributes so that they match the
    * rest of the current user console.
    */
   public void setWindowAttributes( WindowAttributes wa )
   {
      /* Save the new attributes */
      windowAttributes = wa;

      /* Only appy the attributes if we have an image_panel */
      if (image_panel != null)
      {
         JComponent panels[] = {image_panel};
         if (panel != null)
         {
            panels = new JComponent[3];
            panels[0] = image_panel;
            panels[1] = panel;
            panels[2] = icon_panel;
         }
         /* Apply the new attributes to our window */
         ImageLabel labels[] = {};
         JComponent others[] = {};
         windowAttributes.applyColors(panels, labels, others);

         /* Repaint the window */
         image_panel.invalidate();
         image_panel.validate();
         image_panel.repaint();
      }
   }

   /**
    * Retrieve the current window attributes
    */
   public WindowAttributes getWindowAttributes()
   {
      return windowAttributes;
   }

   /**
    * Add a property change listener to the snap-in.
    */
   public void addPropertyChangeListener( PropertyChangeListener listener )
   {
      prop.addPropertyChangeListener( listener );
   }

   /**
    * Remove a property change listener from the snap-in.
    */
   public void removePropertyChangeListener( PropertyChangeListener listener )
   {
      prop.removePropertyChangeListener( listener );
   }


   /**
    * This internal method is used to build the sub-components
    * of the display window.
    */
   private void buildView()
   {
      /* If we previously built a view, then remove all of the */
      /* old subcomponents (let them be garbage collected)     */
      if (view_built)
      {
         image_panel.removeAll();
      }

      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* Create sub-components */
      panel = new JPanel();
      //panel.setLayout(new BorderLayout());
      panel.setLayout(new GridBagLayout());
      label_text = new JLabel(bundle.getString("Name"));
      label_value = new JTextArea();
      label_value.setLineWrap( true );
      label_value.setEditable( false );
      oid_text = new JLabel(bundle.getString("oid"));
      oid_value = new JTextArea();
      oid_value.setLineWrap( true );
      oid_value.setEditable( false );

      icon_panel = new JLabel();
      icon_panel.setHorizontalAlignment(SwingConstants.CENTER);

      separator = new JLabel();
      /* Set flag to indicate the view has been built */
      view_built = true;
   }

   /**
    * This internal method is called to update the view
    * with data from the instance attributes.
    */
   private void updateView ()
   {
     // label_value.setText(mibnode.Name);
     // oid_value.setText(mibnode.OID);

//      tivoliIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(tivoliIconPath));
      ibmIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(ibmIconPath));
      icon_panel.setIcon(ibmIcon);

      /* Add all of the view constructs to the display */
      Insets i = new Insets(0,5,0,5);
      makeGridBagLayoutItem (panel, new JPanel(),    0,0, 1,1,1,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    6,0, 1,1,1,0,i);
      makeGridBagLayoutItem (panel, separator,       0,1, 3,1,0,0,i);
      makeGridBagLayoutItem (panel, label_text,      1,2, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, label_value,     2,2, 3,1,1,0,i);
      makeGridBagLayoutItem (panel, separator,       0,3, 3,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,4, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, oid_text,        1,5, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, oid_value,       2,5, 3,1,1,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,6, 1,1,0,1,i);
      makeGridBagLayoutItem (panel, icon_panel,      2,7, 3,1,1,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,8, 1,1,0,1,i);
      panel.setSize(panel.getPreferredSize());

      image_panel.add(panel,BorderLayout.CENTER);
      //setWindowAttributes(getWindowAttributes());
      /* Repaint the window */
      image_panel.invalidate();
      image_panel.validate();
      image_panel.repaint();
   }

   /**
    * This is an internal method used to simplify the usage
    * of GridBagLayout.
    */
   private void makeGridBagLayoutItem (Container cont,
                                       Component comp,
                                       int       x,
                                       int       y,
                                       int       w,
                                       int       h,
                                       double    weightx,
                                       double    weighty,
                                       Insets    i)
   {
     GridBagLayout gbl = (GridBagLayout)cont.getLayout();
     GridBagConstraints c = new GridBagConstraints();
     c.anchor     = GridBagConstraints.NORTHWEST;
     c.fill       = GridBagConstraints.BOTH;
     c.gridx      = x;
     c.gridy      = y;
     c.gridwidth  = w;
     c.gridheight = h;
     c.weightx    = weightx;
     c.weighty    = weighty;
     if (i != null)
     {
        c.insets = i;
     }
     cont.add(comp);
     gbl.setConstraints(comp, c);
   }

}
