/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 * 
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2002 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
******************************************************************************
*
* Change Activity:
*   01-21-99 Candi King - created 
*   03-24-99 Nathan Fritze - stole mercilessly to create cluster browser
*  <CMVC header here>
*****************************************************************************/
package com.tivoli.twg.mscs;

import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.NotSerializableException;
import java.io.IOException;
import java.net.UnknownHostException;
import java.text.MessageFormat;
import java.util.Date;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.tree.DefaultMutableTreeNode;

import com.tivoli.twg.console.NLSUtilities;
import com.tivoli.twg.console.TWGConsoleColor;
import com.tivoli.twg.console.TWGPanelTitle;
import com.tivoli.twg.engine.TWGConManagedObject;
import com.tivoli.twg.engine.TWGConObject;
import com.tivoli.twg.engine.TWGTaskActivator;
import com.tivoli.twg.guilibs.ImageLabel;
import com.tivoli.twg.guilibs.TWGFindParameters;
import com.tivoli.twg.guilibs.TWGSplitPane;
import com.tivoli.twg.guilibs.TWGTreeListCellRenderer;
import com.tivoli.twg.guilibs.UFTreeListView;
import com.tivoli.twg.guilibs.UFTreeListViewEvent;
import com.tivoli.twg.guilibs.UFTreeListViewListener;
import com.tivoli.twg.guilibs.WindowAttributeEvent;
import com.tivoli.twg.guilibs.WindowAttributeListener;
import com.tivoli.twg.guilibs.WindowAttributes;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.MultiLocaleBundle;

import com.tivoli.uif.core.UFImageLoader;
  
public class TWGMSCSClassBrowser extends TWGSplitPane
                                implements WindowAttributeListener,
                                           PropertyChangeListener
{

   public final static String SPLIT_BAR_LOC = "sbl";
    
   //
   // private data
   //
   
   private JPanel             classesPanel      = null;
   private UFTreeListView     classesTreeView   = null;
   private TWGPanelTitle      classesPanelLabel = null;
   private TWGMSCSClassListener classListener     = null;
   private TWGMSCSClusterObject current_selection = null;

   private JPanel             instancesPanel    = null;
   private JLabel             iSelectLabel      = null;
   private JPanel             nip               = null;
   private JPanel             fillerPanel1      = null;
   private JPanel             fillerPanel2      = null;
   
   private TWGSplitPane splitter=null;
   private float splitBarLoc = 0.40f;

   private Font boldFont = null;
   
   private MultiLocaleBundle      mlb          = null;
   private static final String ADDRESSNLSCLASS = "com.tivoli.twg.mscs.TWGMSCSResources";
   private String namespace = null;
   private LongValueSet targetset = null;
   private TWGMSCSClassBrowser Owner = null;
   private TWGMSCSObjectBrowser    instBr        = null;
   private TWGMSCSClusterObject    root          = null; // the parent of all cluster objects!
   private int                     updateCount   = 0;

   private PropertyChangeSupport   propchg       = null;
   private String                  statusStr     = null;
   private boolean                 nipFlag       = false;
   private Integer                 status        = new Integer( TWGMSCSConstants.UNKNOWN );
   private Integer                 cstatus       = new Integer( TWGMSCSConstants.UNKNOWN );
   private TWGTaskActivator        ta            = null;
   ImageIcon PendingIcon   = new ImageIcon(UFImageLoader.getImage("/com/tivoli/twg/snmp/images/snrqpg16.gif"));

   public TWGMSCSClassBrowser(TWGTaskActivator taskActivator, LongValueSet targetSet){
      //constructor
      Owner = this;
      ta = taskActivator;
      setStatus( TWGMSCSConstants.UNKNOWN );
      targetset = targetSet;
      boldFont = NLSUtilities.getBoldFont();

      try 
         {
         mlb = MultiLocaleBundle.getSharedBundle( ADDRESSNLSCLASS );
         } 
      catch ( MissingResourceException mrx ) 
         {
         mlb = null;
         }

      // build the root of the cluster tree
      root=new TWGMSCSClusterObject("root", null, null);

      //tree view in left pane for queries
      classesPanel = new JPanel();
      classesPanel.setLayout(new BorderLayout());
      classesPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
      classesPanel.setAlignmentY(JPanel.TOP_ALIGNMENT);

      classesPanelLabel = new TWGPanelTitle( getNlsString("MSCSTreePanelTitle") ); 
      classesPanelLabel.setHorizontalAlignment( JLabel.CENTER );
      classesPanelLabel.setAlignmentX(JPanel.CENTER_ALIGNMENT);
      classesPanelLabel.setFont( boldFont );
      classesPanel.add(classesPanelLabel, BorderLayout.NORTH);

      populateClustersTree(true); //true means create for the first time.
      classesPanel.add( classesTreeView, BorderLayout.CENTER );
      //create a listener for the classes panel
      classListener = new TWGMSCSClassListener(classesTreeView);
      classesTreeView.addTreeListViewListener(classListener);

      instancesPanel = new JPanel();
      instancesPanel.setLayout(new BorderLayout());
      instancesPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
      instancesPanel.setAlignmentY(JPanel.TOP_ALIGNMENT);

      nip = new JPanel();
      nip.setLayout(new GridBagLayout());
      fillerPanel1 = new JPanel();
      fillerPanel2 = new JPanel();
      makeGridBagLayoutItem( nip, fillerPanel1, 0,0,1,1,0,1,null );
      iSelectLabel = new JLabel( getNlsString("selectAClass"), SwingConstants.CENTER );
      makeGridBagLayoutItem( nip, iSelectLabel, 0,1,1,1,0,0,null );
      makeGridBagLayoutItem( nip, fillerPanel2, 0,2,1,1,0,1,null );
      instancesPanel.add( nip, BorderLayout.CENTER );

      this.add(classesPanel, BorderLayout.WEST);
      this.add(instancesPanel, BorderLayout.EAST);
      this.setSplitBarPosition( splitBarLoc );
      this.setBorder(new javax.swing.border.LineBorder( TWGConsoleColor.getBlackColor() ));
      

      /* Handle user window attributes */
      updateWithNewAttributes(ta.getMainConsole().getWindowAttributes());
      ta.getMainConsole().addWindowAttributeChangeListener(this);
      
   }
   
   private void populatePartialTree(DefaultMutableTreeNode partialparent, TWGMSCSClusterObject newchild) {
       // add the current child
       newchild.setIcon();
       classesTreeView.addNode(partialparent,newchild);
       if (partialparent instanceof TWGMSCSClusterObject) {
           TWGMSCSClusterObject tmco = (TWGMSCSClusterObject)partialparent;
           if (tmco.getTree() != null) {
               newchild.setTree(tmco.getTree());
           }
       }
       // add any children to this child
       if (newchild.getChild() != null) {
           TWGMSCSClusterObject cur=newchild.getChild();
           do {
               populatePartialTree(newchild, cur);
               cur=cur.getSibling();
           } while ((cur != null) && (cur != newchild.getChild()));
           cur=null;
       }
   }

   public void populateClustersTree(boolean create){
      // mark we're doing something
      setStatus(TWGMSCSConstants.BUSY);
      // put the system names in the tree when creating.
      if (create) {
         classesTreeView = new UFTreeListView(UFTreeListView.TREE_VIEW);
         classesTreeView.setAsSorted(UFTreeListView.SORT_ASCENDING);
         classesTreeView.setAsSingleSelect();
         classesTreeView.enableRefresh( true );
         classesTreeView.setTreeCellRenderer(new TWGTreeListCellRenderer());
         
         int length = targetset.Length();  // get the length of the array

         // enable prop change support
         propchg = new PropertyChangeSupport( this );

         for (int x = 0;x != length;x++){
            Long moid = new Long(targetset.GetValue(x));
            // put the cimresponses in a vector

            long oid = moid.longValue();
            // Get the managed object shadow (to get updated info)
            TWGConManagedObject obj = (TWGConManagedObject)TWGConObject.FindObject(oid);
       
            // If we have an object to update from.
            if (obj != null) {
                // build the branch for this cluster
                TWGMSCSClusterObjectWithServiceNode cowsn=new
                    TWGMSCSClusterObjectWithServiceNode(obj.getName(),null,null,oid);

                // make sure this node knows what tree it belongs to!
                cowsn.setTree(classesTreeView);

                // update the data for this cluster
                if (cowsn.Update(this)) {
                    updateCount++;
                }

                // attach this branch to the root
                cowsn.setClusterParent(root);

                // release the local copy of the reference
                cowsn=null;
            }
         }

      } else {
         // we're doing a refresh - update the tree!
         TWGMSCSClusterObject                   tmco;
         TWGMSCSClusterObjectWithServiceNode    twsn;

         // enum all children of root, and tell them to Update()
         if (root != null) {
            tmco=root.getChild();
            if (tmco != null) {
                do {
                    // if this child is a cluster root, process it
                    if (tmco instanceof TWGMSCSClusterObjectWithServiceNode) {
                        twsn=(TWGMSCSClusterObjectWithServiceNode)tmco;
                        // it is, update the data
                        if (twsn.Update(this)) {
                            updateCount++;
                        }
                    }
                    // go to the next child
                    tmco=tmco.getSibling();
                } while ((tmco!=null) && (tmco!=root.getChild()));
            }
         }
      }
   }

   public void completeUpdate(TWGMSCSClusterObjectWithServiceNode tree) {
       // update the icon
       tree.setIcon();
       // okay, update the tree with the new data
       if (classesTreeView != null) {
           populatePartialTree(classesTreeView.getRoot(), tree);
       }

       // check if all clusters are done refreshing, and mark ready if so
       updateCount--;
       if (updateCount <= 0) {
           updateCount=0;
           setStatus(TWGMSCSConstants.READY);
       }
       // refresh the tree view
       classesTreeView.refresh();

       // if we have a current selection, reselect it to update the right panel
       if (current_selection != null) {
           // load the object browser for the class
           instBr = new TWGMSCSObjectBrowser(current_selection);

           // refresh the right panel
           // clear out the old panel
           if ( instancesPanel != null ) {
              remove( instancesPanel );
           }

           // build the new panel
           instancesPanel = new JPanel();
           instancesPanel.setLayout(new BorderLayout());
           instancesPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
           instancesPanel.setAlignmentY(JPanel.TOP_ALIGNMENT);
           instancesPanel.add(instBr.getDisplayWindow(), BorderLayout.CENTER);
           instBr.setWindowAttributes(ta.getMainConsole().getWindowAttributes());
           instBr.addPropertyChangeListener( Owner );
           add(instancesPanel, BorderLayout.EAST);

           // refresh the window
           invalidate();
           validate();
           repaint();
       }
   }

   public String getNlsString( String ID ) {
      String name = null;

      try {
         name = mlb.getString( Locale.getDefault(), ID );
      } 
      catch ( MissingResourceException mrx ) {
         // error - missing resources
      }

      return name;
   }

   public int getStatus(){
      //return status info
      // UNKNOWN    -1
      // READY       0
      // IN_PROGRESS  1
      // BUSY        2
      // CLOSE       3

      // nkf - the instance broswer no longer has a meaningful status
      /*
      switch (status.intValue()) {
         case TWGMSCSConstants.IN_PROGRESS:
            if (instBr != null) {
               if ( instBr.getStatus() == TWGMSCSConstants.BUSY ) {
                  return instBr.getStatus();
               }
            } 
            break;
         case TWGMSCSConstants.READY:
            if (instBr != null) {
               return instBr.getStatus();
            } 
            break;
      }
      */
      return status.intValue();
   }



   // This method sets the instance browser's own internal status,
   // represented by the 'istatus' variable, which may be different
   // from that of the property browser.  After it sets this, it 
   // calls evaluateStatus for evaluation, to see if this change will 
   // affect the overall status if the instance browser/property browser
   // team, which is represented by the 'status' variable.
   public synchronized void setStatus( int newStatus ){
      cstatus = new Integer( newStatus );
      /*
      // Lines for debugging
      if ( cstatus.intValue() == TWGMSCSConstants.READY )
         System.out.println( "CLASS BROWSER IS NOW READY" );
      else if ( cstatus.intValue() == TWGMSCSConstants.BUSY )
         System.out.println( "CLASS BROWSER IS NOW BUSY" );
      else if ( cstatus.intValue() == TWGMSCSConstants.CLOSE )
         System.out.println( "CLASS BROWSER IS NOW GOINT TO CLOSE" );
      else if ( cstatus.intValue() == TWGMSCSConstants.IN_PROGRESS )
         System.out.println( "CLASS BROWSER IS NOW IN_PROGRESS" );
      */
      evaluateStatus();
      return;
   }



   // This method sets the overall status of the Default Instance
   // Browser/ Property Browser team.  It must derive a new joint
   // status based on its internal istatus variable and the
   // Property Browser's getStatus() method.
   public synchronized void evaluateStatus(){
      // SET status info 
      // UNKNOWN    -1
      // READY       0
      // IN_PROGRESS  1
      // BUSY        2
      // CLOSE       3 

      int     newStatus = TWGMSCSConstants.UNKNOWN;
      Integer oldstatus = status;

      switch ( cstatus.intValue() ){
         case TWGMSCSConstants.BUSY:
            // The Class Browser is already busy
            setStatusText( getNlsString( "Busy" ) );
            newStatus = TWGMSCSConstants.BUSY;
            break;

         case TWGMSCSConstants.IN_PROGRESS:
            // The Class Browser is in progress
            setStatusText( getNlsString( "pending" ) );
            newStatus = TWGMSCSConstants.IN_PROGRESS;
//            if ( instBr != null ) {
//               if ( instBr.getStatus() == TWGMSCSConstants.BUSY ) {
//                  // if the instance browser is busy, so are we
//                  setStatusText( instBr.getStatusText() );
//                  newStatus = TWGMSCSConstants.BUSY;
//               }
//            } 
            break;
         case TWGMSCSConstants.CLOSE:
            if ( instBr != null )
            {
               instBr.setStatus( TWGMSCSConstants.CLOSE );
            }
            pCleanUp();
            break;
         case TWGMSCSConstants.READY:
            // The class browser is ready
            setStatusText( getNlsString( "Ready" ) );
            newStatus = TWGMSCSConstants.READY;
//            if ( instBr != null ) {
//               setStatusText( instBr.getStatusText() );
//               newStatus = instBr.getStatus();
//            } 
            break;
      }

      status = new Integer( newStatus );

      if ( propchg != null ) 
      {
         if ( oldstatus.intValue() != status.intValue() )
         {
            /*
            // Lines here for debugging
            System.out.println( "CLASS BROWSER FIRING OFF PROPERTY CHANGE EVENT" );
            if ( oldstatus.intValue() == TWGMSCSConstants.READY )
               System.out.println( "        OLD STATUS IS READY" );
            else if ( oldstatus.intValue() == TWGMSCSConstants.BUSY )
               System.out.println( "        OLD STATUS IS BUSY" );
            else if ( oldstatus.intValue() == TWGMSCSConstants.IN_PROGRESS )
               System.out.println( "        OLD STATUS IS IN_PROGRESS" );
      
            if ( status.intValue() == TWGMSCSConstants.READY )
               System.out.println( "        NEW STATUS IS READY" );
            else if ( status.intValue() == TWGMSCSConstants.BUSY )
               System.out.println( "        NEW STATUS IS BUSY" );
            else if ( status.intValue() == TWGMSCSConstants.IN_PROGRESS )
               System.out.println( "        NEW STATUS IS IN_PROGRESS" );
            */
            propchg.firePropertyChange( TWGMSCSConstants.STATUS_CHANGE,
                                        oldstatus,
                                        new Integer(newStatus) );
         }
      }
      return;
 
   }



   public String getStatusText(){
      //return status info text to update status info bar at bottom of panel
      return(statusStr);
   }
   public void setStatusText(String txt){
      // SET status info when status change occurs so updates can be made
      statusStr = txt;
   }
   public void setWindowAttributes(){
   }
   public void getWindowAttributes(){
   }
   /**
    * This method is called when the user changes the preferences
    * for console window attributes (background color, shadow 
    * border, background image, etc.).
    *
    * @param event - The window attribute event
    */


   public void windowAttributesChanged( WindowAttributeEvent event )
   {
      updateWithNewAttributes( event.getWindowAttributes() );
   }

   /**
    * This method is used to update the window based on changes
    * to the console window attributes (background color, shadow 
    * border, background image, etc.).
    *
    * @param windowAttributes - The new console window attributes
    */
   public void updateWithNewAttributes( WindowAttributes windowAttributes )
   {
      /* Update the window appearance */
      JComponent consoles[] = {classesTreeView, instancesPanel};
      if (instBr != null)
      {
         consoles = new JComponent[1];
         consoles[0] = classesTreeView;
      }
      ImageLabel labels[] = {classesPanelLabel};
      windowAttributes.applyColors(consoles, labels, null);
      nip.setBackground(windowAttributes.getAppearanceColor(2));
      fillerPanel1.setBackground(windowAttributes.getAppearanceColor(2));
      fillerPanel2.setBackground(windowAttributes.getAppearanceColor(2));
      invalidate();
      validate();
      repaint();
      if (instBr != null ) {
         instBr.setWindowAttributes(windowAttributes);
      }
   }
   public void setSelectedClass(){
      //association stuff
   }
   public void getSelectedClass(){
      //association stuff
   }
   public void pCleanUp(){
      //clean up listeners & such on close
      if (ta != null) {
         ta.getMainConsole().removeWindowAttributeChangeListener(this);
      }
      if ( instBr != null)
      {
         instBr.removePropertyChangeListener( this );
         instBr = null;
      }
      if (root != null ) {
          root.RemoveYourChildren();
          root=null;
      }
      ta                = null;
      statusStr         = null;
      status            = null;
      cstatus           = null;
      Owner             = null;
      namespace         = null;
      classesPanel      = null;
      classesTreeView   = null;
      classesPanelLabel = null;
      classListener     = null;
      current_selection = null;
      targetset         = null;
      instancesPanel    = null;
      nip               = null;
      fillerPanel1      = null;
      fillerPanel2      = null;
      splitter          = null;

   }

   public synchronized void addPropertyChangeListener( PropertyChangeListener listener )
   {
      if ( propchg != null )
         propchg.addPropertyChangeListener( listener );
   }

   public synchronized void removePropertyChangeListener( PropertyChangeListener listener )
   {
      if ( propchg != null )
         propchg.removePropertyChangeListener( listener );
   }

   public void propertyChange( PropertyChangeEvent pevt )
   {
      if ( pevt.getPropertyName().equals( TWGMSCSConstants.STATUS_CHANGE ))
      {
      int newstatus = ((Integer)pevt.getNewValue()).intValue();

      /* 
      // Lines here for debugging
      if ( newstatus == TWGMSCSConstants.READY )
         System.out.println( "INSTANCE BROWSER REQUESTING STATUS READY" );
      else if ( newstatus == TWGMSCSConstants.BUSY )
         System.out.println( "INSTANCE BROWSER REQUESTING STATUS BUSY" );
      else if ( newstatus == TWGMSCSConstants.IN_PROGRESS )
         System.out.println( "INSTANCE BROWSER REQUESTING STATUS IN_PROGRESS" );
      */

      evaluateStatus();
      }
   }
   /**
    * This internal method is used to simplify the window layout.
    *
    * @param cont - Parent container component
    * @param comp - Component being added
    * @param x - Column for component
    * @param y - Row for the component 
    * @param w - Number of columns for the component
    * @param h - Number of rows for the component
    * @param weightx - Weight of component columns used for layout
    * @param weighty - Weight of component rows used for layout
    * @param i - Insets
    */
   private void makeGridBagLayoutItem (Container cont,
                                       Component comp,
                                       int       x,
                                       int       y,
                                       int       w,
                                       int       h,
                                       double    weightx,
                                       double    weighty,
                                       Insets    i)
   {
     GridBagLayout gbl = (GridBagLayout)cont.getLayout();
     GridBagConstraints c = new GridBagConstraints();
     c.anchor     = GridBagConstraints.NORTHWEST;
     c.fill       = GridBagConstraints.BOTH;
     c.gridx      = x;
     c.gridy      = y;
     c.gridwidth  = w;
     c.gridheight = h;
     c.weightx    = weightx;
     c.weighty    = weighty;
     if (i != null)
     {
        c.insets = i;
     }
     cont.add(comp);
     gbl.setConstraints(comp, c);
   }

/** Main Query Object Listener Class*/
   public class TWGMSCSClassListener
                implements UFTreeListViewListener,
                           ActionListener,
                           PopupMenuListener
   {
      private UFTreeListView ivView = null;
//      private MSCSClassData   selectedNode   = null;
      private boolean seeSysCheckBox = true;
      JCheckBoxMenuItem    cbmi;
      Object               popupobject=null;
      private TWGFindParameters findParms = new TWGFindParameters();
      // ****************************************************************
      //
      // Public Member Functions
      //
      // ****************************************************************

      /** Public constructor  */
      public TWGMSCSClassListener( UFTreeListView view )
      {
        ivView = view;
      }

      /** Return the tasks view that this task listener is registered
          to (listening to) */
      public UFTreeListView getView()
      {
        return ivView;
      }

      /**
       * Listener function called whenever a tree list view event occurs.
       */
      public void ufTreeListViewEvent( UFTreeListViewEvent evt )
      {

         //if (!ivView.isEnabled()) {
         //   return;
         //}
         Object                 evt_object;
         Component              window = null;
         TWGMSCSClusterObject   mco;
         JPopupMenu             popup = null;
         JMenu                  jm;
         JMenuItem              mi;
         
         evt_object = evt.getTopic();
         window = (Component) evt.getSource();
      
         switch ( (int)evt.getType( ) )
         {
            case UFTreeListViewEvent.TLVE_NO_ITEMS_SELECTED:
               instancesPanel.removeAll();

               // clean up our instance browser, since we're not using it
               // at the moment anyway
               if (instBr!=null) {
                   instBr=null;
               }
               
               nip = new JPanel();
               nip.setLayout(new GridBagLayout());
               fillerPanel1 = new JPanel();
               fillerPanel2 = new JPanel();
               makeGridBagLayoutItem( nip, fillerPanel1, 0,0,1,1,0,1,null );
               iSelectLabel = new JLabel( getNlsString("selectAClass"), SwingConstants.CENTER );
               makeGridBagLayoutItem( nip, iSelectLabel, 0,1,1,1,0,0,null );
               makeGridBagLayoutItem( nip, fillerPanel2, 0,2,1,1,0,1,null );
               instancesPanel.add(nip, BorderLayout.CENTER);

               updateWithNewAttributes(ta.getMainConsole().getWindowAttributes());
               break;
            case UFTreeListViewEvent.TLVE_ITEM_COLLAPSED:
                if ( evt_object instanceof TWGMSCSClusterObject ) {
                    mco = (TWGMSCSClusterObject) evt_object;
                    mco.setExpanded(false);
                    mco.setIcon();
                }
                break;
            case UFTreeListViewEvent.TLVE_ITEM_EXPANDED:
               {  
                  //put select msg in instances panel
                  if (nipFlag) {
                     nipFlag = false;
                     nip = new JPanel();
                     nip.setLayout(new GridBagLayout());

                     makeGridBagLayoutItem( nip, fillerPanel1, 0,0,1,1,0,1,null );
                     iSelectLabel = new JLabel( getNlsString("selectAClass"), SwingConstants.CENTER );
                     makeGridBagLayoutItem( nip, iSelectLabel, 0,1,1,1,0,0,null );
                     makeGridBagLayoutItem( nip, fillerPanel2, 0,2,1,1,0,1,null );
                     instancesPanel.removeAll();
                     instancesPanel.add(nip, BorderLayout.CENTER);
                     updateWithNewAttributes(ta.getMainConsole().getWindowAttributes());
                  }

                  if ( evt_object instanceof TWGMSCSClusterObject ) {
                      TWGMSCSClusterObject  cur;

                      mco = (TWGMSCSClusterObject) evt_object;
                      mco.setExpanded(true);
                      mco.setIcon();
                  }
               }
               break;
          
            // A select event occurred in the task container.
            case UFTreeListViewEvent.TLVE_SELECT_ITEM:

                if ((evt_object != null) && (window == classesTreeView)) {
                        
                    mco = (TWGMSCSClusterObject) evt_object;

                    // clean up existing instance panel
                    if (instBr != null) {
                        instBr.setStatus(TWGMSCSConstants.CLOSE);
                    }

                    // mark our current selection
                    current_selection=mco;

                    // load the object browser for the class
                    instBr = new TWGMSCSObjectBrowser(current_selection);
                    
                    // refresh the right panel
                    // clear out the old panel
                    if ( instancesPanel != null ) {
                       remove( instancesPanel );
                    }

                    // build the new panel
                    instancesPanel = new JPanel();
                    instancesPanel.setLayout(new BorderLayout());
                    instancesPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
                    instancesPanel.setAlignmentY(JPanel.TOP_ALIGNMENT);
                    instancesPanel.add(instBr.getDisplayWindow(), BorderLayout.CENTER);
                    instBr.setWindowAttributes(ta.getMainConsole().getWindowAttributes());
                    instBr.addPropertyChangeListener( Owner );
                    add(instancesPanel, BorderLayout.EAST);

                    // refresh the window
                    invalidate();
                    validate();
                    repaint();
                }
                break;
         
            // A popup event occurred in the task container.
            case UFTreeListViewEvent.TLVE_POPUP_ITEM:
               if (evt_object != null) {
                   popup = new JPopupMenu();

                   mi = new JMenuItem(getNlsString("miRefresh"));
                   mi.setActionCommand("refresh");
                   mi.addActionListener(this);
                   popup.add(mi);

                   // make sure we know who's calling us
                   popupobject=classesTreeView.overNode(evt.getPoint(), true);
                   if ((popupobject!=null) && (popupobject instanceof DefaultMutableTreeNode)) {
                       classesTreeView.selectNode((DefaultMutableTreeNode)popupobject);
                   }
                   // if this is one of my objects with its own popup menus, add them, too
                    if ((popupobject!=null) && (popupobject instanceof TWGMSCSClusterObject)) {
                        TWGMSCSClusterObject tmcs=(TWGMSCSClusterObject)popupobject;
                        int  menucount, i;

                        menucount=tmcs.getPopupItemCount();
                        if (menucount>0) {
                            popup.add(new JSeparator());
                            for (i=0;i<menucount;i++) {
                                if (tmcs.getPopupItem(i)==null) {
                                    popup.add(new JSeparator());
                                } else {
                                    popup.add(tmcs.getPopupItem(i));
                                }
                            }
                        }

                    }
                   popup.addPopupMenuListener(this);
               } else {
                   popup = new JPopupMenu();

                   mi = new JMenuItem(getNlsString("miRefreshTree"));
                   mi.setActionCommand("refreshall");
                   mi.addActionListener(this);
                   popup.add(mi);
               }
               if (popup != null) {
                   popup.show( classesTreeView, evt.getPoint().x, evt.getPoint().y);
               }

               break;
            // A open event occurred in the query container.
            case UFTreeListViewEvent.TLVE_OPEN_ITEM:
            // A drop event occurred in the query container.
            // (evtObject is TWGDragInfo)
            case UFTreeListViewEvent.TLVE_DROP_OCCURRED:
               break;
         }//end switch
         return;
      }

      public void popupMenuCanceled(PopupMenuEvent e) {
          popupobject=null;
      }

      public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
      }

      public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
      }

      public void actionPerformed( ActionEvent evt ) {
         String command = evt.getActionCommand();

         if (command.equals("refreshall")) { // refresh all tree data, all clusters
             populateClustersTree(false);
         } else if (command.equals("refresh")) { // refresh the pointed-to cluster subtree
             // verify that we're on a supported subtree object
             if ((popupobject!=null) && (popupobject instanceof TWGMSCSClusterObject)) {
                 // we are.  need to find my closest parent who's got a service node
                 TWGMSCSClusterObject   cur=(TWGMSCSClusterObject)popupobject;

                 while ((cur!=null) && (!(cur instanceof TWGMSCSClusterObjectWithServiceNode))) {
                     cur=cur.getClusterParent();
                 }
                 // do we have a service node?
                 if (cur != null) {
                     // yup.  do the update/refresh
                     ((TWGMSCSClusterObjectWithServiceNode)cur).Update(Owner);
                     // release the reference
                     cur=null;
                 }
             }
             if (popupobject!=null) {
                 popupobject=null;
             }
         }
      }
   }// end INNER CLASS TWGMSCSClassListener
}
