package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.libs.TWGConVersion;
import com.tivoli.twg.mscs.*;

/** 
 * Object to represent a node in the cluster.  At the moment, it inherits
 * pretty much everything from the base cluster object.  (Although 
 * state and description should actually be non-null for a node)
 */
public class TWGMSCSClusterNodeObject extends TWGMSCSClusterObject
                                      implements ActionListener
                           {
    private JMenuItem     fPopupPause = null;
    private JMenuItem     fPopupResume = null;
    private JMenuItem     fPopupEvict = null;
    private JMenuItem     fPopupStartSvc = null;
    private JMenuItem     fPopupStopSvc = null;

    public TWGMSCSClusterNodeObject(String nname, String nstate, String ndesc) {
        super(nname,nstate,ndesc);
    }

    public  String  getIconName() {
        return (new String("/com/tivoli/twg/engine/images/twr16on.gif"));
    }

    public int      getColumns() {
        return 3;
    }

    public String   getColumnHeader(int column) {
        switch (column) {
            case 0:
                return getNlsString("colnameName");
            case 1:
                return getNlsString("colnameState");
            case 2:
                return getNlsString("colnameDesc");
            default:
                return getNlsString("stateUnknown");
        }
    }

    public String   getColumnString(int column) {
        switch (column) {
            case 0:
                if (getName()!=null) {
                    return getName();
                }
                break;
            case 1:
                if (getState()!=null) {
                    return getState();
                }
                break;
            case 2:
                if (getDescription()!=null) {
                    return getDescription();
                } else {
                    return new String("");
                }
            default:
                break;
        }
        return getNlsString("stateUnknown");
    }

    public void setState(String nstate) {
        // have my superclass update my state
        super.setState(nstate);

        // if we're too low a version, skip this whole thing
        if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
            // update my menu items
            // turn all three off, then turn back on the right ones
            if (fPopupPause!=null) {
                fPopupPause.setEnabled(false);
            }
            if (fPopupResume!=null) {
                fPopupResume.setEnabled(false);
            }
            if (fPopupEvict!=null) {
                fPopupEvict.setEnabled(false);
            }
            if (nstate.equals(getNlsString("stateUp"))) {
                if (fPopupPause!=null) {
                    fPopupPause.setEnabled(true);
                }
            } else if (nstate.equals(getNlsString("statePaused"))) {
                if (fPopupResume!=null) {
                    fPopupResume.setEnabled(true);
                }
            } else if (nstate.equals(getNlsString("stateDown"))) {
                if (fPopupEvict!=null) {
                    fPopupEvict.setEnabled(true);
                }
            }
        }
    }

    public int      getPopupItemCount() {
        if (super.getPopupItemCount()>0) {
            return super.getPopupItemCount()+4;
        } else {
            return 3;
        }
    }

    public JMenuItem getPopupItem(int index) {
        switch (index) {
            case 0:
                if (fPopupPause==null) {
                    fPopupPause=new JMenuItem(getNlsString("miPause"));
                    fPopupPause.setActionCommand("pause");
                    fPopupPause.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateUp")))) {
                        fPopupPause.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupPause.setEnabled(false);
                    }
                }
                return fPopupPause;
            case 1:
                if (fPopupResume==null) {
                    fPopupResume=new JMenuItem(getNlsString("miResume"));
                    fPopupResume.setActionCommand("resume");
                    fPopupResume.addActionListener(this);
                    if (!(getState().equals(getNlsString("statePaused")))) {
                        fPopupResume.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupResume.setEnabled(false);
                    }
                }
                return fPopupResume;
            case 2:
                if (fPopupEvict==null) {
                    fPopupEvict=new JMenuItem(getNlsString("miEvict"));
                    fPopupEvict.setActionCommand("evict");
                    fPopupEvict.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateDown")))) {
                        fPopupEvict.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupEvict.setEnabled(false);
                    }
                }
                return fPopupEvict;
//             case 3:
//                 if (fPopupStartSvc==null) {
//                     fPopupStartSvc=new JMenuItem(getNlsString("miStartsvc"));
//                     fPopupStartSvc.setActionCommand("startsvc");
//                     fPopupStartSvc.addActionListener(this);
//                     fPopupStartSvc.setEnabled(false);
//                     if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
//                         fPopupStartSvc.setEnabled(false);
//                     }
//                 }
//                 return fPopupStartSvc;
//             case 4:
//                 if (fPopupStopSvc==null) {
//                     fPopupStopSvc=new JMenuItem(getNlsString("miStopsvc"));
//                     fPopupStopSvc.setActionCommand("stopsvc");
//                     fPopupStopSvc.addActionListener(this);
//                     fPopupStopSvc.setEnabled(false);
//                     if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
//                         fPopupStopSvc.setEnabled(false);
//                     }
//                 }
//                 return fPopupStopSvc;
            case 3:
                // return null to have the adder add a separator instead
                return null;
            default:
                break;
        }
        if (index<super.getPopupItemCount()+4) {
            return super.getPopupItem(index-4);
        } else {
            // don't know what to add, so add a separator, I guess
            return null;
        }
    }

    public void actionPerformed( ActionEvent evt )
    {
      // Place menu and/or button handling code here.
      // Remove listener interface if neither of these are required.
      //         ....................
      String                               cmd = evt.getActionCommand();
      TWGMSCSClusterObjectWithServiceNode  owsn = null;
      ServiceNode                          sn = null;
      MSCSAgentCommand                     agtcmd = null;

      owsn=getParentWithServiceNode();
      if (owsn!=null) {
          sn=owsn.getServiceNode();
      }

      if (cmd.equals("pause")) {
          // pause processing on this cluster node
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_PAUSE_NODE, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }
          }

      } else if (cmd.equals("resume")) {
          // resume processing on this cluster node
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_RESUME_NODE, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }
          }

      } else if (cmd.equals("evict")) {
          // evict this node from the cluster
          Object[] nameargs = {getName()};
          if (JOptionPane.showConfirmDialog(null,
                                        MessageFormat.format(getNlsString("areYouSureEvict"), nameargs),
                                        getNlsString("TWGMSCSBrowseFrameTitle"),
                                        JOptionPane.YES_NO_OPTION,
                                        JOptionPane.QUESTION_MESSAGE) ==
                  JOptionPane.YES_OPTION) {
              if (owsn != null) {
                  agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_EVICT_NODE, owsn.GetMyDestination());
                  agtcmd.addCompUnicodeInputParam(getName());
                  if (sn != null) {
                      // Send command to server
                      try{
                          sn.SendAsynchCommand(agtcmd);
                      }
                      catch(InterruptedException x){
                          return;
                      } 
                      catch(ServiceNodeClosedException x){
                          return;
                      }
                      catch(ServiceNodeException x){
                          return;
                      }
                  }
              }
          }

      } else if (cmd.equals("startsvc")) {
          // start the cluster service on this node

      } else if (cmd.equals("stopsvc")) {
          // stop the cluster service on this node
          Object[] nameargs = {getName()};
          if (JOptionPane.showConfirmDialog(null,
                                        MessageFormat.format(getNlsString("areYouSureStopSvc"), nameargs),
                                        getNlsString("TWGMSCSBrowseFrameTitle"),
                                        JOptionPane.YES_NO_OPTION,
                                        JOptionPane.QUESTION_MESSAGE) ==
                  JOptionPane.YES_OPTION) {
          }

      } else {
          super.actionPerformed(evt);
      }
    } // end actionPerformed
}

