package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.ImageIcon;
import javax.swing.event.*;
import javax.swing.tree.*;
import com.tivoli.uif.core.UFImageLoader;
import com.tivoli.uif.core.*;
import com.tivoli.uif.controls.*;
import com.tivoli.twg.console.*;
import com.tivoli.twg.guilibs.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.libs.TWGConVersion;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.log.*;
import com.tivoli.twg.mscs.*;


public class TWGMSCSClusterObject extends UFTreeListViewObject
                                  implements ActionListener
                 {
    private boolean                 expanded=false;
    private boolean                 stale=false;
    private MultiLocaleBundle       mlb=null;
    private String                  name=null;
    private String                  state=null;
    private String                  description=null;
    private UFTreeListView          tree=null;
    private static final String ADDRESSNLSCLASS = "com.tivoli.twg.mscs.TWGMSCSResources";
    private JMenu                   newmenu=null;
    private JMenuItem               fPopupNewGroup=null;
    private JMenuItem               fPopupNewResource=null;
    private JMenuItem               fPopupProperties=null;

    private TWGMSCSClusterObject    clusterparent=null;
    private TWGMSCSClusterObject    sibling=null;
    private TWGMSCSClusterObject    child=null;

    public TWGMSCSClusterObject(String nname, String nstate, String ndesc) {
        super(nname, null);
        // get the refs
        name=nname;
        state=nstate;
        description=ndesc;
    
        // get my resources
        try {
           mlb = MultiLocaleBundle.getSharedBundle( ADDRESSNLSCLASS );
        }
        catch ( MissingResourceException mrx ) {
           mlb = null;
        }
    }

    public String getNlsString( String ID ) {
       String name = null;

       try {
          name = mlb.getString( Locale.getDefault(), ID );
       } 
       catch ( MissingResourceException mrx ) {
          // error - missing resources
       }

       return name;
    }
    
    public String getName() {
        return name;
    }

    public boolean isNameValid(String newname) {
        if (newname!=null) {
            int i;
            for (i=0;i<newname.length();i++) {
                if (newname.charAt(i)!=' ' && newname.charAt(i)!='\t') {
                    break;
                }
            }
            if (i<newname.length()) {
                return true;
            }
        }
        return false;
    }

    public void setState(String nstate) {
        state=nstate;
    }

    public String getState() {
        return state;
    }

    public void setDescription(String ndesc) {
        description=ndesc;
    }

    public String getDescription() {
        return description;
    }

    public void addChild(TWGMSCSClusterObject newchild) {
        newchild.setClusterParent(this);
    }

    public TWGMSCSClusterObject getChild() {
        return child;
    }

    public TWGMSCSClusterObject getChild(int childno) {
        int                     i;
        TWGMSCSClusterObject    cur;

        // loop through to find the numbered child
        for (i=0, cur=getChild();(cur!=null)&&(i<childno);i++, cur=cur.getSibling()) { }
        // check for a result
        if (cur!=null) {
            return cur;
        } else {
            return child;
        }
    }

    public TWGMSCSClusterObject getSibling() {
        return sibling;
    }

    public void setClusterParent(TWGMSCSClusterObject newparent) {
        TWGMSCSClusterObject    curptr=null;

        if (clusterparent != newparent) {
            // first, detach myself from my old parent
            if (clusterparent != null) {
                // might have to detach myself from the sibling ring
                if (sibling != null) {
                    // yup.
                    if (sibling==this) {
                        // if I'm the only one in the ring, kill the ring
                        sibling=null;
                    } else {
                        curptr=sibling;
                        // loop until I find myself again
                        while ((curptr != null) && (curptr.sibling != this)) {
                            curptr=curptr.sibling;
                        }
                        // if found (is still a ring), remove myself
                        if (curptr != null) {
                            curptr.sibling=sibling;
                            curptr=null;
                        }
                    }
                }
                // if I am the head of the sibling ring, point
                // my parent at my next sibling (might be null)
                if (clusterparent.child==this) {
                    clusterparent.child=sibling;
                }
                // deref the sibling
                sibling=null;
            }
            // should at this point not be attached to a parent, so I can
            // attach to my new parent
            clusterparent=newparent;
            if (clusterparent != null) {
                // do I need to attach to a sibling ring?
                if (clusterparent.child != null) {
                    // yup, add myself right after the head
                    sibling=clusterparent.child.sibling;
                    clusterparent.child.sibling=this;
                } else {
                    // no children - make me the whole ring
                    clusterparent.child=this;
                    sibling=this;
                }
            } else {
                // if no parent, take this time to detach from the tree, too, if necessary
                tree=null;
            }
        }
    }
    
    public TWGMSCSClusterObject getClusterParent() {
        return clusterparent;
    }

    protected TWGMSCSClusterObjectWithServiceNode getParentWithServiceNode() {
        if (this instanceof TWGMSCSClusterObjectWithServiceNode) {
            return (TWGMSCSClusterObjectWithServiceNode)this;
        } else if (clusterparent!=null) {
            return clusterparent.getParentWithServiceNode();
        } else {
            return null;
        }
    }

    public void setExpanded(boolean b) {
        expanded=b;
    }

    public boolean getExpanded() {
        return expanded;
    }

    public  String getIconName() {
        if (expanded) {
            return (new String("/com/tivoli/twg/guilibs/images/fldro16.gif"));
        } else {
            return (new String("/com/tivoli/twg/guilibs/images/fldrc16.gif"));
        }
    }

    public  void   setStale() {
        stale=true;
    }

    public  void   clearStale() {
        stale=false;
    }

    public boolean getStale() {
        return stale;
    }

    public void setStaleRecursive() {
        TWGMSCSClusterObject    cur;

        // mark myself as stale
        setStale();
        // make all my kids (and their kids) stale, too
        cur=getChild();
        if (cur != null) {
            do {
                cur.setStaleRecursive();
                cur=cur.getSibling();
            } while (cur!=null && cur!=getChild());
        }
    }

    public void deleteStaleRecursive() {
        TWGMSCSClusterObject    cur, sib;

        // if I'm stale, need to get rid of my kids, then myself.
        if (stale) {
            // get rid of the kids
            RemoveYourChildren();
            // get rid of myself
            if ((clusterparent!=null) && (tree != null)) {
                tree.removeNode(clusterparent, this);
                tree=null;
            } else {
                removeFromParent();
            }
            setClusterParent(null);
        } else {
            // otherwise, have each of my kids check to see if they're stale
            cur=getChild();
            if (cur != null) {
                do {
                    sib=cur.getSibling();
                    cur.deleteStaleRecursive();
                    cur=sib;
                } while (cur!=null && cur!=getChild());
            }
        }
    }

    public  void   setTree(UFTreeListView ntree) {
        tree=ntree;
    }

    public UFTreeListView getTree() {
        return tree;
    }

    /** 
     * Unlink my kids so they can be garbage collected gracefully
     */
    public void RemoveYourChildren() {
        TWGMSCSClusterObject    cur;

        // for each child, let it clean up its kids, too
        for (cur=getChild();cur!=null;cur=getChild()) {
            cur.RemoveYourChildren();
 //            // remove from the tree if necessary
 //            try {
 //                if ((clusterparent != null) && (tree != null)) {
 //                    tree.removeNode(clusterparent,this);
 //                    tree=null;
 //                } else {
 //                    removeFromParent();
 //                }
 //            } catch (ArrayIndexOutOfBoundsException a) {
 //                // try a last ditch effort, just to make sure
 //                try {
 //                    removeFromParent();
 //                } catch (ArrayIndexOutOfBoundsException a2) {
 //                    ;
 //                }
 //            }
            // remove from the tree if necessary -
            // ObjectWithServiceNode are roots and should not be removenoded
            if ((cur.clusterparent != null) && (cur.tree != null) && !(cur instanceof TWGMSCSClusterObjectWithServiceNode)) {
                cur.tree.removeNode(cur.clusterparent,cur);
            } else {
                cur.removeFromParent();
            }
            // remove my references to this child
            cur.tree=null;
            cur.setClusterParent(null);
            cur=null;
        }
    }

    public  void    setIcon() {
        setImage(new ImageIcon(UFImageLoader.getImage(getIconName())));
    }

    public TWGMSCSClusterObject findChild(String name) {
        TWGMSCSClusterObject    cur;

        // loop through my kids, looking for a child with this name
        cur=getChild();
        if (cur!=null) {
            do {
                if (cur.getName().equals(name)) {
                    break;
                }
                cur=cur.getSibling();
            } while ((cur!=null) && (cur!=getChild()));

            // if didn't find it, mark that fact
            if ((cur!=null) && (!cur.getName().equals(name))) {
                cur = null;
            }
        }
        return cur;
    }

    public int  getClusterChildCount() {
        int                     count=0;
        TWGMSCSClusterObject    cur;

        // count my kids;
        cur=getChild();
        if (cur!=null) {
            do {
                count++;
                cur=cur.getSibling();
            } while ((cur!=null) && (cur!=getChild()));
        }
        return count;
    }

    public int  getChildColumns() {
        if (getChild() != null) {
            return (getChild().getColumns());
        }
        return getColumns();
    }

    public String   getChildColumnHeader(int column) {
        if (getChild() != null) {
            return (getChild().getColumnHeader(column));
        } else {
            return getColumnHeader(column);
        }
    }

    public String   getChildColumnString(int child, int column) {
        int                     i;
        TWGMSCSClusterObject    cur;

        // loop through to find the numbered child
        for (i=0, cur=getChild();(cur!=null)&&(i<child);i++, cur=cur.getSibling()) { }
        // check for a result
        if (cur!=null) {
            return cur.getColumnString(column);
        } else {
            return getColumnString(column);
        }
    }

    public int      getColumns() {
        return 1;
    }

    public String   getColumnHeader(int column) {
        switch (column) {
            case 0:
                return getNlsString("colnameName");
            case 1:
                return getNlsString("colnameState");
            case 2:
                return getNlsString("colnameDesc");
            default:
                return getNlsString("stateUnknown");
        }
    }

    public String   getColumnString(int column) {
        switch (column) {
            case 0:
                if (getName()!=null) {
                    return getName();
                }
                break;
            case 1:
                if (getState()!=null) {
                    return getState();
                } else {
                    return new String("");
                }
            case 2:
                if (getDescription()!=null) {
                    return getDescription();
                } else {
                    return new String("");
                }
            default:
                break;
        }
        return getNlsString("stateUnknown");
    }

    public int      getPopupItemCount() {
        // return 3;
        return 0;
    }

    public JMenuItem getPopupItem(int index) {
        switch (index) {
            case 0:
                if (newmenu==null) {
                    newmenu=new JMenu(getNlsString("mNew"));
                    if (fPopupNewGroup==null) {
                        fPopupNewGroup=new JMenuItem(getNlsString("miNewgroup"));
                        fPopupNewGroup.setActionCommand("newgroup");
                        fPopupNewGroup.addActionListener(this);
                        if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                            fPopupNewGroup.setEnabled(false);
                        }
                    }
                    newmenu.add(fPopupNewGroup);
                    if (fPopupNewResource==null) {
                        fPopupNewResource=new JMenuItem(getNlsString("miNewresource"));
                        fPopupNewResource.setActionCommand("newresource");
                        fPopupNewResource.addActionListener(this);
                        if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                            fPopupNewResource.setEnabled(false);
                        }
                    }
                    newmenu.add(fPopupNewResource);
                }
                return newmenu;
            case 1:
                // return null to have the adder add a separator instead
                return null;
            case 2:
                if (fPopupProperties==null) {
                    fPopupProperties=new JMenuItem(getNlsString("miProperties"));
                    fPopupProperties.setActionCommand("properties");
                    fPopupProperties.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupProperties.setEnabled(false);
                    }
                }
                return fPopupProperties;
            default:
                break;
        }
        // don't know what to add, so add a separator, I guess
        return null;
    }

    public void actionPerformed( ActionEvent evt )
    {
      // Place menu and/or button handling code here.
      // Remove listener interface if neither of these are required.
      //         ....................
      String                   cmd = evt.getActionCommand();

      if (cmd.equals("newgroup")) {
          // pop up the new group interface

      } else if (cmd.equals("newresource")) {
          // pop up the new resource interface
      
      } else if (cmd.equals("properties")) {
          // pop up the properties interface
      }
    } // end actionPerformed
}

