package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.tivoli.twg.libs.*;
import com.tivoli.twg.libs.TWGConVersion;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.log.*;
import com.tivoli.twg.mscs.*;

/**
 * This class basically should represent the root of a given cluster,
 * and will own the service node for talking to said cluster.  It also has
 * the standard fields for any cluster object.
 */
public class TWGMSCSClusterObjectWithServiceNode extends TWGMSCSClusterObject 
        implements CommandCompleteListener, Runnable {
    private long                        targetMoid;
    private ServiceNode                 sn;
    private String                      destAddr;
    private MSCSGetFullConfigCommand    mgfcc=null;
    private TWGMSCSClassBrowser         treeowner=null;
    private long                        updateLevel=0;
    private JMenuItem                   fPopupRename = null;

    public TWGMSCSClusterObjectWithServiceNode(String nname, String nstate, String ndesc, long Moid) {
        super(nname,nstate,ndesc);

        targetMoid=Moid;
        try {
            sn=new ServiceNode();
            destAddr=GetMyDestination();
        }
        catch (ServiceNodeException x) {
            // oops, didn't work.  make sure destAddr and sn are null
            sn=null;
            destAddr=null;
        }
    }

    ServiceNode getServiceNode() {
        return sn;
    }

    String GetMyDestination() {
        TWGGetAttributeCommand cGetAddress;

        // fail if no service node
        if (sn==null) {
            return null;
        }

        // Build command to retrieve address from server
        cGetAddress = new TWGGetAttributeCommand( targetMoid, 
                                                  TWGNativeManagedObjectConstants.ATTRIB_IPCPATH );
        if (cGetAddress==null) {
            return null;
        }
        cGetAddress. SetDestinationAddress(TWGServerCommand.TWGSERVERNAME);

        // Send command to server
        try{
            sn.SendCommand(cGetAddress);
        }
        catch(InterruptedException x){
            return null;
        } 
        catch(ServiceNodeClosedException x){
            return null;
        }
        catch(ServiceNodeException x){
            return null;
        }
        if (cGetAddress.ReturnCode()==0){// Success
            DataValue val = cGetAddress. getAttributeValue( targetMoid, 
                                                            TWGNativeManagedObjectConstants.ATTRIB_IPCPATH );
            TWGString str = (TWGString) val;
            if (str != null) {
                return (str.toString() + "::" + 
                        com.tivoli.twg.mscs.MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME);   // build up the service target
            }
        }
        return null;
    }

    /** 
     * Update the data associated with a "Node" cluster object (or create
     * the object if it doesn't exist), including group, resource, and
     * network interface linking if necessary.
     */
    public void UpdateNode(String nodeName, long nodeState, String nodeDesc, String []nodeNetInts) {
        int                         j;
        String                      strState;
        TWGMSCSClusterObject        tfolder, niFolder, childFolder, cur;
        TWGMSCSClusterNodeObject    myNode;

        // first, do things common to both paths
        // convert state to a string
        nodeState = nodeState&0xffff;  // mask off only the bits we care about
        // convert state info into name
        if (nodeState==MSCSAgentCommand.ClusterNodeStateUnknown) {
            strState=getNlsString("stateUnknown");
        } else if (nodeState==MSCSAgentCommand.ClusterNodeUp) {
            strState=getNlsString("stateUp");
        } else if (nodeState==MSCSAgentCommand.ClusterNodeDown) {
            strState=getNlsString("stateDown");
        } else if (nodeState==MSCSAgentCommand.ClusterNodePaused) {
            strState=getNlsString("statePaused");
        } else if (nodeState==MSCSAgentCommand.ClusterNodeJoining) {
            strState=getNlsString("stateJoining");
        } else {
            // don't know this state, mark it unknown
            strState=getNlsString("stateUnknown");
        }

        // now see if my node already exists.  If it does, only need to update it, not
        // create it
        cur=findChild(nodeName);
        if (cur != null) {
            // exists, do an update
            // if the node exists, and is not a cluster node object, need to bail
            if (!(cur instanceof TWGMSCSClusterNodeObject)) {
                return;
            }
            myNode=(TWGMSCSClusterNodeObject)cur;
            // can do update now
            myNode.setState(strState);
            myNode.setDescription(nodeDesc);
            myNode.clearStale();
        } else {
            // doesn't exist, create a new node
            myNode=new TWGMSCSClusterNodeObject(nodeName, strState, nodeDesc);
            myNode.setClusterParent(this);
        }

        // need to update which network interfaces I own
        tfolder=findChild(getNlsString("objnameNetInts"));
        for (j=0;j<nodeNetInts.length;j++) {
            TWGMSCSClusterNetInterfaceObject cnio=
                (TWGMSCSClusterNetInterfaceObject)tfolder.findChild(nodeNetInts[j]);
            if (cnio !=null ) {
                cnio.setNode(nodeName);
                cnio.clearStale();
            }
        }
        // refresh the icon, if necessary
        myNode.setIcon();

        // now, need to create/update subsidiary nodes, add network interfaces, groups,
        // resources, etc
        // nodes have 3 predefined children - set those up, or update them
        cur=myNode.findChild(getNlsString("objnameActGroups"));
        if (cur==null) {
            // "Active Groups"
            cur=new TWGMSCSClusterObject(getNlsString("objnameActGroups"),null,null);
            cur.setClusterParent(myNode);
            cur.setIcon();
        } else {
            cur.clearStale();
        }

        // find the groups that this node "owns" and mark them active here
        tfolder=findChild(getNlsString("objnameGroups"));
        TWGMSCSClusterGroupObject cgo=(TWGMSCSClusterGroupObject)tfolder.getChild();
        if (cgo != null) {
            do {
                // found one - add a copy to my active groups folder
                if (cgo.getOwner().equals(nodeName) && !cgo.getStale()) {
                    TWGMSCSClusterGroupObject ncgo=
                        (TWGMSCSClusterGroupObject)cur.findChild(cgo.getName());
                    if (ncgo==null) {
                        ncgo=new TWGMSCSClusterGroupObject(cgo.getName(), cgo.getState(),
                                                      cgo.getDescription(), cgo.getOwner());
                        ncgo.setClusterParent(cur);
                    } else {
                        ncgo.setState(cgo.getState());
                        ncgo.setDescription(cgo.getDescription());
                        ncgo.setOwner(cgo.getOwner());
                        ncgo.clearStale();
                    }
                    ncgo.setIcon();
                }
                cgo=(TWGMSCSClusterGroupObject)cgo.getSibling();
            } while ((cgo!=null) && (cgo!=tfolder.getChild()));
        }

        cur=myNode.findChild(getNlsString("objnameActRes"));
        if (cur==null) {
            // "Active Resources"
            cur=new TWGMSCSClusterObject(getNlsString("objnameActRes"),null,null);
            cur.setClusterParent(myNode);
            cur.setIcon();
        } else {
            cur.clearStale();
        }

        // find the resources that this node "owns" and mark them active here
        tfolder=findChild(getNlsString("objnameResources"));
        TWGMSCSClusterResourceObject cro=(TWGMSCSClusterResourceObject)tfolder.getChild();
        if (cro != null) {
            do {
                // found one - add a copy to my active groups folder
                if (cro.getOwner().equals(nodeName) && !cro.getStale()) {
                    TWGMSCSClusterResourceObject ncro=
                        (TWGMSCSClusterResourceObject)cur.findChild(cro.getName());
                    if (ncro==null) {
                        ncro=new TWGMSCSClusterResourceObject(cro.getName(), cro.getState(),
                                                         cro.getDescription(), cro.getOwner(),
                                                         cro.getGroup(), cro.getResType());
                        ncro.setClusterParent(cur);
                    } else {
                        ncro.setState(cro.getState());
                        ncro.setDescription(cro.getDescription());
                        ncro.setOwner(cro.getOwner());
                        ncro.setGroup(cro.getGroup());
                        ncro.setResType(cro.getResType());
                        ncro.clearStale();
                    }
                    ncro.setIcon();
                }
                cro=(TWGMSCSClusterResourceObject)cro.getSibling();
            } while ((cro!=null) && (cro!=tfolder.getChild()));
        }

        cur=myNode.findChild(getNlsString("objnameNetInts"));
        if (cur==null) {
            // "Network Interfaces"
            cur=new TWGMSCSClusterObject(getNlsString("objnameNetInts"),null,null);
            cur.setClusterParent(myNode);
            cur.setIcon();
        } else {
            cur.clearStale();
        }

        // find the network interfaces that this node owns, copy them
        // to my local "network interfaces" subfolder
        tfolder=findChild(getNlsString("objnameNetInts"));
        if (tfolder != null) {
            TWGMSCSClusterNetInterfaceObject cnio=(TWGMSCSClusterNetInterfaceObject)tfolder.getChild();
            if (cnio != null) {
                do {
                    // found one - add a copy to my active groups folder
                    if ((cnio.getNode()!=null) && (cnio.getNode().equals(nodeName)) && !cnio.getStale()) {
                        TWGMSCSClusterNetInterfaceObject ncnio=
                            (TWGMSCSClusterNetInterfaceObject)cur.findChild(cnio.getName());
                        if (ncnio==null) {
                            ncnio=new TWGMSCSClusterNetInterfaceObject(cnio.getName(), cnio.getState(),
                                                             cnio.getDescription(), cnio.getAddress(),
                                                             cnio.getNetwork(), cnio.getNode());
                            ncnio.setClusterParent(cur);
                        } else {
                            ncnio.setState(cnio.getState());
                            ncnio.setDescription(cnio.getDescription());
                            ncnio.setAddress(cnio.getAddress());
                            ncnio.setNetwork(cnio.getNetwork());
                            ncnio.setNode(cnio.getNode());
                            ncnio.clearStale();
                        }
                        ncnio.setIcon();
                    }
                    cnio=(TWGMSCSClusterNetInterfaceObject)cnio.getSibling();
                } while ((cnio!=null) && (cnio!=tfolder.getChild()));
            }
        }
    }

    /** 
     * Update the data associated with a "Resource Type" cluster object (or create
     * the object if it doesn't exist)
     */
    public void UpdateResType(String resName, String resDLLName, String strDesc) {
        TWGMSCSClusterObject        rtFolder, cur;
        TWGMSCSClusterResTypeObject crto;

        String  nlsName;
        // names coming from the base are probably english.  if it's a
        // name of a resource type that we know, fix it to the correct
        // language
        nlsName=getNLSResTypeStr(resName);

        // now find if the restype already exists
        rtFolder=findChild(getNlsString("objnameResTypes"));
        cur=rtFolder.findChild(nlsName);
        if (cur != null) {
            if (!(cur instanceof TWGMSCSClusterResTypeObject)) {
                // bad node - bail!
                return;
            }
            // is an existing object - update it!
            crto=(TWGMSCSClusterResTypeObject)cur;
            crto.setResDLL(resDLLName);
            crto.setDescription(strDesc);
            crto.clearStale();
            // force an icon update, if necessary (probably not)
            crto.setIcon();
        } else {
            // new restype - add it!
            crto=new TWGMSCSClusterResTypeObject(nlsName, resDLLName, strDesc);
            crto.setClusterParent(rtFolder);
            crto.setIcon();
        }
    }

    /** 
     * Refresh all data from my service base
     */
    public boolean Update(TWGMSCSClassBrowser newtree) {

        // make sure we're going to update the right tree
        if (treeowner != newtree) {
            treeowner=newtree;
        }

        // if we've already got an outstanding refresh, don't
        // need another one
        if (mgfcc != null) {
            return false;
        }

        // if I have a destination address and a service node, get new data!
        if ((destAddr!=null)&&(sn!=null)) {
            // make sure we reset the change notification, too
            MSCSAgentCommand mac=new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_GET_NOTIFIES, destAddr);
            mac.addDWORDInputParam(updateLevel);
            mac.setCommandCompleteListener(this);
            // Send command
            try{
                sn.SendAsynchCommand(mac);
            }
            catch(InterruptedException x){
            }
            catch(ServiceNodeClosedException x){
            }
            catch(ServiceNodeException x){
            }

            // get data from the agent
            mgfcc=new MSCSGetFullConfigCommand(destAddr);
            mgfcc.setCommandCompleteListener(this);

            // Send command
            try{
                sn.SendAsynchCommand(mgfcc);
            }
            catch(InterruptedException x){
                mgfcc=null;
                return false;
            } 
            catch(ServiceNodeClosedException x){
                mgfcc=null;
                return false;
            }
            catch(ServiceNodeException x){
                mgfcc=null;
                return false;
            }
            return true;
        } else {
            return false;
        }
    }

    public void run() {
        // got a command back - should at this point be able to update the
        // tree, or mark a bad status message if necessary
        if (mgfcc != null) {
            if (mgfcc.ReturnCode() == 0 ) {
                // good code, update the tree
                int                     i,j,cnt;
                TWGMSCSClusterObject    cur;

                // first, set everything in the tree to "stale"
                setStaleRecursive();

                // I am not stale.
                clearStale();

                // rebuild the standard groups first, because we're going to need them
                // to put the kids into!
                // "Groups"
                cur=findChild(getNlsString("objnameGroups"));
                if (cur==null) {
                    cur=new TWGMSCSClusterObject(getNlsString("objnameGroups"),null,null);
                    cur.setClusterParent(this);
                    cur.setIcon();
                } else {
                    cur.clearStale();
                }
                // "Resources"
                cur=findChild(getNlsString("objnameResources"));
                if (cur==null) {
                    cur=new TWGMSCSClusterObject(getNlsString("objnameResources"),null,null);
                    cur.setClusterParent(this);
                    cur.setIcon();
                } else {
                    cur.clearStale();
                }
                // "Resource Types"
                cur=findChild(getNlsString("objnameResTypes"));
                if (cur==null) {
                    cur=new TWGMSCSClusterObject(getNlsString("objnameResTypes"),null,null);
                    cur.setClusterParent(this);
                    cur.setIcon();
                } else {
                    cur.clearStale();
                }
                // "Networks"
                cur=findChild(getNlsString("objnameNetworks"));
                if (cur==null) {
                    cur=new TWGMSCSClusterObject(getNlsString("objnameNetworks"),null,null);
                    cur.setClusterParent(this);
                    cur.setIcon();
                } else {
                    cur.clearStale();
                }
                // "Network Interfaces"
                cur=findChild(getNlsString("objnameNetInts"));
                if (cur==null) {
                    cur=new TWGMSCSClusterObject(getNlsString("objnameNetInts"),null,null);
                    cur.setClusterParent(this);
                    cur.setIcon();
                } else {
                    cur.clearStale();
                }
                // rebuild my kids
                cnt=mgfcc.getNumberOfElements();
                // loop through what we got back, putting it in its place
                for (i=0;i<cnt;i++) {
                    long    type=mgfcc.getElementType(i);
                    String  ename=mgfcc.getElementName(i);
                    String  statename=null;
                    String  strDesc=null;
                    if (type==MSCSAgentCommand.CLUSTER_ENUM_RESTYPE) {
                        // get resource dll name
                        String  resDLLname=mgfcc.getElementResourceTypeResourceDLL(i);
                        // get description
                        strDesc=mgfcc.getElementResourceTypeDescription(i);

                        UpdateResType(ename, resDLLname, strDesc);

                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_RESOURCE) {
                        // get state info
                        long    estate=mgfcc.getElementResourceState(i);

                        // convert state info into name
                        if (estate==MSCSAgentCommand.ClusterResourceStateUnknown) {
                            statename=getNlsString("stateUnknown");
                        } else if (estate==MSCSAgentCommand.ClusterResourceInherited) {
                            statename=getNlsString("stateInherited");
                        } else if (estate==MSCSAgentCommand.ClusterResourceInitializing) {
                            statename=getNlsString("stateInitializing");
                        } else if (estate==MSCSAgentCommand.ClusterResourceOnline) {
                            statename=getNlsString("stateOnline");
                        } else if (estate==MSCSAgentCommand.ClusterResourceOffline) {
                            statename=getNlsString("stateOffline");
                        } else if (estate==MSCSAgentCommand.ClusterResourceFailed) {
                            statename=getNlsString("stateFailed");
                        } else if (estate==MSCSAgentCommand.ClusterResourcePending) {
                            statename=getNlsString("statePending");
                        } else if (estate==MSCSAgentCommand.ClusterResourceOnlinePending) {
                            statename=getNlsString("stateOnPending");
                        } else if (estate==MSCSAgentCommand.ClusterResourceOfflinePending) {
                            statename=getNlsString("stateOffPending");
                        } else {
                            // don't know this state, mark it unknown
                            statename=getNlsString("stateUnknown");
                        }
                        // should get description here - add later
                        strDesc=mgfcc.getElementResourceDescription(i);
                        // get my owner (node)
                        String strOwner=mgfcc.getElementResourceOwnerNode(i);
                        // get my owner (group)
                        String strGroup=mgfcc.getElementResourceCurrentOwner(i);
                        // get my resource type
                        String strResType=getNLSResTypeStr(mgfcc.getElementResourceType(i));

                        // if the node already exists, just update it
                        TWGMSCSClusterResourceObject cro=(TWGMSCSClusterResourceObject)
                            ((findChild(getNlsString("objnameResources"))).findChild(ename));
                        if (cro==null) {
                            // add the node!
                            cro=new TWGMSCSClusterResourceObject(ename, statename, strDesc,
                                                                 strOwner, strGroup, strResType);
                            cro.setClusterParent(findChild(getNlsString("objnameResources")));
                        } else {
                            // update the node
                            cro.setState(statename);
                            cro.setDescription(strDesc);
                            cro.setOwner(strOwner);
                            cro.setGroup(strGroup);
                            cro.setResType(strResType);
                            cro.clearStale();
                        }
                        cro.setIcon();

                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_GROUP) {
                        // get state info
                        long    estate=mgfcc.getElementGroupState(i);

                        // convert state info into name
                        if (estate==MSCSAgentCommand.ClusterGroupStateUnknown) {
                            statename=getNlsString("stateUnknown");
                        } else if (estate==MSCSAgentCommand.ClusterGroupOnline) {
                            statename=getNlsString("stateOnline");
                        } else if (estate==MSCSAgentCommand.ClusterGroupOffline) {
                            statename=getNlsString("stateOffline");
                        } else if (estate==MSCSAgentCommand.ClusterGroupFailed) {
                            statename=getNlsString("stateFailed");
                        } else if (estate==MSCSAgentCommand.ClusterGroupPartialOnline) {
                            statename=getNlsString("statePartial");
                        } else {
                            // don't know this state, mark it unknown
                            statename=getNlsString("stateUnknown");
                        }
                        // get description
                        strDesc=mgfcc.getElementGroupDescription(i);
                        // get my owner (node)
                        String strOwner=mgfcc.getElementGroupNodeName(i);

                        // if the node already exists, just update it
                        TWGMSCSClusterGroupObject cgo=(TWGMSCSClusterGroupObject)
                            ((findChild(getNlsString("objnameGroups"))).findChild(ename));
                        if (cgo==null) {
                            // add the node!
                            cgo=new TWGMSCSClusterGroupObject(ename, statename, strDesc, strOwner);
                            cgo.setClusterParent(findChild(getNlsString("objnameGroups")));
                        } else {
                            // update the existing node
                            cgo.setState(statename);
                            cgo.setDescription(strDesc);
                            cgo.setOwner(strOwner);
                            cgo.clearStale();
                        }
                        cgo.setIcon();

                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NETWORK) {
                        String  strRole;
                        String  strMask;

                        // get state info
                        long    estate=mgfcc.getElementGroupState(i);

                        // convert state info into name
                        if (estate==MSCSAgentCommand.ClusterNetworkStateUnknown) {
                            statename=getNlsString("stateUnknown");
                        } else if (estate==MSCSAgentCommand.ClusterNetworkUnavailable) {
                            statename=getNlsString("stateUnavailable");
                        } else if (estate==MSCSAgentCommand.ClusterNetworkDown) {
                            statename=getNlsString("stateDown");
                        } else if (estate==MSCSAgentCommand.ClusterNetworkPartitioned) {
                            statename=getNlsString("statePartitioned");
                        } else if (estate==MSCSAgentCommand.ClusterNetworkUp) {
                            statename=getNlsString("stateUp");
                        } else {
                            // don't know this state, mark it unknown
                            statename=getNlsString("stateUnknown");
                        }
                        // get description
                        strDesc=mgfcc.getElementNetworkDescription(i);
                        // get network role
                        long    erole=mgfcc.getElementNetworkRole(i)&0xffff;

                        // convert role info into name
                        if (erole==MSCSAgentCommand.ClusterNetworkRoleNone) {
                            strRole=getNlsString("roleNone");
                        } else if (erole==MSCSAgentCommand.ClusterNetworkRoleInternalUse) {
                            strRole=getNlsString("roleInternal");
                        } else if (erole==MSCSAgentCommand.ClusterNetworkRoleClientAccess) {
                            strRole=getNlsString("roleClient");
                        } else if (erole==MSCSAgentCommand.ClusterNetworkRoleInternalAndClient) {
                            strRole=getNlsString("roleBoth");
                        } else {
                            strRole=getNlsString("stateUnknown");
                        }
                        // get network mask
                        strMask=mgfcc.getElementNetworkAddressMask(i);

                        // if the node already exists, just update it
                        TWGMSCSClusterNetworkObject cno=(TWGMSCSClusterNetworkObject)
                            ((findChild(getNlsString("objnameNetworks"))).findChild(ename));
                        if (cno==null) {
                            // add the node!
                            cno=new TWGMSCSClusterNetworkObject(ename, statename, strDesc, strRole, strMask);
                            cno.setClusterParent(findChild(getNlsString("objnameNetworks")));
                        } else {
                            // update the existing node
                            cno.setState(statename);
                            cno.setDescription(strDesc);
                            cno.setRole(strRole);
                            cno.setMask(strMask);
                            cno.clearStale();
                        }
                        cno.setIcon();

                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NETINTERFACE) {
                        String  strAddr;
                        String  strNetwork;
                        String  strNode;
                        // get state info
                        long    estate=mgfcc.getElementGroupState(i);

                        // convert state info into name
                        if (estate==MSCSAgentCommand.ClusterNetInterfaceStateUnknown) {
                            statename=getNlsString("stateUnknown");
                        } else if (estate==MSCSAgentCommand.ClusterNetInterfaceUnavailable) {
                            statename=getNlsString("stateUnavailable");
                        } else if (estate==MSCSAgentCommand.ClusterNetInterfaceFailed) {
                            statename=getNlsString("stateFailed");
                        } else if (estate==MSCSAgentCommand.ClusterNetInterfaceUnreachable) {
                            statename=getNlsString("stateUnreachable");
                        } else if (estate==MSCSAgentCommand.ClusterNetInterfaceUp) {
                            statename=getNlsString("stateUp");
                        } else {
                            // don't know this state, mark it unknown
                            statename=getNlsString("stateUnknown");
                        }
                        // get description
                        strDesc=mgfcc.getElementNetInterfaceDescription(i);
                        // get address
                        strAddr=mgfcc.getElementNetInterfaceAddress(i);
                        // delay getting network and node until we have the full tree built
                        strNetwork=null;
                        strNode=null;

                        // if the node already exists, just update it!
                        TWGMSCSClusterNetInterfaceObject cnio=(TWGMSCSClusterNetInterfaceObject)
                            ((findChild(getNlsString("objnameNetInts"))).findChild(ename));
                        if (cnio==null) {
                            // add the node!
                            cnio=new TWGMSCSClusterNetInterfaceObject(ename, statename, strDesc,
                                                                 strAddr, strNetwork, strNode);
                            cnio.setClusterParent(findChild(getNlsString("objnameNetInts")));
                        } else {
                            cnio.setState(statename);
                            cnio.setDescription(strDesc);
                            cnio.setAddress(strAddr);
                            cnio.setNetwork(strNetwork);
                            cnio.setNode(strNode);
                            cnio.clearStale();
                        }
                        cnio.setIcon();

//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NODE) {
//                        // do nothing this pass
//                        
                    } else {
                        // if its not one of the above types, ignore it
                    }
                }

                // loop through a second time, this time to make associations we couldn't
                // do in the first pass
                boolean doneGroups=false;
                for (i=0;i<cnt;i++) {
                    TWGMSCSClusterObject    tfolder;
                    long                    type=mgfcc.getElementType(i);
                    String                  ename=mgfcc.getElementName(i);
                    if (type==MSCSAgentCommand.CLUSTER_ENUM_GROUP) {
                        // we do all the groups at once, so skip any additional times
                        if (doneGroups) {
                            continue;
                        } else {
                            doneGroups=true;
                        }
                        TWGMSCSClusterObject resFolder=findChild(getNlsString("objnameResources"));
                        // need to add subservient resources
                        tfolder=findChild(getNlsString("objnameGroups"));

                        // loop through the groups, and assign their resources to them
                        TWGMSCSClusterGroupObject cgo=(TWGMSCSClusterGroupObject)tfolder.getChild();
                        if (cgo != null) {
                            do {
                                TWGMSCSClusterResourceObject cro=
                                    (TWGMSCSClusterResourceObject)resFolder.getChild();
                                if (cro != null) {
                                    do {
                                        if (cro.getGroup().equals(cgo.getName()) && !cro.getStale()) {
                                            TWGMSCSClusterResourceObject ncro=
                                                (TWGMSCSClusterResourceObject)cgo.findChild(cro.getName());
                                            if (ncro==null) {
                                                ncro=new TWGMSCSClusterResourceObject(cro.getName(), cro.getState(),
                                                                                 cro.getDescription(), cro.getOwner(),
                                                                                 cro.getGroup(), cro.getResType());
                                                ncro.setClusterParent(cgo);
                                            } else {
                                                ncro.setState(cro.getState());
                                                ncro.setDescription(cro.getDescription());
                                                ncro.setOwner(cro.getOwner());
                                                ncro.setGroup(cro.getGroup());
                                                ncro.setResType(cro.getResType());
                                                ncro.clearStale();
                                            }
                                            ncro.setIcon();
                                        }
                                        cro=(TWGMSCSClusterResourceObject)cro.getSibling();
                                    } while ((cro!=null) && (cro!=resFolder.getChild()));
                                }
                                cgo=(TWGMSCSClusterGroupObject)cgo.getSibling();
                            } while ((cgo!=null) && (cgo!=tfolder.getChild()));
                        }

                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NETWORK) {
                        // find the network interfaces that this network owns, and mark them
                        // to know that they belong to this network.  (don't copy them local
                        // yet, they're not necessarily done yet)
                        tfolder=findChild(getNlsString("objnameNetInts"));
                        String slist[] = mgfcc.getElementNetworkNetworkInterfaceList(i);
                        for (j=0;j<slist.length;j++) {
                            TWGMSCSClusterNetInterfaceObject cnio=
                                (TWGMSCSClusterNetInterfaceObject)tfolder.findChild(slist[j]);
                            if (cnio !=null ) {
                                cnio.setNetwork(ename);
                                cnio.clearStale();
                            }
                        }

//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NODE) {
//                        // nothing to do this pass, skip it
//                    
//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_RESTYPE) {
//                        // nothing to do this pass, skip it
//                    
//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_RESOURCE) {
//                        // nothing to do this pass, skip it
//                    
//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NETINTERFACE) {
//                        // need to update node and network, but these will be handled for
//                        // me by the node and network code, respectively, so skip it!
//                    
                    } else {
                        // if its not one of the above types, ignore it
                    }
                }

                // only do nets once in this pass
                boolean doneNets=false;
                // loop through a third time to do the final associations (network interfaces)
                for (i=0;i<cnt;i++) {
                    TWGMSCSClusterObject    tfolder;
                    long                    type=mgfcc.getElementType(i);
                    String                  ename=mgfcc.getElementName(i);
                    if (type==MSCSAgentCommand.CLUSTER_ENUM_NODE) {
                        // make the node!
                        UpdateNode(ename, mgfcc.getElementNodeState(i),
                                   mgfcc.getElementNodeDescription(i),
                                   mgfcc.getElementNodeNetworkInterfaceList(i));

                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NETWORK) {
                        // we do all the networks at once (this phase), so skip any additional times
                        if (doneNets) {
                            continue;
                        } else {
                            doneNets=true;
                        }
                        TWGMSCSClusterObject niFolder=findChild(getNlsString("objnameNetInts"));
                        // need to add subservient net interfaces
                        tfolder=findChild(getNlsString("objnameNetworks"));

                        // loop through the networks, and assign their interfaces to them
                        TWGMSCSClusterNetworkObject cno=(TWGMSCSClusterNetworkObject)tfolder.getChild();
                        if (cno != null) {
                            do {
                                TWGMSCSClusterNetInterfaceObject cnio=
                                    (TWGMSCSClusterNetInterfaceObject)niFolder.getChild();
                                if (cnio != null) {
                                    do {
                                        if (cnio.getNetwork().equals(cno.getName()) && !cnio.getStale()) {
                                            TWGMSCSClusterNetInterfaceObject ncnio=
                                                (TWGMSCSClusterNetInterfaceObject)cno.findChild(cnio.getName());
                                            if (ncnio==null) {
                                                ncnio=new TWGMSCSClusterNetInterfaceObject(cnio.getName(), cnio.getState(),
                                                                                 cnio.getDescription(), cnio.getAddress(),
                                                                                 cnio.getNetwork(), cnio.getNode());
                                                ncnio.setClusterParent(cno);
                                            } else {
                                                ncnio.setState(cnio.getState());
                                                ncnio.setDescription(cnio.getDescription());
                                                ncnio.setAddress(cnio.getAddress());
                                                ncnio.setNetwork(cnio.getNetwork());
                                                ncnio.setNode(cnio.getNode());
                                                ncnio.clearStale();
                                            }
                                            ncnio.setIcon();
                                        }
                                        cnio=(TWGMSCSClusterNetInterfaceObject)cnio.getSibling();
                                    } while ((cnio!=null) && (cnio!=niFolder.getChild()));
                                }
                                cno=(TWGMSCSClusterNetworkObject)cno.getSibling();
                            } while ((cno!=null) && (cno!=tfolder.getChild()));
                        }

//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_GROUP) {
//                        // nothing to do this pass, skip it
//
//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_RESTYPE) {
//                        // nothing to do this pass, skip it
//
//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_RESOURCE) {
//                        // nothing to do this pass, skip it
//
//                    } else if (type==MSCSAgentCommand.CLUSTER_ENUM_NETINTERFACE) {
//                        // need to update node and network, but these will be handled for
//                        // me by the node and network code, respectively, so skip it!
//
                    } else {
                        // if its not one of the above types, ignore it
                    }
                }

                // all done, get rid of the extra ref
                cur=null;

                // now, clear all the stale parts of the tree
                deleteStaleRecursive();
            } else {
                // bad rc, mark in the status bar
            }
            // release this command, so we can send other ones in the future
            mgfcc=null;

            // if we know which tree owns us, tell it to update itself
            if (treeowner!=null) {
                treeowner.completeUpdate(this);
            }
        }
    }

    public  void    CommandComplete(Command cmd) {
        if (cmd instanceof MSCSGetFullConfigCommand) {
            // need to be in the gui thread to update the tree
            SwingUtilities.invokeLater(this);
        } else if ((cmd instanceof MSCSAgentCommand) && 
                   (cmd.CommandCode()==MSCSAgentCommand.MSCSAGT_GET_NOTIFIES)) {
            // we got a notify update!
            long             newLevel;
            MSCSAgentCommand mac=(MSCSAgentCommand)cmd;

            if (mac.ReturnCode() == 0) {
                newLevel=mac.readDWORDOutputParam(0,0);
                // see if there's later data than we currently have
                if (newLevel>updateLevel) {
                    // send a refresh command, if there's not already one outstanding
                    if (mgfcc==null) {
                        mgfcc=new MSCSGetFullConfigCommand(destAddr);
                        mgfcc.setCommandCompleteListener(this);

                        // Send command
                        try{
                            sn.SendAsynchCommand(mgfcc);
                        }
                        catch(InterruptedException x){
                            mgfcc=null;
                        }
                        catch(ServiceNodeClosedException x){
                            mgfcc=null;
                        }
                        catch(ServiceNodeException x){
                            mgfcc=null;
                        }
                    }
                    // dispatch an update command, and update the "current level"
                    updateLevel=newLevel;
                }
            }
            // need to give the base another one to act as an outstanding command
            mac=new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_GET_NOTIFIES, destAddr);
            mac.addDWORDInputParam(updateLevel);
            mac.setCommandCompleteListener(this);
            // Send command
            try{
                sn.SendAsynchCommand(mac);
            }
            catch(InterruptedException x){
            }
            catch(ServiceNodeClosedException x){
            }
            catch(ServiceNodeException x){
            }
        }
    }

    public  String  getIconName() {
        return (new String("/com/tivoli/twg/engine/images/clus16on.gif"));
    }

    public String getNLSResTypeStr(String erestype) {
        String nlsName;

        if (erestype.equals("Physical Disk")) {
            nlsName=getNlsString("rtypePhysDisk");
        } else if (erestype.equals("Distributed Transaction Coordinator")) {
            nlsName=getNlsString("rtypeDTC");
        } else if (erestype.equals("File Share")) {
            nlsName=getNlsString("rtypeFileShare");
        } else if (erestype.equals("Generic Application")) {
            nlsName=getNlsString("rtypeGenApp");
        } else if (erestype.equals("Generic Service")) {
            nlsName=getNlsString("rtypeGenSrv");
        } else if (erestype.equals("IIS Virtual Root")) {
            nlsName=getNlsString("rtypeIISRoot");
        } else if (erestype.equals("IP Address")) {
            nlsName=getNlsString("rtypeIPAddr");
        } else if (erestype.equals("Microsoft Message Queue Server")) {
            nlsName=getNlsString("rtypeMMQS");
        } else if (erestype.equals("Network Name")) {
            nlsName=getNlsString("rtypeNetName");
        } else if (erestype.equals("Print Spooler")) {
            nlsName=getNlsString("rtypePrtSpool");
        } else if (erestype.equals("Time Service")) {
            nlsName=getNlsString("rtypeTimeSrv");
        } else {
            nlsName=erestype;
        }
        return nlsName;
    }

    public int      getPopupItemCount() {
        if (super.getPopupItemCount()>0) {
            return super.getPopupItemCount()+2;
        } else {
            return 1;
        }
    }

    public JMenuItem getPopupItem(int index) {
        switch (index) {
            case 0:
                if (fPopupRename==null) {
                    fPopupRename=new JMenuItem(getNlsString("miRename"));
                    fPopupRename.setActionCommand("rename");
                    fPopupRename.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupRename.setEnabled(false);
                    }
                }
                return fPopupRename;
            case 1:
                // return null to have the adder add a separator instead
                return null;
            default:
                break;
        }
        if (index<super.getPopupItemCount()+2) {
            return super.getPopupItem(index-2);
        } else {
            // don't know what to add, so add a separator, I guess
            return null;
        }
    }

    public void actionPerformed( ActionEvent evt ) {
        // Place menu and/or button handling code here.
        // Remove listener interface if neither of these are required.
        //         ....................
        String                   cmd = evt.getActionCommand();
        TWGMSCSClusterObjectWithServiceNode  owsn = null;
        ServiceNode                          sn = null;
        MSCSAgentCommand                     agtcmd = null;

        owsn=getParentWithServiceNode();
        if (owsn!=null) {
            sn=owsn.getServiceNode();
        }


        if (cmd.equals("rename")) {
            // rename the cluster
            String newname;

            newname=JOptionPane.showInputDialog(null,
						getNlsString("renameNewName"),
                                                getNlsString("renameTitle"),
                                                JOptionPane.QUESTION_MESSAGE);
	    if (newname != null) {
		if (!isNameValid(newname)) {
		    Object[] badname = {getName(), newname};
		    JOptionPane.showMessageDialog(null,
						  MessageFormat.format(getNlsString("errNameBad"),badname),
						  getNlsString("TWGMSCSBrowseFrameTitle"),
						  JOptionPane.ERROR_MESSAGE);
		} else {
		    Object[] nameargs = {getName(), newname};
		    // verify that they Really want to rename the cluster
		    if (JOptionPane.showConfirmDialog(null,
						      MessageFormat.format(getNlsString("areYouSureRename"),nameargs),
						      getNlsString("TWGMSCSBrowseFrameTitle"),
						      JOptionPane.YES_NO_OPTION,
						      JOptionPane.QUESTION_MESSAGE) ==
                        JOptionPane.YES_OPTION) {
			// they said yes, do the rename
			agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_SET_CLUSTER_NAME, owsn.GetMyDestination());
			agtcmd.addCompUnicodeInputParam(newname);
			if (sn != null) {
			    // Send command to server
			    try{
				sn.SendAsynchCommand(agtcmd);
			    }
			    catch(InterruptedException x){
				return;
			    } 
			    catch(ServiceNodeClosedException x){
				return;
			    }
			    catch(ServiceNodeException x){
				return;
			    }
			}
		    }
		}
	    }

        } else {
            super.actionPerformed(evt);
        }
    } // end actionPerformed
}

