package com.BobCo.rmon;

import com.tivoli.twg.snmp.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.*;
import java.util.*;
import java.net.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.snmp.*;

/**
 * This class represents a base level SNMP Device.  This is the class that all other
 * TWGSNMPxxxx device types are derived from.  It represents the existance of an SNMP
 * agent at a particular IP address, and the existance of at least part of MIB II.
 *
 * @author  Paul A. Ruocchio
 * @version 1.00, 09/01/97
 */
public class TWGRMONDevice extends TWGSNMPDevice {

/* Our classname                                                        */
private static final String CLASSNAME =
    "com.BobCo.rmon.TWGRMONDevice";

private static final int CLASS_VERSION = 1;

   /**
     * Construct A TWGSNMPDevice instance into the system. This usually occurs as the
     * result of information obtained from the IP/IPX discovery process.
     *
     * @param   Address  An IP address, in string format.
     * @param   Context  An SNMP V1 Community name.
     * @param   SubNet   The subnet on which this device exists.
     */
   public TWGRMONDevice(String Address,String Context,int SubNet){
      super(Address,Context,SubNet);
   }


   /**
     * Construct A TWGSNMPDevice instance into the system. This will occur ONLY at system
     * restart, and will be used to rebuild objects we already knew about.
     *
     */
   public TWGRMONDevice(){
      super();              // call the constructor for the base class
   }

   /**
    * Copy instance data to another managed object.
    * @param managedObject target managed object.
    */
   protected void copyTo(TWGManagedObject managedObject){
      super.copyTo(managedObject);
      if(managedObject instanceof TWGRMONDevice){

         //PAR if we had some data, this is the place to copy it

      }
   }

   protected static int GetRMONDeviceID(String Address){
      return GetSNMPDeviceID(Address);
   }

   protected static TWGRMONDevice GetRMONDevice(String Address){
      return (TWGRMONDevice)GetSNMPDevice(Address);
   }

   public static TWGRMONDevice GetRMONDeviceFromID(long ID){
      return (TWGRMONDevice)GetDeviceFromID(ID);
   }

   /**
    * Method for saving persistent object data.
    *
    * @param TWGPersistentObjectDictionary used to collect persistent object.
    * @exception TWGPersistentObjectSaveException thrown to abort save procedure.
    */
   protected void saveData(TWGPersistentObjectDictionary dictionary ) throws
       TWGPersistentObjectSaveException {
       super.saveData(dictionary);

       TWGByteArrayOutputStream Buffer = new TWGByteArrayOutputStream();

       Buffer.writeInt(CLASS_VERSION);

       // Add record to dictionary
       dictionary.put(CLASSNAME,Buffer.toByteArray());
   }


   /**
    * Method for restoring persistent object data.
    *
    * @param TWGPersistentObjectDictionary used to restore persistent object.
    * @param resolveObjectReferences used to indicate if object references should be
    *   resolved during restore.
    * @exception TWGPersistentObjectRestoreException thrown to abort restore procedure.
    */
   protected void restoreData(TWGPersistentObjectDictionary dictionary,
       boolean resolveObjectReferences) throws
       TWGPersistentObjectRestoreException {

       /* Restore base class                                               */
       super.restoreData(dictionary, resolveObjectReferences);

       if(resolveObjectReferences == false){
          // now let's go get our stuff here
          TWGByteArrayInputStream Buffer = new TWGByteArrayInputStream(dictionary.get(CLASSNAME));

          int VerTest = Buffer.readInt();    // get the version here first
          if(VerTest != CLASS_VERSION){      // OK, we know how to deal with this one
             TWGRas.entryExit(TWGRas.SNMP,"RMON Device Restore failed - Invalid TWGSNMPDevice Class Version");
             throw new TWGPersistentObjectRestoreException();
          }
       }
       else{  // second pass, now we can resolve the ID stuff
          TWGRas.entryExit(TWGRas.SNMP,"RMON Device Restored: " + GetDeviceID() + " Address: " + GetAddress());
       }
   }

   /**
    * Destructor for TWGNativeManagedObject: remove object from lists
    */
   public void Delete() {
       super.Delete();
   }
   /**
    * Destroy for TWGNativeManagedObject: send delete to NetMgr
    * @exception TWGObjectDestroyException if error during destroy
    */
   public void Destroy() throws TWGObjectDestroyException {
       super.Destroy();

       // we don't need to do anything here...

   }

}

