
package com.BobCo.rmon;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.alertmgr.TWGEventServer;
import java.util.*;
import java.net.*;
import com.tivoli.twg.snmp.*;

/**
 * Base class for representing the managed object factory for
 * Tivoli Workgroup RMON clients.
 *
 * @author Paul A. Ruocchio
 * @version 0.01 - 10/06/97
 */
public class TWGRMONDeviceObjectFactory extends TWGManagedObjectFactory {

    /* Our classname                                                        */
    private static final String CLASSNAME =
    "com.tivoli.twg.rmon.TWGRMONDeviceObjectFactory";
    /** Name of TWGManagedObject class supportd by factory          */
    public static final String MO_CLASS =
        "com.tivoli.twg.rmon.TWGRMONDevice";

    private static final int CLASS_VERSION = 1;

    /* Name of our address entry shadow class                               */
    private static final String ADDRESSSHADOWCLASS =
        "com.tivoli.twg.rmon.TWGRMONAddressEntryShadow";

    /* Name of our address entry shadow class                               */
    private static final String ADDRESSNLSCLASS =
        "com.tivoli.twg.rmon.TWGRMONResources";

    /** Name ID for our native managed objects                              */
    private static final String MONAME = "rmonmof.name";

    /* Reference to TWGNativeManagedObjectFactory instance          */
    private static TWGRMONDeviceObjectFactory factory = null;
    /* Flag indicating if we're active                              */
    private boolean is_active;
    /* Current version stamp                                        */
    private int current_version;

    /* image set used for this type of device */
    protected static final String[] IMAGE_SET = {
        "/com/tivoli/twg/rmon/images/rmonof32.gif",
        "/com/tivoli/twg/rmon/images/rmonon32.gif",
        "/com/tivoli/twg/rmon/images/rmonof32.gif",
        "/com/tivoli/twg/rmon/images/rmonon32.gif",
        "/com/tivoli/twg/rmon/images/rmonof32.gif",
        "/com/tivoli/twg/rmon/images/rmonon32.gif",
        "/com/tivoli/twg/rmon/images/rmonof16.gif",
        "/com/tivoli/twg/rmon/images/rmonon16.gif",
        "/com/tivoli/twg/rmon/images/rmonof16.gif",
        "/com/tivoli/twg/rmon/images/rmonon16.gif",
        "/com/tivoli/twg/rmon/images/rmonof16.gif",
        "/com/tivoli/twg/rmon/images/rmonon16.gif"
    };


    protected static final String[] FILTER_SET = {
       "/com/tivoli/twg/rmon/images/rmondf16.gif",
        "/com/tivoli/twg/rmon/images/rmondf32.gif"
    };

    /**
     * Default constructor
     * @exception ClassNotFoundException if MO class not loaded
     */
    public TWGRMONDeviceObjectFactory() throws ClassNotFoundException {
        this(MO_CLASS);
    }

    /**
     * Constructor for creating a subclass of this factory.
     * @param m_o_cname - TWGManagedObject classname associated with factory
     * @exception ClassNotFoundException if MO class not loaded
     */
    public TWGRMONDeviceObjectFactory(String m_o_cname) throws ClassNotFoundException {
        super(m_o_cname);
        factory = this;
        is_active = false;
        current_version = 0;
        setPingType(TWGPing.PING_TYPE_SERVER_DRIVES_INDIVIDUAL);
        setMorphInterval( 8 * 60 * 60 * 1000 ); /* 8 hrs */
        setAddressEntryShadow(ADDRESSSHADOWCLASS);      /* Set addr shadow  */
        try{
           TWGImageSet Temp = TWGImageSet.FindImageSet(TWGRMONDeviceObjectFactory.IMAGE_SET);
           TWGImageSet Temp2 = TWGImageSet.FindImageSet(TWGRMONDeviceObjectFactory.FILTER_SET);
           try{
              setImageSet((int)Temp.ObjectID());
              setFilterImageSet((long)Temp2.ObjectID());
           }
           catch(InvalidObjectIDException Err){}
        }
        catch(TWGPersistentObjectSaveException Err){}
        setName(MONAME,ADDRESSNLSCLASS);
    }


    /**
     * Get instance of native managed object factory
     * @return TWGRMONDeviceObjectFactory instance
     */
    public static TWGRMONDeviceObjectFactory GetFactory() {
        return factory;
    }

    /**
     * Activate the topology processing.
     * @exception ManagedObjectFactoryActivateException if error
     */
    public void Activate() throws ManagedObjectFactoryActivateException {
       if(!is_active) {
          is_active = true;
       }
    }

    /**
     * Deactivate the topology processing
     */
    public void Deactivate() {
        if(is_active) is_active = false;
    }

    /**
     * Start a discovery pass: operation should return quickly, with
     * actual discovery logic not being completed before the method
     * returns.  The default method does nothing.
     * @exception ManagedObjectFactoryDiscoverException if error starting
     *   discovery pass
     */
    public void StartDiscoveryPass() throws ManagedObjectFactoryDiscoverException {
    }


    /**
     * Test a managed object instance for inclusion into a managed object
     * factory class. This method is used during managed object morphing.
     * It is used by TWGEngine processes to determine the lowest level
     * managed object factory class to which a managed object instace belongs.
     * This method should bve inplemented by all managed object factory
     * extension classes.
     * @param mofInstanceOf Reference to TWGMofInstanceOf callback interface object.
     * @param mo Reference to managed object instance to be tested.
     * @return byte see TWGManagedObjectFactory.
     */
    public byte instanceOf( TWGMofInstanceOf mofInstanceOf, TWGManagedObject mo ) {
       new TWGRMONTest(mofInstanceOf,(TWGSNMPDevice)mo);
       return TWGManagedObjectFactory.INSTANCE_OF_CALLBACK;  // we'll get back to you
    }

    /**
     * Address entry record support method.  This method generates a single
     * record encoding to be interpreted by TWGNativeAddressEntry.  The
     * record simply consists of the compressed unicode encoding of the
     * current best IPC path.
     * should be subclassed by MOF subclasses in order to provide support
     * for encoding address records.  Each address record must consist of
     * a list of records (1 or more), ordered from superclass to subclass,
     * with each record consisting of an overall record length (SHORT) and
     * a magic number (SHORT), followed by class-specific data.
     * @param obj - Managed object to request address encoding for
     * @return byte array containing encoded address record, or null no
     * support
     */
    protected byte[] getOurAddressEntryRecord(TWGManagedObject obj) {

        TWGRMONDevice ThisDevice = (TWGRMONDevice)obj;  // type it to make stuff easier
        return ThisDevice.getOurAddressEntryRecord();
    }

}

