/*
 * Created on Oct 7, 2003
 *
 * 
 */
package com.ibm.sysmgt.sdk.remote.test;

import java.util.List;
import java.util.Locale;
import java.util.Map;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.ibm.sysmgt.sdk.remote.Attribute;
import com.ibm.sysmgt.sdk.remote.DirectorEvent;
import com.ibm.sysmgt.sdk.remote.DirectorEventLog;
import com.ibm.sysmgt.sdk.remote.DirectorRemoteSdk;
import com.ibm.sysmgt.sdk.remote.DirectorServer;
import com.ibm.sysmgt.sdk.remote.Log;
import com.ibm.sysmgt.sdk.remote.LoginSession;
import com.ibm.sysmgt.sdk.remote.ManagedGroup;
import com.ibm.sysmgt.sdk.remote.ManagedSystem;
import com.ibm.sysmgt.sdk.remote.Task;
import com.ibm.sysmgt.sdk.remote.UnknownManagerException;
import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.TWGEnvironment;
import com.tivoli.twg.libs.TWGLong;

/**
 * 
 * 
 * Class to test the DirectorRemoteSdk
 */
public class TestDirectorRemoteSdk extends SetupToUseDirectorServer {
	public void testGetSystemByOid() {
		try {

			List systemlist = ManagedSystem.getAllSystems(s);
			Assert.assertTrue(systemlist.size() != 0);

			ManagedSystem firstSystem = (ManagedSystem) systemlist.get(0);
			long oid = firstSystem.getOid();

			ManagedSystem foundSystem = ManagedSystem.getSystemByOid(s, oid);

			Assert.assertEquals(firstSystem.getOid(), foundSystem.getOid());
			Assert.assertEquals(firstSystem.getLabel(), foundSystem.getLabel());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testGetByUniqueID() {
		try {

			if (s == null)
				fail();

			List systemlist = ManagedSystem.getAllSystems(s);
			if (systemlist.size() == 0)
				fail();

			boolean foundSytemWithUID = false;
			DataValue dvUid = null;

			int i = 0;
			while (!foundSytemWithUID && (i < systemlist.size())) {
				ManagedSystem system = (ManagedSystem) systemlist.get(i);
				Map attributeList = system.getSystemAttributes(s);
				println(
					"Attributes for system '"
						+ system.getLabel()
						+ "': "
						+ attributeList);

				Object obj = attributeList.get("NativeMO.uniqueid");
				println("NativeMO.uniqueid=" + obj);
				Attribute uidAttrib = (Attribute) obj;
				dvUid = (DataValue) uidAttrib.getValue();

				if (dvUid != null) {
					println("Found a UID Datavalue ");
					println(
						"UID Datavalue class = "
							+ dvUid.getClass().getName()
							+ " Found a UID Datavalue value="
							+ dvUid.toString());
					foundSytemWithUID = true;

				}

				i++;
			}

			Assert.assertTrue("no native systems", dvUid != null);

			TWGLong twgLong = (TWGLong) dvUid;
			long uid = twgLong.getValue();
			if (dvUid.getDataValueType() == DataValue.LONG_TYPE) {
				println("Uid type is long");

			} else {
				fail("UID type is not long");
			}

			List foundList = ManagedSystem.getSystemsByUniqueID(s, uid);
			Assert.assertTrue(foundList.size() > 0);

			println("Found by UID: " + foundList);
			ManagedSystem foundSystem = (ManagedSystem) foundList.get(0);
			
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

	}



	public void testNavigation() {
		try {

			/* 
			 * This is a test of how to navigate through 
			 * Director Framework Objects using the RemoteSdk
			 * 
			 * The first step is to get access to the Manager.
			 * The DirectorServer object represents the Director Server.
			 */
			DirectorServer mgr = null;
			mgr = DirectorRemoteSdk.getManager();

			/* 
			 * The next step is to create a session to the DirectorServer
			 * The Session is currently using ServiceNodes with
			 * the LocalImplFactory. The plan is to allow for 
			 * customization to select from three options for 
			 * sessions:
			 * 	LocalImplFactory - same machine but several simultaneous
			 * 		Sessions (different users, different Locales). 
			 *  RemoteImplFactory - remote machine but only one userid/Locale
			 *  Using the existing factory - possibly for Director Console 
			 * 		programming or existing separate JVMs. 
			 */
			LoginSession s =
				mgr.createLoginSession(
					"dirconsole",
					"asdf",
					Locale.getDefault());
			if (s == null)
				fail();
			println("Session: " + s);

			/* 
			 * There are RemoteSdk objects to represent each 
			 * of the main Director Framework objects including:
			 * 	Groups
			 * 	Systems
			 * 	Events
			 * The RemoteSdk objects are named as the user
			 * sees the objects from the Director Console.
			 * The goal is to minimize the learning curve for 
			 * RemoteSdk programmers who have never written a 
			 * Director Extension.  The idea is that if someone 
			 * has used the Director Console, then they will 
			 * easily understand this programming interface. 
			 * 
			 * The methods on these objects will return 
			 * Java types as opposed to Director internal types 
			 * whenever possible.  For instance,
			 * A List is returned instead of a LongValueSet.
			 * 
			 * The API concept is still important, but the basic
			 * Framework functions do not require an API.
			 * 
			 * */
			List groupList = ManagedGroup.getAllGroups(s);
			println("List of Groups: " + groupList);

			ManagedGroup firstGroup = (ManagedGroup) groupList.get(0);

			List systemlist = firstGroup.getChildSystems(s);
			println(
				"List of Systems in  '"
					+ firstGroup.getLabel()
					+ "': "
					+ systemlist);

			/* 
			 * ManagedSystem is the representation of 
			 * a Director TWGManagedObject, but on the Director 
			 * Console, they are displayed as "systems".
			 */
			ManagedSystem firstSystem = (ManagedSystem) systemlist.get(0);

			Map attributeList = firstSystem.getSystemAttributes(s);
			println(
				"Attributes for system '"
					+ firstSystem.getLabel()
					+ "': "
					+ attributeList);

			/* 
			 * Static methods in the RemoteSdk classes return specific 
			 * instances of objects, like the Event Log 
			 * 
			 */
			DirectorEventLog evlog = (DirectorEventLog) Log.getLog("EventLog");
			List eventList = evlog.getAllEvents(s);

			println(
				"Events for system '"
					+ firstSystem.getLabel()
					+ "': "
					+ eventList);

			if (eventList.size() > 0) {
				DirectorEvent dirEvent = (DirectorEvent) eventList.get(0);

				Map eventDetails = dirEvent.getDetails(s);
				println(
					"Events Details for Event'"
						+ dirEvent.getEventType()
						+ "': "
						+ eventDetails);

			}
			List taskList = Task.getTasksForSystem(s, firstSystem);
			println(
				"Tasks for system '"
					+ firstSystem.getLabel()
					+ "': "
					+ taskList);

			/* 
			 * The session must be cleaned up once the RemoteSdk
			 * client is finished with it. 
			 */
			s.endSession();
		} catch (UnknownManagerException ume) {
			println("UnknownManagerException=" + ume);
			ume.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

	}

	public void testconnection_to_localhost() {
		DirectorServer mgr = null;

		try {
			mgr = DirectorRemoteSdk.getManager("localhost");
		} catch (UnknownManagerException ume) {
			println("UnknownManagerException=" + ume);
			ume.printStackTrace();
		}

		try {
			LoginSession s =
				mgr.createLoginSession(
					"dirconsole",
					"asdf",
					Locale.getDefault());
			if (s == null)
				fail();
			s.endSession();
		} catch (Exception e) {
			fail();
		}

	}


	/**
	* Constructor for RemoteSdkTest.
	* @param arg0
	*/
	public TestDirectorRemoteSdk(String arg0) {
		super(arg0);
		//println("TestDirectorRemoteSdk(" + arg0 + ")");
	}

}
