/* file:  monitors\com\apitest.cpp */
// Licensed Materials - Property of IBM
//
// OCO Source Materials
// 
// 09N6996
//
// (C) Copyright IBM Corp. 1999, 2002 All Rights Reserved
//
// The source code for this program is not published or other-
// wise divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
#include "sys_inc.hpp"        // include required system files
#include <stdio.h>
#include "apidll.hpp"
#include "dll.hpp"
#include "string.hpp"


#if defined(__NETWARE_386__) // NetWare ======================================

   #include "dosclls.hpp"

   extern NewHandler handler;
   int               ThreadID;      // main thread id
   int               cUsers;        // count of NLMS using this NLM

   /* initialization code - the NLM needs a initialization routine to
    * return its custom data to the library code. so that the resources
    * can be picked up transparently
    */
   extern "C"
   {
      extern NLMInitData   __NLMInitData;
      NLMInitData*         APIENTRY  APITEST_NLMInit();
      void                 APIENTRY  APITEST_NLMTerm();
   };

   extern "C" void Cleanup()
   {
      // call our shutdown routine to cleanup
      handler.ReleaseMemory();
   }

   NLMInitData* APIENTRY APITEST_NLMInit()
   {
      __NLMInitData.closeFile = 1;
      return &__NLMInitData;
   }

   void APIENTRY APITEST_NLMTerm()
   {
   }

#endif

//=======================
// Main Application Code
//=======================
#if defined(__NETWARE_386__)

void main()
{
   atexit( Cleanup );
   DOSCLLS_InitLibrary();
   ThreadID = GetThreadID();
   // SuspendThread( ThreadID );

#else

int __cdecl main( int argc, char *argv[ ], char *envp[ ])
{

#endif

   long              aSize, totalSize;
   TWGMonAPIDLL      *monapi = NULL;
   AttributePath     *pAPath = NULL;
   AttributeNodeList *pANList = NULL;
   AttributeNode     *pANode = NULL;
   MonHandle         *mHandle = NULL;
   MonMetaData       *mMeta = NULL;
   MonData           *mData = NULL;
   Str               lang(Str::GetISOLanguage());  // Java-style locale
   Str               str1("MONCPU");
   Str               str2("2872344980");           // CPU Utilization
   long              dataRate = 5000;

   // init vars
   lang.LocalToCompUnicode();
   str1.LocalToCompUnicode();
   str2.LocalToCompUnicode();
   aSize = sizeof(AttributePath);


   printf("\nAPITest for IT Director Resource Monitor Agent\n");

   // create our API interface
   #ifdef __NETWARE_386__
   // monapi = new TWGMonAPIDLL("SYS:TIVOLIWG\\twgmonap.nlm");
   monapi = new TWGMonAPIDLL("TWGMONAP.NLM");
   #else
   monapi = new TWGMonAPIDLL("twgmonap");
   #endif
   if (monapi == NULL)
   {
      printf("Could not create our MONAPI interface!\n");
      flushall();
#ifdef __NETWARE_386__
      exit(-1);
#else
      return(-1);
#endif
   }
   else if (monapi->isOK() == FALSE)
   {
      printf("Did not succesfully create our MONAPI interface!\n");
#ifdef __NETWARE_386__
      exit(-2);
#else
      return(-2);
#endif
   }
   else
   {
      printf("TWGMonAPIDLL wrapper created\n");
      monapi->logDebug("APITEST - TWGMonAPIDLL wrapper created");
   }

   //---------------------------------
   // TWGMonAPIGetNodeList for MONCPU
   //---------------------------------
   // determine the size of the path
   totalSize  = aSize;
   totalSize += (str1.Length() + 1);
   if (pAPath != NULL)
   {
      delete [] (BYTE *)pAPath;
      pAPath = NULL;
   }
   // create the attribute path to ask for
   pAPath = (AttributePath *) new BYTE[totalSize];
   if (pAPath == NULL)
   {
#ifdef __NETWARE_386__
      exit(RC_MEMORY_FAILURE);
#else
      return RC_MEMORY_FAILURE;
#endif
   }
   memset( pAPath, 0, totalSize);
   pAPath->size = totalSize;
   pAPath->version = PATH_VERSION_1;
   strcpy(pAPath->lang, (PCHAR)lang);
   pAPath->members = 1;
   pAPath->offsets[0] = aSize;
   strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[0]), (PCHAR)str1);
   //-------------------------------
   // get a the nodelist for MONCPU
   //-------------------------------
   printf("Getting the NodeList for MONCPU\n");
   pANList = monapi->getNodeList(pAPath);
   if (pANList != NULL)
   {
      if (pANList->rc == RC_OK)
      {
         Str   nodeLang("");
         Str   guiName("");
         Str   symName("");
         Str   nodeType("");
         printf("\tReceived a valid NodeList with members=%d\n", pANList->members);
         for (int j=1; j <= pANList->members; j++)
         {
            if (pANode == NULL)
            {
               // FirstNode
               pANode = ((AttributeNode MONFAR *)((PCHAR)pANList + sizeof(AttributeNodeList)));
            }
            else
            {
               // NextNode
               pANode = ((AttributeNode MONFAR *)((PCHAR)pANode + pANode->size));
            }
            nodeLang = pANode->lang;
            nodeLang.CompUnicodeToLocal();
            guiName = (char *)(((char *)pANode)+pANode->nameOffset);
            guiName.CompUnicodeToLocal();
            symName = (char *)(((char *)pANode)+pANode->symNameOffset);
            symName.CompUnicodeToLocal();
            if (pANode->type == DOUBLE_NODEDATA)
            {
               nodeType = "DOUBLE_NODEDATA";
            }
            else if (pANode->type == STRING_NODEDATA)
            {
               nodeType = "STRING_NODEDATA";
            }
            else
            {
               nodeType = "INVALID_NODEDATA";
            }
            printf("\t\tNode %d\n", j);
            printf("\t\t        - GUI Name      = %s\n", (char *)guiName);
            printf("\t\t        - GUI Locale    = %s\n", (char *)nodeLang);
            printf("\t\t        - Symbolic Name = %s\n", (char *)symName);
            printf("\t\t        - isLeaf        = %s\n", (BOOL)(pANode->isLeaf)?"TRUE":"FALSE");
            printf("\t\t        - NodeData Type = %s\n", (char *)nodeType);
         }
      }
      else
      {
         printf("\tReceived an invalid NodeList\n");
      }
      //------------------------------
      // free the nodelist for MONCPU
      //------------------------------
      printf("Freeing NodeList\n");
      monapi->freeNodeList(pANList);
      pANList = NULL;
   }
   else
   {
      printf("\tReceived a NULL NodeList\n");
   }


   //-------------------------------
   // MonCPU - CPU Utilization Data
   //-------------------------------
   // determine the size of the path
   aSize += sizeof(long);              // space for other path member offset
   totalSize  = aSize;
   totalSize += (str1.Length() + 1);
   totalSize += (str2.Length() + 1);

   if (pAPath != NULL)
   {
      delete [] (BYTE *)pAPath;
      pAPath = NULL;
   }
   // create the attribute path to ask for
   pAPath = (AttributePath *) new BYTE[totalSize];
   if (pAPath == NULL)
   {
#ifdef __NETWARE_386__
      exit(RC_MEMORY_FAILURE);
#else
      return RC_MEMORY_FAILURE;
#endif
   }
   memset( pAPath, 0, totalSize);
   pAPath->size = totalSize;
   pAPath->version = PATH_VERSION_1;
   strcpy(pAPath->lang, (PCHAR)lang);
   pAPath->members = 2;
   pAPath->offsets[0] = aSize;
   strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[0]), (PCHAR)str1);
   pAPath->offsets[1] = pAPath->offsets[0] + str1.Length() + sizeof('\0');
   strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[1]), (PCHAR)str2);

   //-------------------------------
   // get a handle to the attribute
   //-------------------------------
   printf("Getting a handle to MONCPU - CPU Utilization\n");
   mHandle = monapi->getHandle(pAPath);
   delete pAPath;
   pAPath = NULL;

   if ((mHandle != NULL) && (mHandle->rc != INVALID_HANDLE))
   {
      printf("Received handle=0x%08X\n", mHandle->handle);
      //----------------------
      // get its metadata
      //----------------------
      printf("\tGetting its metadata\n");
      mMeta = monapi->getMetaData(mHandle->handle);
      if (mMeta != NULL)
      {
         if (mMeta->rc == RC_OK)
         {
            dataRate = mMeta->dataRate;
            printf("\t\tReceived valid metadata with dataRate=%d\n", dataRate);
         }
         else
         {
            printf("\t\tReceived invalid metadata\n");
         }
      }
      else
      {
         printf("\tTWGMonAPIGetMetaData() returned a NULL pointer\n");
      }

      //----------------------
      // get some data points
      //----------------------
      for (int i=1; i<11; i++)
      {
         printf("\tGetting datapoint %d of 10\n", i);
         mData = monapi->getData(mHandle->handle);

         if (mData != NULL)
         {
            if (mData->rc == RC_OK)
            {
               if ((mData->dataType & DOUBLE_NODEDATA) != 0)
               {
                  printf("\t\tReceived a double datapoint with data=%0.2f\n", mData->dValue);
               }
               if ((mData->dataType & STRING_NODEDATA) != 0)
               {
                  Str      strData((PCHAR)((PCHAR)(mData)+(mData->strOffset)));
                  strData.CompUnicodeToLocal();
                  printf("\t\tReceived a string datapoint with data=%s\n", (PCHAR)strData);
               }
               // printf("\t\tReceived a datapoint timestamp=0x%08X\n", mData->timeStamp);
               // get the formatted local time
               time_t   tStamp = mData->timeStamp;
               // Str   info(ctime((long *)&(mData->timeStamp)));
               Str   info(ctime(&tStamp));
               info[info.Length()-1] = '\0';
               printf("\t\tReceived a datapoint timestamp= %s\n", (PCHAR)info);
            }
            else
            {
               printf("\t\tReceived an invalid datapoint\n");
            }

            //------------------------------
            // free the attribute data
            //------------------------------
            printf("\tFreeing data\n");
            monapi->freeData(mData);
            mData = NULL;
         }
         else
         {
            printf("\tTWGMonAPIGetData() returned a NULL pointer\n");
         }
         #ifdef __WIN32__
         Sleep(dataRate);
         #elif defined (__OS2__)
         DosSleep(dataRate);
         #elif defined (__NETWARE_386__)
         delay(dataRate);
         #endif
      }

      //------------------------------
      // free the attribute metadata
      //------------------------------
      if (mMeta != NULL)
      {
         printf("\tFreeing metadata\n");
         monapi->freeMetaData(mMeta);
         mMeta = NULL;
      }

      //------------------------------
      // release the attribute handle
      //------------------------------
      printf("Releasing handle\n");
      monapi->releaseHandle(mHandle);
   }
   else
   {
      printf("Received an INVALID_HANDLE\n");
   }

   // delete our API interface
   printf("Freeing our TWGMonAPIDLL wrapper\n");
   monapi->logDebug("APITEST - Freeing our TWGMonAPIDLL wrapper");
   delete monapi;
   monapi = NULL;

   printf("Program finished\n");
#ifdef __NETWARE_386__
   exit(RC_OK);
#else
   return RC_OK;
#endif
}

