/* file:  monitors\com\monapi.h */
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 * 
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2002 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
#ifndef _MONAPI_H_INCLUDED_
#define _MONAPI_H_INCLUDED_

#include "sys_inc.hpp"     // include correct system files for each OS
#include "mon_sdk.h"

/****************************************************************************/
/* Monitor API Library data structures                                      */
/****************************************************************************/
#define MONHANDLE_VERSION_1                  0x00000001
typedef struct _TWGMONAPI_MONHANDLE {
        long   size;                   /* size of this struct               */
        long   version;                /* version of this struct used       */
        long   rc;                     /* return code from mon_sdk.h        */
        ULONG  handle;                 /* handle to an attribute or INVALID_HANDLE */
} MonHandle, *PMonHandle;


#define MONMETADATA_VERSION_1                0x00000001
typedef struct _TWGMONAPI_METADATA {
        long   size;                   /* size of this struct w/ string data     */
        long   version;                /* version of this struct used            */
        long   rc;                     /* return code from mon_sdk.h             */
        long   dataType;               /* NODEDATA type from mon_sdk.h           */
        ULONG  handle;                 /* handle of the attribute the metadata is from */
        long   dataRate;               /* msecs between datapoint updates        */
        long   unitsOffset;            /* offset inside struct to the units str  */
        long   formatOffset;           /* offset inside struct to printf string  */
        /* CompUnicode str[] - location of CompUnicode text strings if present   */
} MonMetaData, *PMonMetaData;


#define MONDATA_VERSION_1                    0x00000001
typedef struct _TWGMONAPI_MONDATA {
        long   size;                   /* size of this struct w/ string data     */
        long   version;                /* version of this struct used            */
        long   rc;                     /* return code from mon_sdk.h             */
        long   dataType;               /* NODEDATA type from mon_sdk.h           */
        ULONG  handle;                 /* handle of the attribute the data is from */
        ULONG  timeStamp;              /* data timeStamp in secs since 1/1/1970  */
        double dValue;                 /* raw value returned by the monitor      */
        long   strOffset;              /* offset inside struct to text or 0      */
        /* CompUnicode str[] - location of CompUnicode attribute text string     */
        /*                                or formatted monitor data              */
} MonData, *PMonData;



/****************************************************************************/
/* Monitor API Library entry point definitions                              */
/****************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************************/
/* Allocate resources required by our client API interface.                 */
/* Returns RC_OK if the API interface was initialized and is ready to use   */
/****************************************************************************/
DLLEXPORT long TWGMonAPIInit();


/****************************************************************************/
/* Get the children (nodes or leaves) for a given path.                     */
/* Returns a nodelist with 0 or more AttributeNode structures or a NULL ptr.*/
/****************************************************************************/
DLLEXPORT AttributeNodeList * TWGMonAPIGetNodeList( AttributePath MONFAR *pAPath);


/****************************************************************************/
/* Free the NodeList allocated by a call to TWGMonAPIGetNodeList().         */
/* Returns RC_OK if call was sucessful or another RC_XXX for errors.        */
/****************************************************************************/
DLLEXPORT long TWGMonAPIFreeNodeList( AttributeNodeList MONFAR *pANList);


/****************************************************************************/
/* Acquire a handle to a monitor attribute to use in the calls to           */
/* TWGMonAPIGetData(), TWGMonAPIFreeData() and TWGMonAPIReleaseHandle()     */
/* Returns INVALID_HANDLE or a handle to an attribute.                      */
/****************************************************************************/
DLLEXPORT MonHandle * TWGMonAPIGetHandle( AttributePath MONFAR *pAPath);


/****************************************************************************/
/* Retreive the current metadata for the given attribute handle.            */
/* Returns a pointer to the metadata or a NULL ptr if an error occurred.    */
/****************************************************************************/
DLLEXPORT MonMetaData * TWGMonAPIGetMetaData( ULONG handle);


/****************************************************************************/
/* Free the memory allocated by a call to TWGMonAPIGetMetaData()            */
/* Returns RC_OK if call was sucessful or another RC_XXX for errors.        */
/****************************************************************************/
DLLEXPORT long  TWGMonAPIFreeMetaData( MonMetaData MONFAR *pMData);


/****************************************************************************/
/* Retreive the current data point for the given attribute handle           */
/* Returns a pointer to a datapoint or a NULL pointer if an error occurred. */
/****************************************************************************/
DLLEXPORT MonData * TWGMonAPIGetData( ULONG handle);


/****************************************************************************/
/* Free the memory allocated by a call to TWGMonAPIGetData()                */
/* Returns RC_OK if call was sucessful or another RC_XXX for errors.        */
/****************************************************************************/
DLLEXPORT long  TWGMonAPIFreeData( MonData MONFAR *pMData);


/****************************************************************************/
/* Release a handle to a monitor returned by TWGMonAPIGetHandle()           */
/* Returns RC_OK if call was sucessful or another RC_XXX for errors.        */
/****************************************************************************/
DLLEXPORT long  TWGMonAPIReleaseHandle( MonHandle MONFAR *mHandle);


/****************************************************************************/
/* Release resources allocated by TWGMonAPIInit()                           */
/* Returns RC_OK if successful.                                             */
/****************************************************************************/
DLLEXPORT long TWGMonAPITerm();


/****************************************************************************/
/* Add a debug statement to the twgmonap.log file if twgmonap.dbg existed   */
/* when the DLL was loaded.  Returns TRUE is the string was appended.       */
/****************************************************************************/
DLLEXPORT BOOL TWGMonAPILogDebug(char MONFAR *str);


#ifdef __cplusplus
}
#endif

#endif
