//*****************************************************************************
// Director Resource Monitor Agent SDK Sample
// Licensed Materials - Property of IBM
// PID (5697-TKT)
// (C) Copyright by IBM 1999.  All Rights Reserved.
//*****************************************************************************
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <time.h>

#if defined(_Windows)

   #include <windows.h>
   #ifndef __WIN32__
      typedef DWORD        ULONG;
      #define APIENTRY     WINAPI
   #endif
   HINSTANCE    hInst = NULL;

#elif defined(__OS2__)

   #define INCL_DOSPROCESS
   #define INCL_DOSSEMAPHORES
   #define INCL_DOSDATETIME
   #define INCL_TYPES
   #define INCL_DOS
   #define PASCAL   _Pascal
   #include <os2.h>

   HMODULE      hInst = NULL;
   extern "C" void __ctordtorInit(void);
   extern "C" void __ctordtorTerm(void);

#elif defined(__NETWARE_386__)

   #include "sys_inc.hpp"
   #include "dosclls.hpp"

   HMODULE      hInst = NULL;

#else

   #error "unsupported OS type"

#endif

#include "string.hpp"
#include "monsdk.h"       // STR defines
#include "mon_sdk.h"
#include "coroner.hpp"

#ifdef __NETWARE_386__
   CHAR     Str::def_lib_name[32] = "MONSDK.NLM";
   #define  MYMODULE    Str::GetDefModule()
#endif

#ifdef _DEBUG
  CoronerLog   corlog;
  #if defined(_Windows)
    #define Debug(str) \
            corlog.LOGENTRY1(str); \
            OutputDebugString((char *)(str)); \
            OutputDebugString((char *)("\n")); \
            flushall();
  #elif defined(__NETWARE_386__)
    #define Debug(str) \
            corlog.LOGENTRY1(str); \
            puts((PCHAR)str);
  #elif defined(__OS2__)
    #define Debug(str) \
            corlog.LOGENTRY1(str); \
            puts((PCHAR)str);
  #else
    #error "ERROR: Debug() macro not defined for this OS!!!"
  #endif
#else
   #define Debug(str)
#endif


/****************************************************************************/
/* System functions and DLL entry points                                    */
/****************************************************************************/
#if defined(_Windows) // Windows ===============================================

   #ifndef __WIN32__
      int FAR PASCAL LibMain(HANDLE, WORD, WORD, LPSTR);

      #pragma argsused
      extern WORD     _WinAllocFlag;
      int FAR PASCAL LibMain(HANDLE hInstance, WORD x, WORD y, LPSTR z)
      {
         hInst = hInstance;
         _WinAllocFlag = GMEM_SHARE;
         return 1;       // Return successful load
      }
   #else
      BOOL WINAPI DllMain(HINSTANCE hMod, DWORD dwReason, LPVOID plvReserved)
      {
         if (dwReason == DLL_PROCESS_ATTACH)
         {
            hInst = hMod;
         }
         return TRUE;
      }
   #endif

#elif defined(__OS2__) // OS/2 =================================================

   unsigned long _System _DLL_InitTerm(HMODULE hMod, unsigned long ulFlag)
   {
      switch (ulFlag) {
         case 0 :
            __ctordtorInit();
            hInst = hMod;        /* Save module handle */
            break;
         case 1 :
            __ctordtorTerm();
            break;
         default  :
            return 0UL;
      }
      return 1UL;
   }

#elif defined(__NETWARE_386__) // NetWare ======================================

   int         ThreadID = 0;  // main thread id
   int         cUsers = 0;    // count of NLMS using this NLM

   extern "C"
   {
      void cleanup();
      extern NLMInitData            __NLMInitData;
      NLMInitData *        APIENTRY    MONSDK_NLMInit();
      void                 APIENTRY    MONSDK_NLMTerm();
      BOOL                 MONENTPNT   MONSDK_InitNodes();
      AttributeNodeList *  MONENTPNT   MONSDK_GetNodeList( AttributePath MONFAR *queriedPath);
      void                 MONENTPNT   MONSDK_FreeNodeList( AttributeNodeList MONFAR *pNList);
      BOOL                 MONENTPNT   MONSDK_NodeConnect( AttributePath MONFAR *pAPath, ULONG MONFAR *key,
                                                           BYTE MONFAR *dataType);
      NodeParms *          MONENTPNT   MONSDK_GetNodeAttributes( ULONG key);
      ULONG                MONENTPNT   MONSDK_NodePulse( ULONG key, ULONG time_since_last);
      NodeData *           MONENTPNT   MONSDK_NodeGetValue( ULONG key);
      void                 MONENTPNT   MONSDK_NodeFreeValue( NodeData MONFAR *nd);
      void                 MONENTPNT   MONSDK_FreeNodeAttributes( NodeParms MONFAR *nParms);
      BOOL                 MONENTPNT   MONSDK_NodeDisconnect( ULONG key);
      BOOL                 MONENTPNT   MONSDK_TermNodes();
      ULONG                MONENTPNT   MONSDK_GetThresholdCount();
      AttributePath *      MONENTPNT   MONSDK_GetThresholdPath( ULONG index, char *lang);
      BOOL                 MONENTPNT   MONSDK_FreeThresholdPath( AttributePath MONFAR *pAPath);
   };

   // rudimentary main routine for library NLM
   void main()
   {
#ifdef _DEBUG
      Debug("MONSDK - main() called");
#endif
      // DOSCLLS_InitLibrary();
      ThreadID = GetThreadID();
      SuspendThread( ThreadID );
#ifdef _DEBUG
      Debug("MONSDK - main() returning");
#endif
   }

   // The NLM initialization routine just bumps a usage count
   NLMInitData * APIENTRY MONSDK_NLMInit()
   {
#ifdef _DEBUG
      Debug("MONSDK_NLMInit() called");
#endif
      cUsers++;
      __NLMInitData.closeFile = 1;
      return &__NLMInitData;
   }

   // The termination routine that properly closes a library NLM when the usage count reaches 0
   void APIENTRY MONSDK_NLMTerm()
   {
#ifdef _DEBUG
      Debug("MONSDK_NLMTerm() called");
#endif
      cUsers--;
      if (cUsers == 0)
      {
         ResumeThread(ThreadID);
      }
   }

#else

   #error "unsupported OS type"

#endif


/****************************************************************************/
/* DynamicNode Example                                                      */
/****************************************************************************/

#define     LEVEL1_NODE       0
#define     LEVEL2_DLEAF      0
#define     LEVEL2_SLEAF      1
#define     WEEKDAY           2
#define     MONTH             3

#define     TEXT_STR_SIZE     128

typedef struct _KeyInfoBlock
{
   Str                  symNameStr;
   CHAR                 textStr[128];  // string data to return
   LONG                 handles;       // number of connections made to this KEY
   time_t               lastTime;      // last time we collected data, also its our numeric data to return
   BOOL                 inUse;         // 0 means struct not used
   BOOL                 validData;     // contains valid data from a NodePulse()
} KEY, *PKEY;

Str                     localJavaLangStr("");   // local Java locale string
static KEY              **gpKeys = NULL;
static ULONG volatile   gnumKeys = 0;
static time_t           collectValue = 30;       // collect data every 30 secs
static ULONG            pulseValue = 30;         // ask to be pulsed every 30 secs
// node symbolic and gui strings setup in InitNodes()
Str                     *symNodeStr = NULL;
Str                     *guiNodeStr = NULL;
Str                     *symDLeafStr = NULL;
Str                     *guiDLeafStr = NULL;
Str                     *symSLeafStr = NULL;
Str                     *guiSLeafStr = NULL;
Str                     *symWeekdayStr = NULL;
Str                     *guiWeekdayStr = NULL;
Str                     *symMonthStr = NULL;
Str                     *guiMonthStr = NULL;


//=============================================================================
//| Internal Helper Functions
//=============================================================================

// delete a nodelist we created
void DeleteNode( AttributeNode MONFAR *pANode)
{
   if (pANode)
   {
      delete [] (BYTE MONFAR *)pANode;
      pANode = NULL;
   }
}


// create our real nodeList to return
void CreateNodeList( AttributeNodeList MONFAR **pNodeList, AttributeNode MONFAR **pNodes,
                     long nodes, long nodesSize)
{
   static long          nlSize = sizeof(AttributeNodeList);
   long                 totalSize;           // total memory block being returned as the NodeList
   long                 i;                   // loop counter
   unsigned long        copyOffset;


#ifdef _DEBUG
   {
      Str e("CreateNodeList() - entered");
      Debug(e);
   }
#endif
   // determine memory needed to return
   totalSize = nlSize + nodesSize;

   // create the memory to return
   *pNodeList = (AttributeNodeList MONFAR *) new BYTE[totalSize];
   if (*pNodeList == NULL)
      return;
   memset( *pNodeList, 0, totalSize);

   (*pNodeList)->size = sizeof(AttributeNodeList);
   (*pNodeList)->totalSize = totalSize;
   (*pNodeList)->version = NODE_LIST_VERSION_2;
   (*pNodeList)->members = nodes;

   // setup copy offset
   copyOffset = nlSize;

#ifdef _DEBUG
   {
      Str e("CreateNodeList() - before copy loop");
      Debug(e);
   }
#endif
   // copy and then delete each node
   for (i=0; i<nodes; i++)
   {
      if (pNodes[i])
      {
#ifdef _DEBUG
         {
            char        szTemp[32];
            Str e1("Monsdk::CreateNodeList() - in copy loop with pNode=");
            sprintf( szTemp, "%p", (void*)pNodes[i]);
            e1 += szTemp;
            e1 += ", size=";
            sprintf( szTemp, "%i", pNodes[i]->size);
            e1 += szTemp;
            e1 += ", version=";
            sprintf( szTemp, "%i", pNodes[i]->version);
            e1 += szTemp;
            e1 += ", lang=";
            e1 += pNodes[i]->lang;
            e1 += ", isLeaf=";
            sprintf( szTemp, "%i", pNodes[i]->isLeaf);
            e1 += szTemp;
            e1 += ", type=";
            sprintf( szTemp, "%i", pNodes[i]->type);
            e1 += szTemp;
            e1 += ", nameOffset=";
            sprintf( szTemp, "%i", pNodes[i]->nameOffset);
            e1 += szTemp;
            e1 += ", symNameOffset=";
            sprintf( szTemp, "%i", pNodes[i]->symNameOffset);
            e1 += szTemp;
            Debug( e1);
         }
#endif
         // copy node
         memcpy( ((BYTE *)(*pNodeList)+copyOffset), pNodes[i], pNodes[i]->size);

         // update copy offset
         copyOffset += pNodes[i]->size;

         // delete the node
         DeleteNode(pNodes[i]);
         pNodes[i] = NULL;
      }
   }
#ifdef _DEBUG
   {
      Str e("CreateNodeList() - after copy loop");
      Debug(e);
   }
#endif

   // delete node list
   if (pNodes)
   {
      delete [] (PVOID)pNodes;
      pNodes = NULL;
   }
#ifdef _DEBUG
   {
      Str e("CreateNodeList() - returning");
      Debug(e);
   }
#endif
}


//-----------------------------------------------------------------------------
//  cleanup( ) -
//
//    Clean up any memory we allocated or globals that need to be reset
//
//-----------------------------------------------------------------------------
void cleanup( )
{

#ifdef _DEBUG
   Debug("MONSDK - cleanup() called");
#endif
   // clean up any memory allocations
   for (ULONG i=0; (i<gnumKeys); i++)
   {
      if (gpKeys[i])
      {
         if (gpKeys[i]->inUse)
         {
            // remove the counter from the PDH query
            // PdhRemoveCounter( gpCib[i]->hCounter);
            gpKeys[i]->inUse = FALSE;
         }
         delete gpKeys[i];
         gpKeys[i] = NULL;
      }
   }
   gnumKeys = 0;
   delete [] gpKeys;

   // cleanup our node strings
   if (symNodeStr)
   {
      delete symNodeStr;
      symNodeStr = NULL;
   }
   if (guiNodeStr)
   {
      delete guiNodeStr;
      guiNodeStr = NULL;
   }
   // cleanup our leaf strings
   if (symDLeafStr)
   {
      delete symDLeafStr;
      symDLeafStr = NULL;
   }
   if (guiDLeafStr)
   {
      delete guiDLeafStr;
      guiDLeafStr = NULL;
   }
   if (symSLeafStr)
   {
      delete symSLeafStr;
      symSLeafStr = NULL;
   }
   if (guiSLeafStr)
   {
      delete guiSLeafStr;
      guiSLeafStr = NULL;
   }
   if (symWeekdayStr)
   {
      delete symWeekdayStr;
      symWeekdayStr = NULL;
   }
   if (guiWeekdayStr)
   {
      delete guiWeekdayStr;
      guiWeekdayStr = NULL;
   }
   if (symMonthStr)
   {
      delete symMonthStr;
      symMonthStr = NULL;
   }
   if (guiMonthStr)
   {
      delete guiMonthStr;
      guiMonthStr = NULL;
   }

}


//=============================================================================
//| Exported DynamicNode Monitor DLL Functions
//=============================================================================

// setup a simple function name mapping so we can have common code with Netware
#if defined(__NETWARE_386__)
   #define API_InitNodes            MONENTPNT MONSDK_InitNodes
   #define API_GetNodeList          MONENTPNT MONSDK_GetNodeList
   #define API_FreeNodeList         MONENTPNT MONSDK_FreeNodeList
   #define API_NodeConnect          MONENTPNT MONSDK_NodeConnect
   #define API_GetNodeAttributes    MONENTPNT MONSDK_GetNodeAttributes
   #define API_NodePulse            MONENTPNT MONSDK_NodePulse
   #define API_NodeGetValue         MONENTPNT MONSDK_NodeGetValue
   #define API_NodeFreeValue        MONENTPNT MONSDK_NodeFreeValue
   #define API_FreeNodeAttributes   MONENTPNT MONSDK_FreeNodeAttributes
   #define API_NodeDisconnect       MONENTPNT MONSDK_NodeDisconnect
   #define API_TermNodes            MONENTPNT MONSDK_TermNodes
   #define API_GetThresholdCount    MONENTPNT MONSDK_GetThresholdCount
   #define API_GetThresholdPath     MONENTPNT MONSDK_GetThresholdPath
   #define API_FreeThresholdPath    MONENTPNT MONSDK_FreeThresholdPath
#else
   #define API_InitNodes            MONENTPNT InitNodes
   #define API_GetNodeList          MONENTPNT GetNodeList
   #define API_FreeNodeList         MONENTPNT FreeNodeList
   #define API_NodeConnect          MONENTPNT NodeConnect
   #define API_GetNodeAttributes    MONENTPNT GetNodeAttributes
   #define API_NodePulse            MONENTPNT NodePulse
   #define API_NodeGetValue         MONENTPNT NodeGetValue
   #define API_NodeFreeValue        MONENTPNT NodeFreeValue
   #define API_FreeNodeAttributes   MONENTPNT FreeNodeAttributes
   #define API_NodeDisconnect       MONENTPNT NodeDisconnect
   #define API_TermNodes            MONENTPNT TermNodes
   #define API_GetThresholdCount    MONENTPNT GetThresholdCount
   #define API_GetThresholdPath     MONENTPNT GetThresholdPath
   #define API_FreeThresholdPath    MONENTPNT FreeThresholdPath
#endif

/*==========================================================================*/
/*| Monitor API Library entry point definitions                            |*/
/*==========================================================================*/

//-----------------------------------------------------------------------------
//  InitNodes( )
//-----------------------------------------------------------------------------
BOOL API_InitNodes()
{

#ifdef _DEBUG
   Debug("MONSDK_InitNodes() called");
#endif

#ifdef __NETWARE_386__

   localJavaLangStr = "en";
   // convert our global strings to Compressed Unicode
   localJavaLangStr.LocalToCompUnicode();

   // setup our node strings
   symNodeStr = new Str("LEVEL1_NODE");
   symNodeStr->LocalToCompUnicode();
   guiNodeStr = new Str("Dynamic Nodes");
   guiNodeStr->LocalToCompUnicode();

   // setup our leaf strings
   symDLeafStr = new Str("LEVEL2_DLEAF");
   symDLeafStr->LocalToCompUnicode();
   guiDLeafStr = new Str("Numeric Time");
   guiDLeafStr->LocalToCompUnicode();

   symSLeafStr = new Str("LEVEL2_SLEAF");
   symSLeafStr->LocalToCompUnicode();
   guiSLeafStr = new Str("String Time");
   guiSLeafStr->LocalToCompUnicode();

   // setup our WEEKDAY leaf strings
   symWeekdayStr = new Str("WEEKDAY");
   symWeekdayStr->LocalToCompUnicode();
   guiWeekdayStr = new Str("Weekday");
   guiWeekdayStr->LocalToCompUnicode();

   // setup our MONTH leaf strings
   symMonthStr = new Str("MONTH");
   symMonthStr->LocalToCompUnicode();
   guiMonthStr = new Str("Month");
   guiMonthStr->LocalToCompUnicode();

#else

   localJavaLangStr = Str(STR_NLS_JAVA_LOCALE, hInst);
   // convert our global strings to Compressed Unicode
   localJavaLangStr.LocalToCompUnicode();

   // setup our node strings
   symNodeStr = new Str("LEVEL1_NODE");
   symNodeStr->LocalToCompUnicode();
   guiNodeStr = new Str(STR_LEVEL1_NODE, hInst);
   guiNodeStr->LocalToCompUnicode();

   // setup our leaf strings
   symDLeafStr = new Str("LEVEL2_DLEAF");
   symDLeafStr->LocalToCompUnicode();
   guiDLeafStr = new Str(STR_LEVEL2_DLEAF, hInst);
   guiDLeafStr->LocalToCompUnicode();

   symSLeafStr = new Str("LEVEL2_SLEAF");
   symSLeafStr->LocalToCompUnicode();
   guiSLeafStr = new Str(STR_LEVEL2_SLEAF, hInst);
   guiSLeafStr->LocalToCompUnicode();

   // setup our WEEKDAY leaf strings
   symWeekdayStr = new Str("WEEKDAY");
   symWeekdayStr->LocalToCompUnicode();
   guiWeekdayStr = new Str(STR_WEEKDAY, hInst);
   guiWeekdayStr->LocalToCompUnicode();

   // setup our MONTH leaf strings
   symMonthStr = new Str("MONTH");
   symMonthStr->LocalToCompUnicode();
   guiMonthStr = new Str(STR_MONTH, hInst);
   guiMonthStr->LocalToCompUnicode();

#endif

   return TRUE;
}


//-----------------------------------------------------------------------------
//  TermNodes( )
//-----------------------------------------------------------------------------
BOOL API_TermNodes()
{
#ifdef _DEBUG
   Debug("MONSDK_TermNodes() called");
#endif
   // cleanup allocated memory
   cleanup();

   return TRUE;
}


//-----------------------------------------------------------------------------
//  FreeNodeList( ) -
//-----------------------------------------------------------------------------
void API_FreeNodeList( AttributeNodeList MONFAR *pNList)
{
   if (pNList)
   {
      delete [] pNList;
      pNList = NULL;
   }
}


//-----------------------------------------------------------------------------
//  GetNodeList( ) -
//-----------------------------------------------------------------------------
AttributeNodeList MONFAR * API_GetNodeList( AttributePath MONFAR *queriedPath)
{
   Str                  requestedJavaLang("");        // requested Java Lang, i.e. "en_US"
   Str                  guiName("");
   Str                  symName("");
   long                 attribs;                      // number of performance monitor items to scan
   long                 nodes;                        // number of AttributeNode nodes created
   long                 nodesSize;                    // size of created AttributeNode nodes
   AttributeNode        **pNodes;
   AttributeNode        *pNode = NULL;
   AttributeNodeList    *pNodeList = NULL;            // ptr to data returned by GetNodeList() call
   BYTE                 dataType = INVALID_NODEDATA;
   BOOL                 isLeaf;


   if (queriedPath == NULL)
   {
#ifdef _DEBUG
      Str e("Monsdk::GetNodeList() was passed a NULL AttributePath struct");
      Debug(e);
#endif
      return pNodeList;
   }

   // get the requested Java lang string and convert it to ASCIIZ
   requestedJavaLang = queriedPath->lang;
   requestedJavaLang.CompUnicodeToLocal();


   // see what the monitor agent wants
   if (queriedPath->members == 1)
   {
      /***********/
      /* Level 1 */
      /***********/

      /**********/
      /* Nodes  */
      /**********/
      isLeaf = FALSE;

      // we know we will only have 1 branch
      attribs = 1;

      // create temp nodeList
      pNodes = (AttributeNode **) new (VOID *[attribs]);
      if (pNodes == NULL)
      {
         // cleanup and return a NULL list
#ifdef _DEBUG
         Str e("Monsdk::GetNodeList() - Level 1 - new of pNodes failed !!!");
         Debug(e);
#endif
         return pNodeList;
      }
      memset( pNodes, 0, sizeof(VOID *) * attribs);

      // clear out our nodesSize before creating the nodes
      nodesSize = 0;
      nodes = 0;


      // now get our node names
      for (long i=0; i<attribs; i++)
      {
         // see which node we want
         if (i == LEVEL1_NODE)
         {
            symName = *symNodeStr;
            guiName = *guiNodeStr;
#ifdef _DEBUG
            Str e("Monsdk::GetNodeList() - Level 1 - creating LEVEL1_NODE");
            Debug(e);
#endif
         }
         else
         {
            // don't create a bad node
            continue;
         }

         // build node with name and symName
         // pNodes[nodes] = mNode = CreateNode( FALSE,              // convert to CompUnicode
         /***
         CreateNode( mNode, FALSE,              // convert to CompUnicode
                                     localJavaLangStr,
                                     isLeaf,
                                     dataType,
                                     guiName,            // gui name
                                     symName);           // symbolic name
         ***/
         {
            int            nodeSize;            // node struct size
            int            nameSize;            // name strlen
            int            symSize;             // symbolic name strlen
            int            totalSize;           // total node size

         #ifdef _DEBUG
            {
               Str e1("Monsdk::GetNodeList() - Level 1 - creating node with localJavaLangStr=");
               e1 += localJavaLangStr;
               e1 += ", guiName=";
               e1 += guiName;
               e1 += ", symName=";
               e1 += symName;
               Debug( e1);
            }
         #endif

            nodeSize = sizeof(AttributeNode);
            nameSize = strlen( guiName) + 1;
            symSize = strlen( symName) + 1;
            totalSize = nodeSize + nameSize + symSize;

            pNode = (AttributeNode *) new BYTE[totalSize];
            if (pNode == NULL)
            {
         #ifdef _DEBUG
               Str e1("Monsdk::GetNodeList() - Level 1 - creation of pNode failed!");
               Debug( e1);
         #endif
               return pNodeList;
            }
            // return NULL;
            memset( pNode, 0, totalSize);

            // fill in values
            pNode->size = totalSize;
            pNode->version = NODE_VERSION_1;
            strncpy(pNode->lang, localJavaLangStr, sizeof(pNode->lang));
            pNode->isLeaf = isLeaf;
            pNode->type = dataType;
            pNode->nameOffset = nodeSize;
            pNode->symNameOffset = nodeSize + nameSize;
            strcpy((PCHAR)(((PCHAR)pNode)+pNode->nameOffset),(PCHAR)guiName);
            strcpy((PCHAR)(((PCHAR)pNode)+pNode->symNameOffset),(PCHAR)symName);

         #ifdef _DEBUG
            {
               char        szTemp[32];
               Str e1("Monsdk::GetNodeList() - Level 1 - created pNode=");
               sprintf( szTemp, "%p", (void*)pNode);
               e1 += szTemp;
               e1 += ", size=";
               sprintf( szTemp, "%i", pNode->size);
               e1 += szTemp;
               e1 += ", version=";
               sprintf( szTemp, "%i", pNode->version);
               e1 += szTemp;
               e1 += ", lang=";
               e1 += pNode->lang;
               e1 += ", isLeaf=";
               sprintf( szTemp, "%i", pNode->isLeaf);
               e1 += szTemp;
               e1 += ", type=";
               sprintf( szTemp, "%i", pNode->type);
               e1 += szTemp;
               e1 += ", nameOffset=";
               sprintf( szTemp, "%i", pNode->nameOffset);
               e1 += szTemp;
               e1 += ", symNameOffset=";
               sprintf( szTemp, "%i", pNode->symNameOffset);
               e1 += szTemp;
               Debug( e1);
            }
         #endif
         }

         if (pNode != NULL)
         {
            pNodes[nodes] = pNode;
#ifdef _DEBUG
            {
               char     szTemp[32];
               Str e1("Monsdk::GetNodeList() - Level 1 - saved pNode with address of ");
               sprintf( szTemp, "%p", (void*)pNode);
               e1 += szTemp;
               Debug(e1);
            }
#endif
            nodesSize += pNodes[nodes]->size;
            nodes++;
         }
         else
         {
#ifdef _DEBUG
            Str e("Monsdk::GetNodeList() - Level 1 - no objects found!");
            Debug(e);
#endif
            continue;
         }
      }  // end for loop

      // create our real nodeList to return
      if (nodes > 0)
      {
#ifdef _DEBUG
         Str e("Monsdk::GetNodeList() - Level 1 - creating nodelist to return");
         Debug(e);
#endif
         CreateNodeList(&pNodeList, pNodes, nodes, nodesSize);
      }

   }
   else if (queriedPath->members == 2)
   {
      /***********/
      /* Level 2 */
      /***********/
      Str      path1Str("");

      /**********/
      /* Leaves */
      /**********/
      // we know we will only have leaves
      isLeaf = TRUE;

      // get the path member for our comparisions
      path1Str  = (PCHAR)(((PCHAR)(queriedPath))+queriedPath->offsets[1]);
      // don't convert to local, we'll just compare it as compUnicode
      // path1Str.CompUnicodeToLocal();

      // we know we will only have 4 attribute nodes
      attribs = 4;

      // create temp nodeList
      pNodes = (AttributeNode **) new (VOID *[attribs]);
      if (pNodes == NULL)
      {
         // cleanup and return a NULL list
#ifdef _DEBUG
         Str e("Monsdk::GetNodeList() - Level 2 - new of pNodes failed !!!");
         Debug(e);
#endif
         return pNodeList;
      }
      memset( pNodes, 0, sizeof(VOID *) * attribs);

      // clear out our nodesSize before creating the nodes
      nodesSize = 0;
      nodes = 0;

      for (long i=0; i<attribs; i++)
      {
         // see which node we want
         if (i == LEVEL2_DLEAF)
         {
            symName = *symDLeafStr;
            guiName = *guiDLeafStr;
            dataType = DOUBLE_NODEDATA;
         }
         else if (i == LEVEL2_SLEAF)
         {
            symName = *symSLeafStr;
            guiName = *guiSLeafStr;
            dataType = STRING_NODEDATA;
         }
         else if (i == WEEKDAY)
         {
            symName = *symWeekdayStr;
            guiName = *guiWeekdayStr;
            dataType = STRING_NODEDATA;
         }
         else if (i == MONTH)
         {
            symName = *symMonthStr;
            guiName = *guiMonthStr;
            dataType = STRING_NODEDATA;
         }
         else
         {
            // don't create a bad node
            continue;
         }

         // only process shares of the type we want
         if (path1Str == *symNodeStr)
         {
            // build node with name and symName
            {
               int            nodeSize;            // node struct size
               int            nameSize;            // name strlen
               int            symSize;             // symbolic name strlen
               int            totalSize;           // total node size

            #ifdef _DEBUG
               {
                  Str e1("Monsdk::GetNodeList() - Level 2 - creating node with localJavaLangStr=");
                  e1 += localJavaLangStr;
                  e1 += ", guiName=";
                  e1 += guiName;
                  e1 += ", symName=";
                  e1 += symName;
                  Debug( e1);
               }
            #endif

               nodeSize = sizeof(AttributeNode);
               nameSize = strlen( guiName) + 1;
               symSize = strlen( symName) + 1;
               totalSize = nodeSize + nameSize + symSize;

               pNode = (AttributeNode *) new BYTE[totalSize];
               if (pNode == NULL)
               {
            #ifdef _DEBUG
                  Str e1("Monsdk::GetNodeList() - Level 2 - creation of pNode failed!");
                  Debug( e1);
            #endif
                  return pNodeList;
               }
               // return NULL;
               memset( pNode, 0, totalSize);

               // fill in values
               pNode->size = totalSize;
               pNode->version = NODE_VERSION_1;
               strncpy(pNode->lang, localJavaLangStr, sizeof(pNode->lang));
               pNode->isLeaf = isLeaf;
               pNode->type = dataType;
               pNode->nameOffset = nodeSize;
               pNode->symNameOffset = nodeSize + nameSize;
               strcpy((PCHAR)(((PCHAR)pNode)+pNode->nameOffset),(PCHAR)guiName);
               strcpy((PCHAR)(((PCHAR)pNode)+pNode->symNameOffset),(PCHAR)symName);

            #ifdef _DEBUG
               {
                  char        szTemp[32];
                  Str e1("Monsdk::GetNodeList() - Level 2 - created pNode=");
                  sprintf( szTemp, "%p", (void*)pNode);
                  e1 += szTemp;
                  e1 += ", size=";
                  sprintf( szTemp, "%i", pNode->size);
                  e1 += szTemp;
                  e1 += ", version=";
                  sprintf( szTemp, "%i", pNode->version);
                  e1 += szTemp;
                  e1 += ", lang=";
                  e1 += pNode->lang;
                  e1 += ", isLeaf=";
                  sprintf( szTemp, "%i", pNode->isLeaf);
                  e1 += szTemp;
                  e1 += ", type=";
                  sprintf( szTemp, "%i", pNode->type);
                  e1 += szTemp;
                  e1 += ", nameOffset=";
                  sprintf( szTemp, "%i", pNode->nameOffset);
                  e1 += szTemp;
                  e1 += ", symNameOffset=";
                  sprintf( szTemp, "%i", pNode->symNameOffset);
                  e1 += szTemp;
                  Debug( e1);
               }
            #endif
            }

            if (pNode != NULL)
            {
               pNodes[nodes] = pNode;
#ifdef _DEBUG
               {
                  char     szTemp[32];
                  Str e1("Monsdk::GetNodeList() - Level 2 - saved pNode with address of ");
                  sprintf( szTemp, "%p", (void*)pNode);
                  e1 += szTemp;
                  Debug(e1);
               }
#endif
               nodesSize += pNodes[nodes]->size;
               nodes++;
            }
            else
            {
#ifdef _DEBUG
               Str e("Monsdk::GetNodeList() - Level 2 - no objects found!");
               Debug(e);
#endif
               continue;
            }
         }
         else
         {
            // doesn't match the type we are looking for, so skip to next one
            continue;
         }

      }  // for loop

#ifdef _DEBUG
      Str e("Monsdk::GetNodeList() - Level 2 - creating nodelist to return");
      Debug(e);
#endif
      // create our real nodeList to return
      if (nodes > 0)
      {
#ifdef _DEBUG
         Str e("Monsdk::GetNodeList() - Level 2 - creating nodelist to return");
         Debug(e);
#endif
         CreateNodeList(&pNodeList, pNodes, nodes, nodesSize);
      }

   }

   else
   {
      /*************/
      /* Bad Level */
      /*************/
#ifdef _DEBUG
      char  szTemp[32];
      Str   e("Monsdk:: error - GetNodeList() was asked for an invalid Level of ");
      sprintf( szTemp, "%i", queriedPath->members);
      e += szTemp;
      Debug (e);
#endif
   }

#ifdef _DEBUG
   {
      char        szTemp[32];
      Str e1("Monsdk::GetNodeList() - returning pNodeList of address=");
      sprintf( szTemp, "%p", (void *)pNodeList);
      e1 += szTemp;
      if (pNodeList != NULL)
      {
         e1 += ", size=";
         sprintf( szTemp, "%i", pNodeList->size);
         e1 += szTemp;
         e1 += ", version=";
         sprintf( szTemp, "%i", pNodeList->version);
         e1 += szTemp;
         e1 += ", totalSize=";
         sprintf( szTemp, "%i", pNodeList->totalSize);
         e1 += szTemp;
         e1 += ", members=";
         sprintf( szTemp, "%i", pNodeList->members);
         e1 += szTemp;
         Debug(e1);
      }
      else
      {
         Debug(e1);
      }
   }
#endif

   // return our node list
   return pNodeList;
}


//-----------------------------------------------------------------------------
//  FreeNodeAttributes( )
//-----------------------------------------------------------------------------
void API_FreeNodeAttributes(NodeParms MONFAR *nParms)
{
   if (nParms)
   {
      delete [] (BYTE *)nParms;
      nParms = NULL;
   }
}


//-----------------------------------------------------------------------------
//  GetNodeAttributes( )
//-----------------------------------------------------------------------------
NodeParms MONFAR * API_GetNodeAttributes(ULONG nKey)
{
   static ULONG         npSize = sizeof(NodeParms);
   ULONG                totalSize;
   NodeParms            *nParms;
   Str                  format("");
   // break out the key and enumeration that we gave them
#if defined(__OS2__) || defined(__NETWARE_386__)
   USHORT               key = HIUSHORT(nKey);
   USHORT               enumKey = LOUSHORT(nKey);
#else
   WORD                 key = HIWORD(nKey);
   WORD                 enumKey = LOWORD(nKey);
#endif

   if ((nKey != INVALID_HANDLE) && (key < gnumKeys))
   {
      if (gpKeys[key]->inUse)
      {
         if (enumKey == LEVEL2_DLEAF)
         {
            // numeric data value
            format = "%0.0f";
         }
         // else if (enumKey == LEVEL2_SLEAF)
         // else if (enumKey == WEEKDAY)
         // else if (enumKey == MONTH)
         else
         {
            // everything else is string data values
            format = "%s";
         }
         format.LocalToCompUnicode();
         totalSize = npSize + format.Length() + 1;
         nParms = (NodeParms MONFAR *)new BYTE[totalSize];
         if (nParms)
         {
            memset( nParms, 0, totalSize);
            nParms->size = totalSize;
            nParms->version = NODEPARMS_VERSION_2;
            nParms->rc = RC_OK;
            nParms->base_pulse = (pulseValue * 1000);    // 30 secs in msec.
            nParms->pulses_per_data = 1;  // collect data on every pulse
            nParms->num_recorded = 10;    // enough for 5 mins.
            nParms->attrib_flags = 0;
            nParms->output_type = 0;
            nParms->number_data_levels = 0;
            nParms->data_levels = NULL;
            // copy over variable length string data
            nParms->units_label_offset = 0;  // not set
            nParms->value_format_offset = npSize;
            strcpy( (PCHAR)(((PCHAR)nParms)+nParms->value_format_offset), format);
         }
      }

   }
   // caller must call FreeNodeAttributes() to release the memory
   return nParms;
}


//-----------------------------------------------------------------------------
//  NodeConnect( )
//-----------------------------------------------------------------------------
BOOL API_NodeConnect( AttributePath MONFAR *pAPath, ULONG MONFAR *key, BYTE MONFAR *dataType)
{
   static ULONG         keySize = sizeof(KEY);
   static ULONG         pKeySize = sizeof(PKEY);
   KEY                  **pTempKeys = NULL;
   USHORT               index, enumKey;
   ULONG                newNum;
   ULONG                nKey = 0;
   BOOL                 found = FALSE;
   Str                  path1Str("");
   Str                  path2Str("");

   // init return values to invalid defaults
   *key = INVALID_HANDLE;
   *dataType = INVALID_NODEDATA;

   if (pAPath->members == 3)
   {
      // setup the key name
      path1Str = (PCHAR)(((PCHAR)(pAPath))+pAPath->offsets[1]);   // share type
      path2Str = (PCHAR)(((PCHAR)(pAPath))+pAPath->offsets[2]);   // share name

      // see if we are already connected to this share
      for (index=0; (!found) && (index<gnumKeys); )
      {
         if (gpKeys[index]->symNameStr == path2Str)
         {
            found = TRUE;
         }
         else
         {
            index++;
         }
      }

      if (!found)
      {
         // find availble struct
         for (index=0; (!found) && (index<gnumKeys); )
         {
            if (gpKeys[index]->inUse == FALSE)
            {
               found = TRUE;
#ifdef _DEBUG
               char        szTemp[32];
               Str e("Monsdk::NodeConnect() found an available entry at index=");
               sprintf(szTemp, "0x%08X", index);
               e += szTemp;
               Debug (e);
#endif
            }
            else
            {
               index++;
            }
         }
      }
   }
   else
   {
      return FALSE;
   }

   if (!found)
   {
#ifdef _DEBUG
      Str e("Monsdk::NodeConnect() going to grow our array");
      Debug (e);
#endif
      // we need to grow our array
      newNum = gnumKeys + 20;
      if (newNum >= 0x0000ffffL)
      {
         // we have passed our max node enum value for a WORD
         return FALSE;
      }

      pTempKeys = new PKEY[newNum];
      if (pTempKeys)
      {
         // init new array and copy over old array
         memset( pTempKeys, 0, pKeySize*newNum);
         if (gnumKeys != 0)
         {
            memcpy( pTempKeys, gpKeys, (gnumKeys * pKeySize));
            // delete old array
            delete [] gpKeys;
         }
         // allocate structs for new pointers
         for (ULONG i=gnumKeys; i < newNum; i++)
         {
            pTempKeys[i] = new KEY;
            memset( pTempKeys[i], 0, keySize);
         }
         // set globals pointers/values to new ones
         gpKeys = pTempKeys;
         gnumKeys += 20;
         found = TRUE;
      }
   }

   if (found)
   {
      // setup the handle to return, which has the key index in the hiword
      //   and the leaf node enum in the low word
      nKey = index;
      nKey = (nKey << 16);
      if (*symDLeafStr == path2Str)
      {
         enumKey = LEVEL2_DLEAF;
      }
      else if (*symSLeafStr == path2Str)
      {
         enumKey = LEVEL2_SLEAF;
      }
      else if (*symWeekdayStr == path2Str)
      {
         enumKey = WEEKDAY;
      }
      else if (*symMonthStr == path2Str)
      {
         enumKey = MONTH;
      }
      else
      {
#ifdef _DEBUG
         Str e("Monsdk::NodeConnect() string compares could not determine the share enum type!");
         Debug (e);
#endif
         *key = INVALID_HANDLE;
         return FALSE;
      }

      // finish setting up the key
      nKey += enumKey;
      *key = nKey;

#ifdef _DEBUG
      {
         char     szTemp[32];
         Str      e1("Monsdk::NodeConnect() enumKey=");
         sprintf( szTemp, "%i", enumKey);
         e1 += szTemp;
         e1 += ", nKey=";
         sprintf( szTemp, "%i", nKey);
         e1 += szTemp;
         Debug( e1);
      }
#endif


      if (gpKeys[index]->inUse == TRUE)
      {
         // we are reusing a share entry, so just increment its usage counter
         gpKeys[index]->handles++;
      }
      else
      {
         // init values
         gpKeys[index]->inUse = TRUE;
         gpKeys[index]->validData = FALSE;
         gpKeys[index]->lastTime = 0;
         gpKeys[index]->handles = 1;
         gpKeys[index]->symNameStr = path2Str;
         memset(gpKeys[index]->textStr,0,TEXT_STR_SIZE);
      }

      // set callers handle to the index into our key array

      // setup the leaf data type for the caller
      if (path2Str == *symDLeafStr)
      {
         // our only numeric data leaf
         *dataType = DOUBLE_NODEDATA;
      }
      else
      {
         // everything else is string data leaves
         *dataType = STRING_NODEDATA;
      }


#ifdef _DEBUG
      {
         char        szTemp[32];
         Str         pathStr("");
         Str         slashStr("\\");
         Str         e1("Monsdk::NodeConnect() handle=");
         sprintf( szTemp, "0x%08X", *key);
         e1 += szTemp;
         e1 += ", path=";
         // setup path
         slashStr.LocalToCompUnicode();
         pathStr  = slashStr;
         pathStr += path1Str;
         pathStr += slashStr;
         pathStr += path2Str;
         pathStr.CompUnicodeToLocal();
         e1 += pathStr;
         Debug( e1);
      }
#endif
      return TRUE;
   }
   else
   {
#ifdef _DEBUG
      Str e("Monsdk::NodeConnect() returning FALSE!");
      Debug (e);
#endif
      return FALSE;
   }
}


//-----------------------------------------------------------------------------
//  NodeDisconnect( )
//-----------------------------------------------------------------------------
BOOL API_NodeDisconnect(ULONG nKey)
{
#if defined(__OS2__) || defined(__NETWARE_386__)
   USHORT               key = HIUSHORT(nKey);
   USHORT               enumKey = LOUSHORT(nKey);
#else
   WORD                 key = HIWORD(nKey);
   WORD                 enumKey = LOWORD(nKey);
#endif

   if ((nKey != INVALID_HANDLE) && (key < gnumKeys))
   {
      if (gpKeys[key]->inUse)
      {
         // decrement the handle count
         gpKeys[key]->handles--;
         if (gpKeys[key]->handles <= 0)
         {
            // clear out the share info
            gpKeys[key]->inUse = FALSE;
            gpKeys[key]->validData = FALSE;
            gpKeys[key]->lastTime = 0;
            gpKeys[key]->handles = 0;
            gpKeys[key]->symNameStr = "";
            memset(gpKeys[key]->textStr,0,TEXT_STR_SIZE);
         }
      }
      return TRUE;
   }
   return FALSE;
}


//-----------------------------------------------------------------------------
//  NodeFreeValue( )
//-----------------------------------------------------------------------------
void API_NodeFreeValue(NodeData MONFAR *nd)
{
#ifdef _DEBUG
   Debug("MONSDK_NodeFreeValue() called");
#endif
   if (nd != NULL)
      delete [] (BYTE MONFAR *)nd;
}


//-----------------------------------------------------------------------------
//  NodeGetValue( )
//-----------------------------------------------------------------------------
NodeData MONFAR * API_NodeGetValue(ULONG nKey)
{
   static long          ndSize = sizeof(NodeData);
   long                 totalSize = ndSize;
   NodeData             *nd = NULL;
#if defined(__OS2__) || defined(__NETWARE_386__)
   USHORT               key = HIUSHORT(nKey);
   USHORT               enumKey = LOUSHORT(nKey);
#else
   WORD                 key = HIWORD(nKey);
   WORD                 enumKey = LOWORD(nKey);
#endif
   Str                  dataStr("");

   if ((nKey != INVALID_HANDLE) && (key < gnumKeys))
   {
      // key is index into our array
      if (gpKeys[key]->inUse == TRUE)
      {
         // determine the total struct size needed
         // see which enum we want
         switch (enumKey)
         {
            case LEVEL2_DLEAF:
               {
                  // the only numeric data leaf
                  // nothing else to do
               }
               break;
            case LEVEL2_SLEAF:
            case WEEKDAY:
            case MONTH:
               {
                  dataStr = gpKeys[key]->textStr;
                  dataStr.LocalToCompUnicode();
                  totalSize += dataStr.Length();
                  totalSize += 1;
               }
               break;
            default:
               {
                  // don't create a data value
                  return nd;
               }
         }

         // allocate a NodeData struct to return
         nd = (NodeData MONFAR *) new BYTE[totalSize];
         if (nd)
         {
            memset( nd, 0, totalSize);
            // set the struct info
            nd->size = totalSize;
            nd->version = NODEDATA_VERSION_1;

            if (gpKeys[key]->validData == TRUE)
            {
               // see which SHARE enum we want
               if (enumKey == LEVEL2_DLEAF)
               {
                  nd->dataType = DOUBLE_NODEDATA;
                  nd->data.dValue = (double)gpKeys[key]->lastTime;
               }
               else
               {
                  nd->dataType = STRING_NODEDATA;
                  nd->data.strOffset = ndSize;
                  strcpy( (PCHAR)(((PCHAR)nd)+nd->data.strOffset), dataStr);
#ifdef _DEBUG
                  Str e("Monsdk::NodeGetValue() setting string data=");
                  e += dataStr;
                  Debug(e);
#endif
               }

               // set our valid rc
               nd->rc = RC_OK;

#ifdef _DEBUG
               char szTemp[32];
               Str e("Monsdk::NodeGetValue(key index=");
               sprintf(szTemp, "0x%04X", key);
               e += szTemp;
               e += ", enum index=";
               sprintf(szTemp, "0x%04X", enumKey);
               e += szTemp;
               e += ") returning RC_OK";
               Debug (e);
#endif
            }
            else
            {
               // set our invalid rc
               nd->rc = RC_INVALID_DATA;
#ifdef _DEBUG
               char szTemp[32];
               Str e("Monsdk::NodeGetValue(key index=");
               sprintf(szTemp, "0x%04X", key);
               e += szTemp;
               e += ", enum index=";
               sprintf(szTemp, "0x%04X", enumKey);
               e += szTemp;
               e += ") returning RC_INVALID_DATA";
               Debug (e);
#endif
            }
         }
      }
   }
#ifdef _DEBUG
   else
   {
      char szTemp[32];
      Str e("Monsdk::NodeGetValue(key=");
      sprintf(szTemp, "0x%08X", key);
      e += szTemp;
      e += ") called with an invalid key!";
      Debug (e);
   }
#endif

   // returning NULL means invalid handle
   return nd;
}


//-----------------------------------------------------------------------------
//  NodePulse( )
//-----------------------------------------------------------------------------
ULONG API_NodePulse(ULONG nKey, ULONG time_since_last)
{
   time_t               currTime = time(NULL);
   time_t               diffTime;
   ULONG                next_time;
   // ULONG             ulData, dataSize;
#if defined(__OS2__) || defined(__NETWARE_386__)
   USHORT               key = HIUSHORT(nKey);
   USHORT               enumKey = LOUSHORT(nKey);
#else
   WORD                 key = HIWORD(nKey);
   WORD                 enumKey = LOWORD(nKey);
#endif


   if ((nKey != INVALID_HANDLE) && (key < gnumKeys))
   {
      diffTime = currTime - (gpKeys[key]->lastTime);
#ifdef _DEBUG
      {
         char szTemp[80];
         sprintf( szTemp, "MONSDK_NodePulse() diffTime=0x%08X", diffTime);
         Debug(szTemp);
      }
#endif

      // get the current values of the query data
      if (diffTime > collectValue)
      {
         // common struct values to set
         gpKeys[key]->lastTime = currTime;
         gpKeys[key]->validData = TRUE;

         // see which enum we want
         switch (enumKey)
         {
            case LEVEL2_DLEAF:
               // nothing else to set
               break;
            case LEVEL2_SLEAF:
               {
                  memset(gpKeys[key]->textStr,0,TEXT_STR_SIZE);
                  strcpy(gpKeys[key]->textStr, ctime( &currTime));
               }
               break;
            case WEEKDAY:
               {
                  tm   *timeInfo = NULL;
                  timeInfo = localtime( &currTime);
                  memset(gpKeys[key]->textStr,0,TEXT_STR_SIZE);
                  sprintf(gpKeys[key]->textStr, "WEEKDAY_%01X", timeInfo->tm_wday);
               }
               break;
            case MONTH:
               {
                  tm   *timeInfo = NULL;
                  timeInfo = localtime( &currTime);
                  memset(gpKeys[key]->textStr,0,TEXT_STR_SIZE);
                  sprintf(gpKeys[key]->textStr, "MONTH_%01X", timeInfo->tm_mon);
               }
               break;
            default:
               ;  // nothing to do
         }
      }
   }
#ifdef _DEBUG
   else
   {
      char szTemp[32] = "";
      Str e("MONSDK_NodePulse - invalid handle or key received, handle=");
      sprintf(szTemp, "0x%08X", nKey);
      e += szTemp;
      e += ", index=";
      sprintf(szTemp, "0x%04X", key);
      e += szTemp;
      e += ", enum=";
      sprintf(szTemp, "0x%04X", enumKey);
      e += szTemp;
      Debug (e);
   }
#endif

   if (time_since_last > 2 * pulseValue)
      next_time = 0;
   else              // Otherwise, schedule one at next appropriate time
      next_time = pulseValue; // + (pulseValue-time_since_last);

   return next_time;
}


ULONG API_GetThresholdCount()
{
   // we only have one default threshold that we want the server to set from
   // our server threshold templates
   //
   // [LEVEL1_NODE][WEEKDAY]
   //
   return 1L;
}


AttributePath MONFAR * API_GetThresholdPath(ULONG index, char MONFAR *lang)
{
   long              aSize = 0;
   long              totalSize = 0;
   AttributePath     *pAPath = NULL;

   if (index == 0)
   {
      //---------------
      // AttributePath - [LEVEL1_NODE][WEEKDAY]
      //---------------
      // determine the size of the path
      aSize = sizeof(AttributePath) + (2 * sizeof(long)); // space for our path offsets

      if (lang == NULL)
      {
         totalSize  = aSize;
         totalSize += (symNodeStr->Length() + 1);
         totalSize += (symWeekdayStr->Length() + 1);
         // create the programmatic attribute path
         pAPath = (AttributePath *) new BYTE[totalSize];
         if (pAPath == NULL)
            return pAPath;
         memset( pAPath, 0, totalSize);
         pAPath->size = totalSize;
         pAPath->version = PATH_VERSION_1;
         strcpy(pAPath->lang, (PCHAR)localJavaLangStr);
         pAPath->members = 2;
         pAPath->offsets[0] = aSize;
         strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[0]), (PCHAR)(*symNodeStr));
         pAPath->offsets[1] = pAPath->offsets[0] + symNodeStr->Length() + 1;
         strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[1]), (PCHAR)(*symWeekdayStr));
      }
      else
      {
         totalSize  = aSize;
         totalSize += (guiNodeStr->Length() + 1);
         totalSize += (guiWeekdayStr->Length() + 1);
         // create the displayable NLS attribute pat
         pAPath = (AttributePath *) new BYTE[totalSize];
         if (pAPath == NULL)
            return pAPath;
         memset( pAPath, 0, totalSize);
         pAPath->size = totalSize;
         pAPath->version = PATH_VERSION_1;
         strcpy(pAPath->lang, (PCHAR)localJavaLangStr);
         pAPath->members = 2;
         pAPath->offsets[0] = aSize;
         strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[0]), (PCHAR)(*guiNodeStr));
         pAPath->offsets[1] = pAPath->offsets[0] + guiNodeStr->Length() + 1;
         strcpy((PCHAR)(((PCHAR)pAPath)+pAPath->offsets[1]), (PCHAR)(*guiWeekdayStr));
      }
   }

   return pAPath;
}


BOOL API_FreeThresholdPath(AttributePath MONFAR *pAPath)
{
   if (pAPath != NULL)
   {
      delete [] (BYTE MONFAR *)pAPath;
      return TRUE;
   }
   return FALSE;
}


