/*
**************************************************************************
**
** Copyright:           (c) IBM Boca Raton
**
** Classification:      IBM Confidential
**
** Description:         Netfinity Netware startup module
**
** Author:              Ed Wachtel
**
** Change History:
**
**
** Environment/Code Cautions:  link NOUNLOAD.OBJ to NetWare modules
**    to provide a warning message to the user attempting to unload
**    a module from the server console.  NOUNLOAD also provides a
**    default memory handler (supported by Watcom) which attempts to
**    unload TWGIPC automatically under memory exhaustion conditions.
**
**************************************************************************
*/

#include <stdio.h>
#include <conio.h>
#include <new.h>
#include <string.h>
#include <stdlib.h>

#undef LONG    // allow Watcom objects to use their defines of LONG and
#undef BYTE    // BYTE and NetFinity objects to use the OS/2 definitions

#include <os2def.h>
#define INCL_DOS
#define INCL_DOSPROCESS
#include <bsedos.h>
#include "nounload.hpp"



/* new handler class definition */

unsigned char *NewHandler::mem = NULL;

NewHandler::NewHandler() : pfnOldHandler(NULL)
{
        // set_new_handler returns old handler

        pfnOldHandler = (PFV)set_new_handler((PFV)new_handler);

        // grab some memory up front as "savings"

        if( mem == NULL )
        {
                mem = new unsigned char[BUFFER_MEM_SIZE];
        }
}


NewHandler::~NewHandler()
{
        if(pfnOldHandler)
        {
                set_new_handler((PFV)pfnOldHandler);
        }
        if( mem != NULL)
        {
                delete mem;
        }
}

void NewHandler::ReleaseMemory()
{
        if( mem != NULL)
        {
                delete mem;
                mem = NULL;
        }
}

void    new_handler(void)
{
        static char *cmd = "unload TWGIPC\r\n";

/* if we have some buffer memory allocated - free it */

        if(NewHandler::mem != NULL)
        {
                delete NewHandler::mem;
                NewHandler::mem = NULL;
        }

/*  print our little message and hope for the best on the way out */

        /************************************************************/
        /* set current screen to system console                     */
        /************************************************************/
        int screenhandle = CreateScreen( "System Console", 0 );
        SetCurrentScreen( screenhandle );

        printf("Netfinity: Memory Allocation Failure Detected. Unloading...\n");
        for(int i=0;i<strlen(cmd);i++)
        {
                ungetch(cmd[i]);
        }

#if 0
        exit(-1);
#endif
	// Garfield put the above exit() call in to prevent the new_handler
	//    from looping when a memory allocation error occurred.  However,
	//    exiting prematurely prevents the thread from signalling any
	//    blocked threads, which prevents TWGIPC from shutting down and
	//    unloading.  Observation indicates that a memory allocation error
	//    seems to assert a continuous error condition; that is, unless
	//    there is some way of resetting this condition, the handler for
	//    these errors will be invoked repeatedly (hence the looping 
	//    behavior).  We're not quite sure if it's possible to clear the
	//    error condition, so we take the other approach and disable the
	//    error handler.
	set_new_handler((PFV)NULL);
	return;
}

/* my new handler */

NewHandler handler;   // statically allocate to set up handlers

// This routine should be referenced using "option check = UnloadCheck"
//    in the link options for the module which includes NOUNLOAD.OBJ.

int UnloadCheck()
{
        ConsolePrintf(NOUNLOAD);  // display informational message
        ConsolePrintf(TOUNLOAD);
        return(-1);   // non-zero return causes NetWare to confirm unload
}


